/***********************************************************************
*                                                                      *
*                              SPECLASS                                *
*                                                                      *
*                                The                                   *
*                                                                      *
*                      Color Spectra Classifier                        *
*                                                                      *
*                          Program  Package                            *
*                                                                      *
*                           Programmed by			       *
*								       *
*			     J. Haanpalo		               *
*                                                                      *
*                      Copyright (c) 1993-1995                         *
*                                                                      *
************************************************************************
*                                                                      *
*  NOTE: This program package is copyrighted in the sense that it      *
*  may be used for scientific purposes. The package as a whole, or     *
*  parts thereof, cannot be included or used in any commercial         *
*  application without written permission granted by its producents.   *
*  No programs contained in this package may be copied for commercial  *
*  distribution. This program package comes 'as it is' and with no     *
*  warranty.                                                           *
*                                                                      *
*  All comments concerning this program package may be sent to the     *
*  e-mail address 'haanpalo@lut.fi'.                                   *
*                                                                      *
***********************************************************************/

#include<stdio.h>
#include<sys/file.h>
#include<string.h>
#include<signal.h>
#include<math.h>
#include"mmatrix.h"

#define NUL "\0"
#define MAT             ".mat"	/* extension for Matlab files   	 */
#define NAME_LEN	100	/* default length for all filenames	 */

/*#define DEBUG /* */

#ifndef BADSIG
#define BADSIG SIG_ERR		/* SysV compatibility */
#endif

/* functions for signal handling	*/
void sighandle(), HUPhandle(), test_signal();

/* functions for reading and writing	*/
int read_input(), write_output();

MMatrix *P,			/* pattern matrix	 */
 *T,				/* target matrix	 */
 *W1,				/* ART weight matrix	 */
 *W2,				/* ART weight matrix	 */
 *M;				/* resonance logmatrix	 */

/* internal layers	*/
MMatrix *a, *b, *c, *d, *e, *f, *g, *B, *Z;

FILE *mfp;			/* file pointer	 */

double vig,			/* vigialence factor	 */
  alpha,			/* constant 	 */
  beta,				/* constant 	 */
  gam,				/* constant 	 */
  delta,			/* constant 	 */
  zeta,				/* constant 	 */
  theta,			/* constant 	 */
  weight;			/* initial value for weight matrices 	 */

int output,			/* if true then write weight matrices and
				 * parameters into file	 */
  do_test;			/* if true then do some testing (signal
				 * handling)		 */

char ofile[NAME_LEN],		/* output file name	 */
  ifile[NAME_LEN],		/* input file name	 */
  logfile[NAME_LEN];		/* log file name	 */

usage(s)
char *s;
{
  fprintf(stderr, "Error: %s\n", s);
  fprintf(stderr, "USAGE: -p <pattern file> [-t <target file>] [-i <infile>] [-o <outfile>]\n");
  fprintf(stderr, "       [-w <initial weight>] [-a <alpha>] [-b <beta>] [-g <gamma>] [-d <delta>]\n");
  fprintf(stderr, "       [-z <zeta> [-h <theta>] [-v <vigialence factor>] [-f] [-e <epoch>]\n");
  fprintf(stderr, "       [-n <layer 2 neurons>]\n\n");
  fprintf(stderr, "    Analog Adaptive Resonance Theory (ART2). Two layer, nearest-neighbor\n");
  fprintf(stderr, "    classifier that stores an arbitrary number of analog spatial patterns.\n");
  fprintf(stderr, "    All input and output files are in Matlab (4.0) format. Pattern file must\n");
  fprintf(stderr, "    be in column order e.g., 100 samples in 61 dimensional space is stored\n");
  fprintf(stderr, "    as a 61x100 matrix.\n\n");
  fprintf(stderr, "PARAMETERS\n");
  fprintf(stderr, "    -a <alpha>	(0<alpha<1)	[0.05]\n");
  fprintf(stderr, "    -b <beta>	(0<beta)	[1.5]\n");
  fprintf(stderr, "    -g <gam>  	(0<gam)		[2.0]\n");
  fprintf(stderr, "    -d <delta>	(0<delta<1)	[0.12]\n");
  fprintf(stderr, "    -z <zeta>   (0<zeta<1)	[0.002]\n");
  fprintf(stderr, "    -h <theta>	(0<theta)	[0.003]\n");
  fprintf(stderr, "    -v <vig>	(0<vig<1)	[0.9]\n");
  fprintf(stderr, "    -w <initial weight>		[0.12]\n");
  fprintf(stderr, "    -f turns fast learning on\n\n");
  exit(1);
}

/* TEST_ART(PAT,TAR,VERBOSE)                                    */
/* TESTS TEST PATTERN VECTORS AND OUTPUTS RESULTS IN A          */
/* CONFUSION MATRIX                                             */
/* PARAMETERS:                                                  */
/*              PAT             PATTERN MATRIX                  */
/*              TAR             TARGET MATRIX                   */
/*              VERBOSE         IF 1 THEN PRINT CONFUSION MATRIX*/
/* RETURNS:                                                     */
/*              DOUBLE VALUE (%)                                */
double test_art(pat, tar, verbose)
MMatrix *pat, *tar;
int verbose;
{
  MMatrix *a, *b, *c, *d, *e, *g;
  int correct, class, **conf, q, dim, Nclasses;
  register i, j;
  double tmp = 0.0, scalar = 0.0, pros = 0.0;

  if (tar == NULL)
    errmsg("No target matrix...testing not possible");

  dim = MxGetM(pat);
  Nclasses = MxGetM(tar);

  /* INITIALIZE CONFUSION MATRIX  */
  conf = init_confusion_matrix(Nclasses);

  /* INITIALIZE LAYER 1 INTERNAL VARIABLES */
  a = Mmatinit(dim, 1, 0.0, "a", NULL);
  b = Mmatinit(dim, 1, 0.0, "b", NULL);
  c = Mmatinit(dim, 1, 0.0, "c", NULL);
  d = Mmatinit(dim, 1, 0.0, "d", NULL);
  e = Mmatinit(dim, 1, 0.0, "e", NULL);
  g = Mmatinit(dim, 1, 0.0, "g", NULL);

  for (q = 0; q < MxGetN(pat); q++) {	/* PRESENT EACH NON-ZERO INPUT VECTOR */

    /* CALCULATE INTERNAL VARIABLES ON LAYER 1 */
    a = Mvecadd(pat, q, Mscalmul(alpha, c, NULL), 0, a);
    b = Mscalmul(1.0 / (gam + Mnorm(a)), a, b);
    d = Mvecadd(Msigmoid2(b, theta, NULL), 0, Mscalmul(beta,
				    Msigmoid2(e, theta, NULL), NULL), 0, d);
    c = Mscalmul(1.0 / (gam + Mnorm(d)), d, c);
    g = Mmatcopy(c, g);

    /* CALCULATE WINNING NEURON IN LAYER 2 */
    class = Mcompet(Mmatmul(Mtransp(W2, NULL), g, NULL), 0);

    /* GET THE CORRECT CLASS FOR PATTERN VECTOR FROM TARGET MATRIX  */
    correct = Mcompet(tar, q);

    /* UPDATE CONFUSION MATRIX      */
    update_confusion_matrix(conf, correct, class, Nclasses);
  }

  /* PRINT CONFUSION MATRIX       */
  pros = print_confusion_matrix(conf, Nclasses);

  MxFreeMatrix(a);
  MxFree(a);
  MxFreeMatrix(b);
  MxFree(b);
  MxFreeMatrix(c);
  MxFree(c);
  MxFreeMatrix(d);
  MxFree(d);
  MxFreeMatrix(e);
  MxFree(e);
  MxFreeMatrix(g);
  MxFree(g);
  return (pros);
}

MMatrix *learn_art2(neuron, lm, ep)
int lm, ep, neuron;
{
  int rows, cols, done, winner, S2, q, reset, epoch = 0, stab, stabilize = 6,
    cycle = 1;
  int win2 = -1;
  double *pr;
  register i, j;
  rows = MxGetM(P);		/* dimension of pattern vector */
  cols = MxGetN(P);		/* number of pattern vectors   */

  /* INITIALIZE WEIGHT MATRICES */

  if (!W2 && !W1) {
    W1 = Mmatinit(rows, neuron, weight, "W1", NULL);	/* layer 1 */
    W2 = Mmatinit(rows, neuron, 0.0, "W2", NULL);	/* layer 2 */
  }
  M = Mmatinit(cols, 2, 0.0, "M", NULL);


  while (epoch++ < ep) {

    printf("Epoch %d\r", epoch);
    fflush(stdout);

    for (q = 0; q < cols; q++) {/* PRESENT EACH NON-ZERO INPUT VECTOR */

      /* INITIALIZE LAYER 1 INTERNAL VARIABLES */
      a = Mmatinit(rows, 1, 0.0, "a", a);
      b = Mmatinit(rows, 1, 0.0, "b", b);
      c = Mmatinit(rows, 1, 0.0, "c", c);
      d = Mmatinit(rows, 1, 0.0, "d", d);
      e = Mmatinit(rows, 1, 0.0, "e", e);
      f = Mmatinit(rows, 1, 0.0, "f", f);
      g = Mmatinit(rows, 1, 0.0, "g", g);
      B = Mmatinit(neuron, 1, 0.0, "B", B);
      Z = Mmatinit(neuron, 1, 0.0, "Z", Z);

  pointti1:

      for (stab = 0; stab < stabilize; stab++) {	/* start of stabilize */

	/* CALCULATE INTERNAL VARIABLES ON LAYER 1 */
	a = Mvecadd(P, q, Mscalmul(alpha, c, NULL), 0, a);
	b = Mscalmul(1.0 / (gam + Mnorm(a)), a, b);
	d = Mvecadd(Msigmoid(b, theta, NULL), 0,
		    Mscalmul(beta, Msigmoid(e, theta, NULL), NULL), 0, d);

	c = Mscalmul(1.0 / (gam + Mnorm(d)), d, c);

	/* RECALCULATE LAYER 1 WITH FEEDBACK FROM LAYER 2 */
	if (win2 == -1)		/* IF LAYER 2 INACTIVE */
	  g = Mmatcopy(c, g);
	else
	 /* IF LAYER 2 ACTIVE */ if (lm == 0)
	  g = Mvecadd(Mcolmul(delta, W2, win2, NULL), 0, c, 0, g);
	else
	  g = Mvecadd(Mcolmul(delta, W1, win2, NULL), 0, c, 0, g);

	e = Mscalmul(1.0 / (gam + Mnorm(g)), g, e);

      }				/* end of stab	 */

#ifdef DEBUG
      Mprintmat(a);
      Mprintmat(b);
      Mprintmat(d);
      Mprintmat(c);
      Mprintmat(g);
      Mprintmat(e);
#endif

  pointti2:

      /* RESET NEURON FIRES IF NEW c TOO DIFFERENT FROM g   */
      f = Mscalmul(1.0 / (gam + Mnorm(c) + Mnorm(Mscalmul(zeta, g, NULL))),
		   Mvecadd(c, 0, Mscalmul(zeta, g, NULL), 0, NULL), f);

#ifdef DEBUG
      Mprintmat(f);
#endif

      reset = hardlim(vig / (gam + Mnorm(f)) - 1.0);
      /*
       * IF RESET NEURON FIRES TAKE WINNING NEURON IN LAYER 2 OUT OF
       * COMPETITION
       */
      if (reset) {
	cycle = 1;
	MxGetPr(B)[0][win2] = -1000.0;
	Mprintmat(B);
	printf("Vector %d resets layer-2 neuron %d\n", q, win2);

	/* IF ALL NEURONS IN LAYER 2 OUT OF COMPETITION ADD NEURON TO LAYER 2 */
	if (Mall(B, -10.0)) {
	  printf("New Layer-2 neuron created.\n");
	  /* W1 = [W1 normc(0.5*rand(S1,1))];  */
	  W1 = Mcolapp(W1, Mmatinit(rows, 1, weight, "tmp", NULL));
	  /* W2 = [W2; normr(0.5*rand(1,S1))]; */
	  W2 = Mcolapp(W2, Mmatinit(rows, 1, 0.0, "tmp", NULL));
	  neuron++;
	  win2 = neuron;
	  B = Mtransp(Mcolapp(Mtransp(B, NULL),
			      Mmatinit(1, 1, 0.0, "tmp", NULL)), B);
	}
	goto pointti1;
      }
      /* IF RESET NEURON DOES NOT FIRE: LAYERS 1 & 2 RESONATE */

      if (cycle == 1) {
	cycle++;

	Z = Mtransp(Mmatmul(Mtransp(g, NULL), W1, NULL), Z);
	Z = Mmatadd(Z, B, Z);

	win2 = Mcompet(Z, 0);

	/* RECALCULATE LAYER 1 WITH FEEDBACK FROM LAYER 2 */

	if (lm == 0)
	  g = Mvecadd(Mcolmul(delta, W2, win2, NULL), 0, c, 0, g);
	else
	  g = Mvecadd(Mcolmul(delta, W1, win2, NULL), 0, c, 0, g);

	e = Mscalmul(1.0 / (gam + Mnorm(g)), g, e);

#ifdef DEBUG
	Mprintmat(c);
	Mprintmat(g);
	Mprintmat(e);
#endif

	goto pointti2;

      }
      if (epoch % 5 == 0)
	printf("Vector %d resonates Layer-2 neuron %d\n", q, win2);

      MxGetPr(M)[0][q] = win2;
      MxGetPr(M)[1][q] = Mcompet(T, q);

      if (lm == 1) {
	W1 = Mmatmerge(W1, Mscalmul(1.0 / (1 - delta), c, NULL), win2);
	W2 = Mmatmerge(W2, Mscalmul(1.0 / (1 - delta), c, NULL), win2);
      } else {
	W1 = Mcoladd(W1, Mscalmul(delta, Mvecsub(g, 0, W1, win2, NULL),
				  NULL), win2, W1);
	W2 = Mcoladd(W2, Mscalmul(delta, Mvecsub(g, 0, W2, win2, NULL),
				  NULL), win2, W2);
      }

#ifdef DEBUG
      Mprintmat(W1);
      Mprintmat(W2);
#endif

      if (do_test)
	test_signal();

      win2 = -1;
      cycle = 1;
    }
    printf("\n");
  }
}

main(argc, argv)
int argc;
char **argv;
{
  int m1, m2, n1, n2, ch, FAST = 1, lm = 0, ep = 50, n = 10, input = 0,
    test = 0;
  double *pr1, *pr2, *pi1, *pi2;

  extern char *optarg;
  extern int optind;
  char patternfile[NAME_LEN], targetfile[NAME_LEN];
  P = T = W2 = W1 = NULL;

  /* SET DEFAULT VALUES FOR PARAMETERS */
  vig = 0.9;
  alpha = 2.0;
  beta = 2.0;
  gam = 0.85;
  delta = 0.000002;
  zeta = 0.55;
  theta = 0.0425;
  weight = 9999;
  output = do_test = 0;

/*	vig = 0.9;
	alpha = 10.0;
	beta = 20.0;
	gam = 0.003;		FAST
	delta = 0.5;
	zeta = 0.5;
	theta = 0.05;
	weight = 9999;
	output = do_test = 0;*/

  while ((ch = getopt(argc, argv, "p:t:o:v:a:b:g:d:z:h:w:n:fe:i:")) != -1) {
    switch (ch) {
    case 'p':
      strcpy(patternfile, optarg);
      if ((mfp = fopen(check_ext(patternfile, MAT), "r")) == NULL)
	errmsg("Cannot open pattern file");
      if ((P = MatGetMatrix(mfp, NULL)) == NULL)
	errmsg("No pattern matrix");
      fclose(mfp);
      input = 1;
      break;
    case 't':
      strcpy(targetfile, optarg);
      if ((mfp = fopen(check_ext(targetfile, MAT), "r")) == NULL)
	errmsg("Cannot open target file");
      if ((T = MatGetMatrix(mfp, NULL)) == NULL)
	errmsg("No target matrix");
      fclose(mfp);
      n = MxGetM(T);
      break;
    case 'v':
      vig = atof(optarg);
      if ((vig < 0) || (vig > 1))
	errmsg("Illegal value for vig");
      break;
    case 'a':
      alpha = atof(optarg);
      if (alpha < 0)
	errmsg("Illegal value for alpha");
      break;
    case 'b':
      beta = atof(optarg);
      if (beta < 0)
	errmsg("Illegal value for beta");
      break;
    case 'g':
      gam = atof(optarg);
      if (gam < 0)
	errmsg("Illegal value for gam");
      break;
    case 'd':
      delta = atof(optarg);
      if ((delta < 0) || (delta > 1))
	errmsg("Illegal value for delta");
      break;
    case 'z':
      zeta = atof(optarg);
      if ((zeta < 0) || (zeta > 1))
	errmsg("Illegal value for zeta");
      break;
    case 'h':
      theta = atof(optarg);
      if (theta < 0)
	errmsg("Illegal value for theta");
      break;
    case 'w':
      weight = atof(optarg);
      break;
    case 'f':
      lm = FAST;
      break;
    case 'e':
      ep = atoi(optarg);
      if (ep < 0)
	errmsg("Illegal value for epoch");
      break;
    case 'n':
      if (T == NULL)
	n = atoi(optarg);
      if (n < 0)
	errmsg("Illegal value for n");
      break;
    case 'i':
      strcpy(ifile, optarg);
      read_input(ifile);
      break;
    case 'o':
      strcpy(ofile, optarg);
      output = 1;
      break;
    }
  }
  if (!input)
    usage("Some parameters are missing...");

  if (signal(SIGINT, sighandle) == BADSIG)
    fprintf(stderr, "Cannot catch interrupt signal\n");
  if (signal(SIGHUP, HUPhandle) == BADSIG)
    fprintf(stderr, "Cannot catch hang-up signal\n");

  if (weight == 9999)
    weight = 1.0 / ((1.0 - delta) * sqrt(MxGetM(P)));

  learn_art2(n, lm, ep);

  /*
   * if(T!=NULL) test_art(P,T,1);
   */

  if (output)
    if (!write_output())
      fprintf(stderr, "Cannot write output file\n");

  MxFreeMatrix(W1);
  MxFree(W1);
  MxFreeMatrix(W2);
  MxFree(W2);
  MxFreeMatrix(a);
  MxFree(a);
  MxFreeMatrix(b);
  MxFree(b);
  MxFreeMatrix(c);
  MxFree(c);
  MxFreeMatrix(d);
  MxFree(d);
  MxFreeMatrix(e);
  MxFree(e);
  MxFreeMatrix(f);
  MxFree(f);
  MxFreeMatrix(g);
  MxFree(g);
  MxFreeMatrix(B);
  MxFree(B);
  MxFreeMatrix(Z);
  MxFree(Z);
  MxFreeMatrix(M);
  MxFree(M);
}

void sighandle()
{
  /* BLOODY SysV drops the signal handler out when the signal comes */
  if (signal(SIGINT, sighandle) == BADSIG) {
    fprintf(stderr, "Cannot catch INT signal any more, next one will kill me!\n");
  }
  if (output)
    if (!write_output())
      fprintf(stderr, "Cannot write output file\n");
  exit(1);
}

void HUPhandle()
{
  /* BLOODY SysV drops the signal handler out when the signal comes */
  if (signal(SIGHUP, SIG_IGN) == BADSIG) {
    fprintf(stderr, "Cannot block hang-up signal\n");
  }
  if (signal(SIGHUP, HUPhandle) == BADSIG) {
    fprintf(stderr, "Cannot recatch hang-up signal\n");
  }
  do_test = 1;
}

void test_signal()
{

  test_art(P, T, 1);
  if (output)
    if (!write_output())
      fprintf(stderr, "Cannot write output file\n");
  do_test = 0;
}

int write_output()
{
  FILE *wfp;
  if ((wfp = fopen(check_ext(ofile, MAT), "w")) == NULL)
    return 0;
  if (MatPutMatrix(wfp, W2))
    return 0;
  if (MatPutMatrix(wfp, W1))
    return 0;
  if (MatPutMatrix(wfp, M))
    return 0;
  if (MatPutMatrix(wfp, Mmatinit(1, 1, alpha, "ALPHA", NULL)))
    return 0;
  if (MatPutMatrix(wfp, Mmatinit(1, 1, beta, "BETA", NULL)))
    return 0;
  if (MatPutMatrix(wfp, Mmatinit(1, 1, gam, "GAMMA", NULL)))
    return 0;
  if (MatPutMatrix(wfp, Mmatinit(1, 1, delta, "DELTA", NULL)))
    return 0;
  if (MatPutMatrix(wfp, Mmatinit(1, 1, zeta, "ZETA", NULL)))
    return 0;
  if (MatPutMatrix(wfp, Mmatinit(1, 1, theta, "THETA", NULL)))
    return 0;
  if (MatPutMatrix(wfp, Mmatinit(1, 1, vig, "VIG", NULL)))
    return 0;
  if (MatPutMatrix(wfp, Mmatinit(1, 1, weight, "WEIGHT", NULL)))
    return 0;
  fclose(wfp);
  return 1;
}

int read_input()
{
  FILE *rfp;
  if ((rfp = fopen(check_ext(ifile, MAT), "r")) == NULL)
    return 0;
  if ((W2 = MatGetMatrix(rfp, "W2")) == NULL)
    return 0;
  if ((W1 = MatGetMatrix(rfp, "W1")) == NULL)
    return 0;
  /*
   * alpha = MxGetScalar(MatGetMatrix(rfp,"ALPHA")); beta =
   * MxGetScalar(MatGetMatrix(rfp,"BETA")); gam =
   * MxGetScalar(MatGetMatrix(rfp,"GAMMA")); delta =
   * MxGetScalar(MatGetMatrix(rfp,"DELTA")); zeta =
   * MxGetScalar(MatGetMatrix(rfp,"ZETA")); theta =
   * MxGetScalar(MatGetMatrix(rfp,"THETA")); vig =
   * MxGetScalar(MatGetMatrix(rfp,"VIG")); weight =
   * MxGetScalar(MatGetMatrix(rfp,"WEIGHT"));
   */
  fclose(rfp);
  return 1;
}
