/***********************************************************************
*                                                                      *
*                              SPECLASS                                *
*                                                                      *
*                                The                                   *
*                                                                      *
*                      Color Spectra Classifier                        *
*                                                                      *
*                          Program  Package                            *
*                                                                      *
*                           Programmed by			       *
*								       *
*			     J. Haanpalo		               *
*                                                                      *
*                      Copyright (c) 1993-1995                         *
*                                                                      *
************************************************************************
*                                                                      *
*  NOTE: This program package is copyrighted in the sense that it      *
*  may be used for scientific purposes. The package as a whole, or     *
*  parts thereof, cannot be included or used in any commercial         *
*  application without written permission granted by its producents.   *
*  No programs contained in this package may be copied for commercial  *
*  distribution. This program package comes 'as it is' and with no     *
*  warranty.                                                           *
*                                                                      *
*  All comments concerning this program package may be sent to the     *
*  e-mail address 'haanpalo@lut.fi'.                                   *
*                                                                      *
***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <math.h>
#include "xglobal.h"
#include "global.h"

#define MAXCOLORS 10

/* DEFAULT VALUES OF THE FORM OF SPECTRUM */
int g_start_wavelen = 400;
int g_end_wavelen = 700;
int g_lkm = 61;

/* GLOBAL XLIB VARIABLES */
Display *dpy;
int screen;
GC gc, gc2;
Pixwin *image_pw;

/* VARIABLES FOR COLOR HANDLING, MUST BE STATIC */
static int ncolors = MAXCOLORS;
static XColor *xcolor;
static XColor xcol;
static Colormap colormap;

unsigned long colors[MAXCOLORS];/* COLORTABLE */
unsigned long foreground_pixel, background_pixel, border_pixel;

int init_colors();
void draw_spek();
void draw_empty_spec();

/* GLOBAL VARIABLES FOR DRAW-FUNCTIONS (SIZES OF COORD.AXES ETC.) */
int Y_axis_used = 1;
int scal = 400;			/* SCALE FACTOR OF COORDINATE, ALSO THE
				 * LENGTH OF COORD AXIS */
int sp_scal = 100;		/* SCALE FACTOR OF SPECTRUM */
int coord_start_x = 40, coord_start_y = 410;	/* LOCATION OF ORIGO */
int spec_start_x = 50, spec_start_y = 260;	/* STARTING POINT OF SPECTRUM */
int spec_max_x = 320, spec_max_y = 240;	/* LENGTH AND HEIGHT OF SPECTRUM AXIS */
int psz = 8;			/* SIZE OF POINT IN COORD. */
int is_set = 0;			/* IS THE FILE NAME A SET? */
int gyplace = 0;
int RED = MAXCOLORS - 1;


void init_canvas()
{
  XGCValues gcvalues;
  int gcvaluemask;

  canvas = xv_create(frame, CANVAS,
		     XV_X, 0,
		     XV_Y, 250,
		     XV_WIDTH, CANVAS_W,
		     XV_HEIGHT, CANVAS_H,
		     XV_VISUAL_CLASS, PseudoColor,
		     CANVAS_X_PAINT_WINDOW, TRUE,
		     NULL);

  image_pw = canvas_pixwin(canvas);

  dpy = (Display *) xv_get(frame, XV_DISPLAY);

  /* SETUP GC */
  gcvalues.function = GXcopy;
  gcvalues.fill_style = FillSolid;
  gcvaluemask = GCFunction | GCFillStyle;

  gc = XCreateGC(dpy, (Window) xv_get(canvas_paint_window(canvas), XV_XID),
		 gcvaluemask, &gcvalues);
  if (gc == NULL) {
    fprintf(stderr, "XCreateGC failed ... exiting... \n");
    exit(1);
  }
  /* ALLOCATE READ/WRITE COLORS */
  if (init_colors() == 0) {
    fprintf(stderr, "wrong visual, couldn't allocate colors\n");
    exit(1);
  }
  /* SET THE LAST XCOLOR TO RED */
  if (!XParseColor(dpy, DefaultColormap(dpy, screen), "red", &xcolor[MAXCOLORS - 1]))
    fprintf(stderr, "XParse failed\n");

  if (!XParseColor(dpy, DefaultColormap(dpy, screen), "green", &xcolor[MAXCOLORS - 2]))
    fprintf(stderr, "XParse failed\n");

  XStoreColor(dpy, DefaultColormap(dpy, screen), &xcolor[MAXCOLORS - 1]);

  XStoreColor(dpy, DefaultColormap(dpy, screen), &xcolor[MAXCOLORS - 2]);

  XSetForeground(dpy, gc, colors[MAXCOLORS - 1]);

  XSetBackground(dpy, gc, WhitePixel(dpy, screen));

  draw_empty_spec();
}




/*      DRAW_EMPTY_SPEC()
        PURPOSE:
DRAWS AXIS AND SOME VALUES OF SCALE OF SPECTRUM, DRAWS ALSO THE LINES */
void draw_empty_spec()
{
  Window drawind;
  int i, x_start, y_start, y_max, x_max, step, lambda;
  int x_coef;
  char numbuf[5];

  drawind = (Window) xv_get(canvas_paint_window(canvas), XV_XID);

  XSetForeground(dpy, gc, BlackPixel(dpy, screen));

  x_start = spec_start_x;
  y_start = spec_start_y;
  y_max = spec_max_y;
  x_max = spec_max_x;

  /* CLEAR AREA FOR WRITING VALUES */
  XClearArea(dpy, drawind, 0, 0, CANVAS_W, CANVAS_H, False);

  XDrawRectangle(dpy, drawind, gc, CANVAS_W - 340, 10, 320, 275);

  /* COORD. AXIS */
  XDrawLine(dpy, drawind, gc, x_start, y_start, x_start + x_max, y_start);
  XDrawLine(dpy, drawind, gc, x_start, y_start, x_start, y_start - y_max);

  /* SCALE */
  for (i = 1; i < 11; i++) {
    XDrawLine(dpy, drawind, gc, x_start - 3,
       y_start - (i * y_max / 10), x_start + 3, y_start - (i * y_max / 10));
    sprintf(numbuf, "%d", i * 10);
    XDrawString(dpy, drawind, gc, x_start - 25,
		y_start - (i * y_max / 10) + 4, numbuf, strlen(numbuf));
  }

  x_coef = (int) (x_max / g_lkm);
  if (x_coef == 0)
    fprintf(stderr, "spectrum size too large\n");

  step = (g_end_wavelen - g_start_wavelen) / (g_lkm - 1);

  for (i = 0, lambda = g_start_wavelen; i <= g_lkm; i++, lambda += step)
    if ((lambda % 50) == 0) {
      XDrawLine(dpy, drawind, gc, x_start + (i * x_coef), y_start - 3,
		x_start + (i * x_coef), y_start + 3);
      sprintf(numbuf, "%d", lambda);
      XDrawString(dpy, drawind, gc, x_start + (i * x_coef) - 5,
		  y_start + 15, numbuf, strlen(numbuf));
    }
  /* WRITE UNITS */
  XDrawString(dpy, drawind, gc, x_start - 40, y_start - y_max + 5, "%", strlen("%"));
  XDrawString(dpy, drawind, gc, x_start + x_max - 25, y_start + 30, "nm", strlen("nm"));
}

void draw_spek(spec, type)
Spec *spec;
int type;
{
  Window drawind;
  int i, x_start, y_start, y_scale, x_scale, x_coef, step, lambda;
  int spek_max = 4096;
  char textbuf[100];

  drawind = (Window) xv_get(canvas_paint_window(canvas), XV_XID);

  x_start = spec_start_x;
  y_start = spec_start_y;
  x_scale = spec_max_x;
  y_scale = spec_max_y;

  x_coef = (int) (x_scale / spec->dim);

  switch (type) {
  case 0:			/* WHITE */
    XSetForeground(dpy, gc, WhitePixel(dpy, screen));
    break;
  case 1:			/* BLACK */
    XSetForeground(dpy, gc, BlackPixel(dpy, screen));
    break;
  case 2:			/* RED */
    XSetForeground(dpy, gc, colors[MAXCOLORS - 1]);
    break;
  case 3:			/* ??? */
    XSetForeground(dpy, gc, colors[MAXCOLORS - 2]);
    break;
  }

  for (i = 0; i < spec->dim - 1; i++) {
    XDrawLine(dpy, drawind, gc, x_start + (x_coef * i),
	      y_start - (int) ((spec->data[i]) * y_scale),
	      x_start + (x_coef * i) + x_coef, y_start - (int) ((spec->data[i + 1]) * y_scale));
  }

  XSetForeground(dpy, gc, BlackPixel(dpy, screen));

  /* CLEAR AREA FOR WRITING VALUES */
  XClearArea(dpy, drawind, CANVAS_W - 320, 15, 300, 240, False);

  sprintf(textbuf, "File        : %s", spec->file);
  XDrawString(dpy, drawind, gc, CANVAS_W - 320, 40, textbuf, strlen(textbuf));

  sprintf(textbuf, "Label       : %s", spec->label);
  XDrawString(dpy, drawind, gc, CANVAS_W - 320, 60, textbuf, strlen(textbuf));

  sprintf(textbuf, "Dimension   : %d", spec->dim);
  XDrawString(dpy, drawind, gc, CANVAS_W - 320, 80, textbuf, strlen(textbuf));

  sprintf(textbuf, "Class       : %d", spec->class);
  XDrawString(dpy, drawind, gc, CANVAS_W - 320, 100, textbuf, strlen(textbuf));

  sprintf(textbuf, "Classified  : %d", spec->classed);
  XDrawString(dpy, drawind, gc, CANVAS_W - 320, 120, textbuf, strlen(textbuf));

  /* A SPECTRUM HAS BEEN REMOVED -> REDRAW COORDINATES	 */
  if (!type)
    draw_empty_spec();
}

init_colors()
{
  Visual *default_visual;
  XVisualInfo visual_info;
  int default_depth, i;
  /* XColor exact_defs[MAXCOLORS]; */
  unsigned long plane_masks[1];

  default_visual = DefaultVisual(dpy, screen);
  default_depth = (int) DefaultDepth(dpy, screen);
  colormap = (Colormap) DefaultColormap(dpy, screen);

  /* CHECK VISUAL (MUST BE AT LEAST PSEUDOCOLOR) */
  if (!XMatchVisualInfo(dpy, screen, default_depth, PseudoColor,
			&visual_info)) {
    if (!XMatchVisualInfo(dpy, screen, default_depth, DirectColor,
			  &visual_info)) {
      /*
       * PROPER VISUAL WAS NOT FOUND, PROGRAM DOESN'T SHOW COLORS CORRECTLY
       */
      border_pixel = BlackPixel(dpy, screen);
      background_pixel = WhitePixel(dpy, screen);
      foreground_pixel = BlackPixel(dpy, screen);
      return (0);
    }
  }
  /* VISUAL IS OK */
  ncolors = MAXCOLORS;

  /* MALLOC SPACE FOR OUR COLORS */
  xcolor = (XColor *) malloc((unsigned) ncolors * sizeof(XColor));

  if (XAllocColorCells(dpy, colormap, False, plane_masks, 0,
		       colors, ncolors) == 0)
    return 0;

  for (i = 0; i < ncolors; i++) {
    xcolor[i].flags = DoRed | DoGreen | DoBlue;
    xcolor[i].pixel = colors[i];
    xcolor[i].red = i * 4000;
    xcolor[i].green = i * 1000;
    xcolor[i].blue = i * 3000;
    XStoreColor(dpy, DefaultColormap(dpy, screen), &xcolor[i]);
  }

}
