/***********************************************************************
*                                                                      *
*                              SPECLASS                                *
*                                                                      *
*                                The                                   *
*                                                                      *
*                      Color Spectra Classifier                        *
*                                                                      *
*                          Program  Package                            *
*                                                                      *
*                           Programmed by			       *
*								       *
*			     J. Haanpalo		               *
*                                                                      *
*                      Copyright (c) 1993-1995                         *
*                                                                      *
************************************************************************
*                                                                      *
*  NOTE: This program package is copyrighted in the sense that it      *
*  may be used for scientific purposes. The package as a whole, or     *
*  parts thereof, cannot be included or used in any commercial         *
*  application without written permission granted by its producents.   *
*  No programs contained in this package may be copied for commercial  *
*  distribution. This program package comes 'as it is' and with no     *
*  warranty.                                                           *
*                                                                      *
*  All comments concerning this program package may be sent to the     *
*  e-mail address 'haanpalo@lut.fi'.                                   *
*                                                                      *
***********************************************************************/

#include "mmatrix.h"
#include <sys/time.h>
#include <sys/resource.h>

/* Do not change this */
#define VERSION		"0.81"

#define MAXCLASS 27
#define SPECMAX 4092
#define SEPARATOR  " "
#ifndef MIN
#define MIN(a, b)  ((a < b) ? (a) : (b))
#endif
#define NUL "\0"

#define PANEL_X		300
#define PANEL_Y		0
#define PANEL_WIDTH	440
#define PANEL_HEIGHT	250

#define MUNSELL_MAX	4096	/* maximum value for munsell a munsell spectrum*/

/* file name extensions	*/
#define MAT		".mat"		/* extension for Matlab files	*/
#define DAT		".dat"		/* extension for ASCII files	*/
#define COD		".cod"		/* LVQ codebook file		*/

#define NAME_LEN	200		/* default length for filenames	*/
#define LINE_LEN	3000		/* default length for ascii line*/

/* data format types */
#define RAW_UCHAR	0		/* raw unsigned char		*/
#define RAW_SHORT	1		/* raw short integer		*/
#define MUNSELL_DATA	2		/* munsell ascii file		*/
#define NATURAL_DATA	3		/* natural ascii file		*/
#define MINOLTA_DATA	4		/* MINOLTA ascii file		*/
#define MATLAB_DATA	5		/* matlab matrix file		*/

/* type of preprosessing of spectrum */
#define SCALED		0		/* divided by maxval		*/
#define NORMALIZED	1		/* normalized to unity		*/
#define RAW		2		/* raw measurement data		*/

/* global Matlab variables */
MMatrix  *pattern_matrix,	/* pattern vectors in column order	*/
	 *target_matrix; 	/* number of rows = number of classes	*/
				/* number of colums = number of pattern	*/
				/* vectors				*/

/* other global variables */
char	*cntable;		/* classes and labels of pattern vectors*/
char	*class_table;		/* names of classes			*/ 
char	*spec_dir;		/* current data directory		*/ 
int 	cnum;			/* number of classes			*/
int 	list1_total;		/* number of spectra in first list	*/
int 	list2_total;		/* number of spectra in second list	*/
int 	highlight;		/* number of highlighted spectra 	*/


typedef struct spectrum {	/* structure for storing spectrum	*/
	int class;		/* correct class for a spectrum		*/
	int classed;		/* classification result		*/
	int dim;		/* dimension of spectrum		*/
	int type;		/* type of spectrum (scaled, normalized)*/
	char *file;		/* file (page) name			*/
	char *label;		/* label of spectrum			*/
	double *data;		/* spectrum vector			*/
	struct spectrum *next;	/* pointer to next spectrum in list	*/
} Spec;

Spec	*start,		/* pointer to first spectrum in linked list	*/
	*lstart;	/* pointer to first spectrum in loaded list	*/

int 	visible;	/* spectra set which is currently shown		*/

int	timem;		/* global time buffer				*/ 
