/***********************************************************************
*                                                                      *
*                              SPECLASS                                *
*                                                                      *
*                                The                                   *
*                                                                      *
*                      Color Spectra Classifier                        *
*                                                                      *
*                          Program  Package                            *
*                                                                      *
*                           Programmed by			       *
*								       *
*			     J. Haanpalo		               *
*                                                                      *
*                      Copyright (c) 1993-1995                         *
*                                                                      *
************************************************************************
*                                                                      *
*  NOTE: This program package is copyrighted in the sense that it      *
*  may be used for scientific purposes. The package as a whole, or     *
*  parts thereof, cannot be included or used in any commercial         *
*  application without written permission granted by its producents.   *
*  No programs contained in this package may be copied for commercial  *
*  distribution. This program package comes 'as it is' and with no     *
*  warranty.                                                           *
*                                                                      *
*  All comments concerning this program package may be sent to the     *
*  e-mail address 'haanpalo@lut.fi'.                                   *
*                                                                      *
***********************************************************************/

#include <string.h>
#include <stdio.h>
#include <sys/wait.h>
#include "xglobal.h"
#include "global.h"

void write_help();

/* HELP WINDOW INITIALIZATION	*/
void init_help()
{

  /* SET UP HELP FRAME */
  help_frame = xv_create(frame, FRAME,
			 XV_LABEL, "Speclass help",
			 NULL);

  xv_set(help_frame, XV_SHOW, FALSE, NULL);
  xv_set(help_frame, FRAME_SHOW_RESIZE_CORNER, FALSE, NULL);

  /* SETUP THE SIZE OF HELP FRAME */
  rect.r_top = rect.r_left = 20;
  rect.r_width = HFRAME_W;
  rect.r_height = HFRAME_H;
  frame_set_rect(help_frame, &rect);

  /* CREATE TEXTSW AND SET THE COLORMAP SEGMENT FOR IT */
  helpsw1 = xv_create(help_frame, TEXTSW, TEXTSW_READ_ONLY, TRUE,
	 TEXTSW_FIRST_LINE, 1, NULL);

  write_help();
}

void write_help()
{
  char buf[2000];

  /* THE INDEX OF HELP */ 
  sprintf(buf, "Speclass Color Spectrum Classifier\n\n");
  strcat(buf, "Index:\n\n");
  strcat(buf, "1. Interface\n\n");
  strcat(buf, "2. Spectrum Editor\n\n");
  strcat(buf, "3. Selecting classifier\n");
  strcat(buf, "  3.1. ART\n  3.2. KNN\n  3.3. LSM\n  3.4. SOM\n\n");
  strcat(buf, "4. Results\n\n\n\n");
  textsw_insert(helpsw1, buf, strlen(buf));

  /* ABOUT THE INTERFACE */
  sprintf(buf, "1. Interface\n\n");
  strcat(buf, "The Speclass interface provides an easy way to contstruct\n");
  strcat(buf, "learning and test sets from various color spectra sets to be\n");
  strcat(buf, "used with neural and traditional classifiers. The interface\n");
  strcat(buf, "consists of two parts from which the Spectrum Editor is\n");
  strcat(buf, "used for handling color spectra sets and the Speclass that\n");
  strcat(buf, "is used for classification and viewing the results.\n\n");
  textsw_insert(helpsw1, buf, strlen(buf));

  sprintf(buf, "2. Spectrum Editor\n\n");
  strcat(buf, "The Spectrum Editor can be started by pressing the \"Editor\" button\n");
  strcat(buf, "of the main Speclass window which opens new window for the editor.\n\n");
  strcat(buf, "The storage format of the spectra to be loaded can be selected\n");
  strcat(buf, "from \"Load Format\" menu. If the AOTF measured Munsell and Natural\n");
  strcat(buf, "spectra are properly installed the spectra can be selected from\n");
  strcat(buf, "the \"Spectra\" menu if MUNSELL or NATURAL is selected.\n");
  strcat(buf, "In other case \"Pattern filename\" and \"Target filename\" (optional) \n");
  strcat(buf, "and the number of \"Channels\" and the \"Max value\" for the \n");
  strcat(buf, "intensity must be given. The \"Max value\" parameter is used for low\n");
  strcat(buf, "pass filtering of spectra.\n\n");
  strcat(buf, "Color spectra can be saved by giving \"Save pattern filename\" and\n");
  strcat(buf, "\"Save target filename\" and pressing the \"Save\" button. The save\n");
  strcat(buf, "format can be selected from the \"Save format\" menu (Matlab recommended).\n");
  strcat(buf, "It is also possible to scale or normalize spectra which are going to\n");
  strcat(buf, "be saved by selecting an option from \"Data type\" menu. Current\n");
  strcat(buf, "options are \"Scale\" which divides each component of spectrum by\n");
  strcat(buf, "the \"Max value\", \"Normalized\" which normalizes the spectrum to\n");
  strcat(buf, "unity and \"Raw\" which does not change spectrum in any way.\n\n");
  strcat(buf, "When a spectrum is loaded it is also displayed on the screen with\n");
  strcat(buf, "some information: filename, label, dimension, target class and\n");
  strcat(buf, "classification result which is -1 if spectra are not classified.\n");
  strcat(buf, "The target class for a spectrum is obtained from a target file or\n");
  strcat(buf, "if no target file exists then the target class value is set the same\n");
  strcat(buf, "as the current \"Toggle class\" value. The \"Toggle class\" value\n");
  strcat(buf, "can be increased or decreased by pressing either \"Inc\" or \"Dec\".\n");
  strcat(buf, "The maximum \"Toggle class\" value is determined by the \"Max class\"\n");
  strcat(buf, "parameter.\n\n");
  textsw_insert(helpsw1, buf, strlen(buf));

  sprintf(buf, "Any spectrum can be highlighted by using the \"Toggle spectrum\" buttons\n");
  strcat(buf, "\"<<\" and \">>\". The target class of a highlighted spectrum can be changed\n");
  strcat(buf, "by choosing a \"Toggle class\" value and by pressing the \"Set current\n");
  strcat(buf, "class\" button. The highlighted spectrum can be also removed from the set\n");
  strcat(buf, "by pressing \"Remove spectrum\" button. The whole set can be removed from\n");
  strcat(buf, "memory by pressing the \"Clear selections\" button.\n\n");
  textsw_insert(helpsw1, buf, strlen(buf));

  sprintf(buf, "The current spectra set can be viewed as the whole or one class at a\n");
  strcat(buf, "time by selecting either \"View all\" or \"View class\". The class to\n");
  strcat(buf, "be viewed is determined by the \"Toggle class\" value.\n\n");
  strcat(buf, "Note! Newly created learning and test sets must be saved before they\n");
  strcat(buf, "can be used by classifiers.\n\n\n");
  textsw_insert(helpsw1, buf, strlen(buf));

  /* Selecting classifier and setting parameters */
  sprintf(buf, "3. Selecting classifier\n\n");
  strcat(buf, "Currently there are 4 classifiers installed and they can be selected\n");
  strcat(buf, "from the \"Classifier\" menu of the \"Speclass\" window. Selecting one of\n");
  strcat(buf, "them opens the parameter panel of the classifier. All the classifiers\n");
  strcat(buf, "need \"Pattern file\", \"Target file\" and \"Output file\" as their\n");
  strcat(buf, "parameters for the learning phase. For testing the classifiers usually\n");
  strcat(buf, "need \"Test pattern file\", \"Test target file\" and \"Input file\". The\n");
  strcat(buf, "\"Input file\" consists of the weights created by the current classifier\n");
  strcat(buf, "after learning phase.\n\n");
  strcat(buf, "Each classifier can also be run without this interface by giving\n");
  strcat(buf, "parameters directly from the command line. Try -h or -? options for more\n");
  strcat(buf, "information about the classifiers.\n");
  textsw_insert(helpsw1, buf, strlen(buf));

  /* Results */
  sprintf(buf, "4. Results\n\n");
  strcat(buf, "The classification results are displayed in a confusion matrix at the\n");
  strcat(buf, "bottom half of the window at the end of test phase. Classified spectra\n");
  strcat(buf, "can be viewed in a similar manner as was described in chapter 2 by\n");
  strcat(buf, "pressing \"View all\" etc. The misclassified spectra are highlighted\n");
  strcat(buf, "with red color.\n");
  textsw_insert(helpsw1, buf, strlen(buf));
}
