/***********************************************************************
*                                                                      *
*                              SPECLASS                                *
*                                                                      *
*                                The                                   *
*                                                                      *
*                      Color Spectra Classifier                        *
*                                                                      *
*                          Program  Package                            *
*                                                                      *
*                           Programmed by			       *
*								       *
*			     J. Haanpalo		               *
*                                                                      *
*                      Copyright (c) 1993-1995                         *
*                                                                      *
************************************************************************
*                                                                      *
*  NOTE: This program package is copyrighted in the sense that it      *
*  may be used for scientific purposes. The package as a whole, or     *
*  parts thereof, cannot be included or used in any commercial         *
*  application without written permission granted by its producents.   *
*  No programs contained in this package may be copied for commercial  *
*  distribution. This program package comes 'as it is' and with no     *
*  warranty.                                                           *
*                                                                      *
*  All comments concerning this program package may be sent to the     *
*  e-mail address 'haanpalo@lut.fi'.                                   *
*                                                                      *
***********************************************************************/

#include<stdio.h>
#include<sys/file.h>
#include<string.h>
#include<signal.h>
#include<math.h>
#include"mmatrix.h"

/*#define DEBUG /* */
#define NUL "\0"
#define MAT             ".mat"	/* extension for Matlab files   */
#define NAME_LEN	100	/* default length for all filenames	 */


/* functions for reading and writing	*/
int read_input(), write_output();

MMatrix *p,			/* pattern matrix	 */
 *t,				/* target matrix	 */
 *P,				/* test pattern matrix	 */
 *T;				/* test target matrix	 */


FILE *mfp;			/* file pointer	 */

char ofile[NAME_LEN],		/* output file name	 */
  ifile[NAME_LEN],		/* input file name	 */
  logfile[NAME_LEN];		/* log file name	 */

usage(s)
char *s;
{
  fprintf(stderr, "Error: %s\n", s);
  fprintf(stderr, "USAGE: -p <pattern file> -t <target file> [-P <test pattern file>]\n");
  fprintf(stderr, "      [-o <output file>][-n <neighborhood size>][-d <dump file>]\n\n");
  fprintf(stderr, "    KNN-classifier\n");
  fprintf(stderr, "    Input matrices must be in MATLAB format and pattern vectors stored\n");
  fprintf(stderr, "    in column order. Patterns in test pattern file are matched against\n");
  fprintf(stderr, "    patterns in pattern file. If no test pattern file is given then pattern\n");
  fprintf(stderr, "    file is used as test pattern file and best matching unit is not used\n");
  fprintf(stderr, "    when determining class. Classification results are stored in output file\n");
  fprintf(stderr, "    which is a MATLAB matrix named KNN. Class distribution of neighbors\n");
  fprintf(stderr, "    for each test pattern vector can be obtained from a dump file which\n");
  fprintf(stderr, "    is a MATLAB matrix named CLASSTABLE.\n");
  fprintf(stderr, "    \n");
  exit(1);
}


MMatrix *class_knn(nbor, fname, teach, dump, dfname)
int nbor, teach, dump;
char *fname, *dfname;
{
  MMatrix *dist, *sorted, *classtable, *results;
  int class, rounds;
  register i, j;
  dist = sorted = classtable = results = NULL;

  dist = Mmatinit(MxGetN(p), 1, 0.0, "dist", NULL);
  sorted = Mmatinit(MxGetN(p), 1, 0.0, "sorted", NULL);
  classtable = Mmatinit(MxGetM(t), MxGetN(P), 0.0, "CLASSTABLE", NULL);
  results = Mmatinit(MxGetN(P), 1, 0.0, "KNN", NULL);
  rounds = MxGetN(P);

  printf("Classifying...\n");

  for (i = 0; i < rounds; i++) {
    dist = Mdistances(p, MxGetPr(P)[i], dist);
    sorted = Msort(dist, sorted);
    for (j = teach; j < (nbor + teach); j++)
      MxGetPr(classtable)[i][Mcompet(t, (int) MxGetPr(sorted)[0][j])] += 1;

    MxGetPr(results)[0][i] = (double) Mcompet(classtable, i);
    printf("Pattern vector %d/%d\r", i, rounds);
    fflush(stdout);
  }

  printf("\n");
  if (!save_matrix(fname, results))
    fprintf(stderr, "Note! Results were not saved\n");

  if (dump)
    if (!save_matrix(dfname, classtable))
      fprintf(stderr, "Note! Dump file was not saved\n");

  MxFreeMatrix(dist);
  MxFree(dist);
  MxFreeMatrix(sorted);
  MxFree(sorted);
  MxFreeMatrix(classtable);
  MxFree(classtable);
  MxFreeMatrix(results);
  MxFree(results);
  MxFreeMatrix(p);
  MxFree(p);
  MxFreeMatrix(t);
  MxFree(t);
  MxFreeMatrix(P);
  MxFree(P);
}

main(argc, argv)
int argc;
char **argv;
{
  int nbor = 5, pn, tn, tpn, ttn, ook, ch, teach = 0, dump = 0;

  extern char *optarg;
  extern int optind;
  char pfile[NAME_LEN], tfile[NAME_LEN], tpfile[NAME_LEN], ttfile[NAME_LEN],
    ofile[NAME_LEN], dfile[NAME_LEN];
  p = t = P = T = NULL;
  pn = tn = tpn = ttn = ook = 0;

  while ((ch = getopt(argc, argv, "p:t:P:n:o:d:")) != -1) {
    switch (ch) {
    case 'p':
      strcpy(pfile, optarg);
      if ((mfp = fopen(check_ext(pfile, MAT), "r")) == NULL)
	usage("Cannot open pattern file");
      if ((p = MatGetMatrix(mfp, NULL)) == NULL)
	usage("No pattern matrix");
      fclose(mfp);
      pn = 1;
      break;
    case 't':
      strcpy(tfile, optarg);
      if ((mfp = fopen(check_ext(tfile, MAT), "r")) == NULL)
	usage("Cannot open target file");
      if ((t = MatGetMatrix(mfp, NULL)) == NULL)
	usage("No target matrix");
      fclose(mfp);
      tn = 1;
      break;
    case 'P':
      strcpy(tpfile, optarg);
      if ((mfp = fopen(check_ext(tpfile, MAT), "r")) == NULL)
	usage("Cannot open test pattern file");
      if ((P = MatGetMatrix(mfp, NULL)) == NULL)
	usage("No pattern matrix");
      fclose(mfp);
      tpn = 1;
      break;
    case 'n':
      nbor = atoi(optarg);
      if (nbor <= 0)
	usage("Illegal value for nbor");

      if (nbor % 2 == 0)
	fprintf(stderr, "knnclass: Note! Neighborhood size is even number\n");

      break;
    case 'o':
      strcpy(ofile, optarg);
      ook = 1;
      break;
    case 'd':
      strcpy(dfile, optarg);
      dump = 1;
      break;
    }
  }
  if (!(pn && tn))
    usage("knnclass: No pattern and/or target files");

  if (!ook)
    usage("knnclass: No output filename");

  if (nbor > MxGetN(p)) {
    fprintf(stderr, "Too large neighborhood. Using %d\n", MxGetN(p));
    nbor = MxGetN(p);
  }
  if (MxGetN(p) != MxGetN(t))
    usage("knnclass: Pattern and target matrix dimensions don't match");

  if (!tpn) {
    teach = 1;
    P = Mmatcopy(p, P);
  } else if (MxGetM(p) != MxGetM(P))
    usage("knnclass: Pattern vector length is not the same as test pattern vector's");

  class_knn(nbor, ofile, teach, dump, dfile);
}
