/***********************************************************************
*                                                                      *
*                              SPECLASS                                *
*                                                                      *
*                                The                                   *
*                                                                      *
*                      Color Spectra Classifier                        *
*                                                                      *
*                          Program  Package                            *
*                                                                      *
*                           Programmed by			       *
*								       *
*			     J. Haanpalo		               *
*                                                                      *
*                      Copyright (c) 1993-1995                         *
*                                                                      *
************************************************************************
*                                                                      *
*  NOTE: This program package is copyrighted in the sense that it      *
*  may be used for scientific purposes. The package as a whole, or     *
*  parts thereof, cannot be included or used in any commercial         *
*  application without written permission granted by its producents.   *
*  No programs contained in this package may be copied for commercial  *
*  distribution. This program package comes 'as it is' and with no     *
*  warranty.                                                           *
*                                                                      *
*  All comments concerning this program package may be sent to the     *
*  e-mail address 'haanpalo@lut.fi'.                                   *
*                                                                      *
***********************************************************************/

#include <math.h>
#include "xglobal.h"
#include "global.h"
#include "matfile.h"

Panel_item ktpattern_item, kttarget_item, ktest_item, knbor_item, koutput_item;

void knn_test();

void init_knn_panel()
{
  if (knn_panel) {
    xv_set(current_panel, XV_SHOW, FALSE, NULL);
    xv_set(knn_panel, XV_SHOW, TRUE, NULL);
    current_panel = knn_panel;
    return;
  }
  current_panel = knn_panel = xv_create(frame, PANEL,
					XV_X, PANEL_X,
					XV_Y, PANEL_Y,
					XV_WIDTH, PANEL_WIDTH,
					XV_HEIGHT, PANEL_HEIGHT,
					NULL);


  knbor_item = xv_create(knn_panel, PANEL_TEXT,
			 XV_X, 0,
			 XV_Y, 0,
			 PANEL_LAYOUT, PANEL_VERTICAL,
			 PANEL_LABEL_STRING, "neighbors",
			 PANEL_VALUE_DISPLAY_LENGTH, DISPLAY_LENGTH,
			 PANEL_VALUE_STORED_LENGTH, DISPLAY_LENGTH,
			 PANEL_VALUE, "3",
			 NULL);

  ktest_item = xv_create(knn_panel, PANEL_BUTTON,
			 XV_X, 100,
			 XV_Y, 20,
			 PANEL_LABEL_STRING, " test ",
			 PANEL_NOTIFY_PROC, knn_test,
			 NULL);

  ktpattern_item = xv_create(knn_panel, PANEL_TEXT,
			     XV_X, 175,
			     XV_Y, 155,
			     PANEL_LAYOUT, PANEL_VERTICAL,
			     PANEL_LABEL_STRING, "Test pattern file:",
			     PANEL_VALUE_DISPLAY_LENGTH, 20,
			     PANEL_VALUE, "",
			     NULL);

  kttarget_item = xv_create(knn_panel, PANEL_TEXT,
			    XV_X, 175,
			    XV_Y, 190,
			    PANEL_LAYOUT, PANEL_VERTICAL,
			    PANEL_LABEL_STRING, "Test target file:",
			    PANEL_VALUE_DISPLAY_LENGTH, 20,
			    PANEL_VALUE, "",
			    NULL);

  koutput_item = xv_create(knn_panel, PANEL_TEXT,
			   XV_X, 2,
			   XV_Y, 190,
			   PANEL_LAYOUT, PANEL_VERTICAL,
			   PANEL_LABEL_STRING, "Output file:",
			   PANEL_VALUE_DISPLAY_LENGTH, 20,
			   PANEL_VALUE, "",
			   NULL);
}

void knn_test(item, event)
Panel_item item;
Event *event;
{

  int nbor, i, **conf, correct, class, Nclasses, test = 0;
  MMatrix *knn, *tp, *tt;
  char cmd_buf[500], pfile[NAME_LEN], tfile[NAME_LEN], tpfile[NAME_LEN],
    ttfile[NAME_LEN], ofile[NAME_LEN], tmp_buf[50];

  nbor = atoi((char *) xv_get(knbor_item, PANEL_VALUE));

  /* check pattern file name (required)   */
  strcpy(pfile, (char *) xv_get(pattern_item, PANEL_VALUE));
  if (strcmp(pfile, "\0") == 0) {
    show_notice(current_panel, item, event, "No test pattern file name");
    return;
  }
  /* check target file name (required)   */
  strcpy(tfile, (char *) xv_get(target_item, PANEL_VALUE));
  if (strcmp(tfile, "\0") == 0) {
    show_notice(current_panel, item, event, "No test target file name");
    return;
  }
  /* check output file name (required)    */
  strcpy(ofile, (char *) xv_get(koutput_item, PANEL_VALUE));
  if (strcmp(ofile, "\0") == 0) {
    show_notice(current_panel, item, event, "No output file name");
    return;
  }
  /* default command for knnclass */
  sprintf(cmd_buf, "%sknnclass -p %s -t %s -o %s -n %d",
	  BINDIR, pfile, tfile, ofile, nbor);

  /* check test pattern file name (optional)   */
  strcpy(tpfile, (char *) xv_get(ktpattern_item, PANEL_VALUE));
  if (strcmp(tpfile, "\0") != 0) {
    sprintf(tmp_buf, " -P %s", tpfile);
    strcat(cmd_buf, tmp_buf);
    test++;
  }
  /* check test target file name (required)  */
  strcpy(ttfile, (char *) xv_get(kttarget_item, PANEL_VALUE));
  if (strcmp(ttfile, "\0") != 0) {
    /*
     * sprintf(tmp_buf, " -T %s",ttfile); strcat(cmd_buf,tmp_buf);
     */
    test++;
  }
  printf("%s\n", cmd_buf);

  show_time(RUSAGE_CHILDREN);

  system(cmd_buf);

  printf("User time used in learning %d s\n", show_time(RUSAGE_CHILDREN));

  /* LOAD RESULT FILE   */
  knn = load_matrix(ofile, "KNN");

  if (test == 2) {
    /* LOAD TEST PATTERN AND TEST TARGET FILES   */
    if ((tp = read_pattern_matrix(tpfile, 2)) == NULL) {
      show_notice(current_panel, item, event, "Check test pattern file");
      return;
    }
    if ((tt = read_target_matrix(ttfile)) == NULL) {
      show_notice(current_panel, item, event, "Check test target file");
      return;
    }
  } else {
    /* LOAD PATTERN AND TARGET FILES   */
    if ((tp = read_pattern_matrix(pfile)) == NULL) {
      show_notice(current_panel, item, event, "Check pattern file");
      return;
    }
    if ((tt = read_target_matrix(tfile)) == NULL) {
      show_notice(current_panel, item, event, "Check target file");
      return;
    }
  }

  Nclasses = MxGetM(tt);

  /* INITIALIZE CONFUSION MATRIX  */
  conf = init_confusion_matrix(Nclasses);

  show_time(RUSAGE_SELF);

  for (i = 0; i < MxGetN(tt); i++) {

    /* GET THE CORRECT CLASS FOR PATTERN VECTOR FROM TARGET MATRIX	 */
    correct = Mcompet(tt, i);

    class = MxGetPr(knn)[0][i];

    mark_class(i, class, correct, 2);

    /* UPDATE CONFUSION MATRIX	 */
    update_confusion_matrix(conf, correct, class, Nclasses);
  }

  printf("User time used in classification %d s\n", show_time(RUSAGE_SELF));

  /* PRINT CONFUSION MATRIX	 */
  xprint_confusion_matrix(conf, Nclasses, "KNN");

  MxFreeMatrix(knn);
  MxFree(knn);
  MxFreeMatrix(tp);
  MxFree(tp);
  MxFreeMatrix(tt);
  MxFree(tt);
}
