/***********************************************************************
*                                                                      *
*                              SPECLASS                                *
*                                                                      *
*                                The                                   *
*                                                                      *
*                      Color Spectra Classifier                        *
*                                                                      *
*                          Program  Package                            *
*                                                                      *
*                           Programmed by			       *
*								       *
*			     J. Haanpalo		               *
*                                                                      *
*                      Copyright (c) 1993-1995                         *
*                                                                      *
************************************************************************
*                                                                      *
*  NOTE: This program package is copyrighted in the sense that it      *
*  may be used for scientific purposes. The package as a whole, or     *
*  parts thereof, cannot be included or used in any commercial         *
*  application without written permission granted by its producents.   *
*  No programs contained in this package may be copied for commercial  *
*  distribution. This program package comes 'as it is' and with no     *
*  warranty.                                                           *
*                                                                      *
*  All comments concerning this program package may be sent to the     *
*  e-mail address 'haanpalo@lut.fi'.                                   *
*                                                                      *
***********************************************************************/

#include<stdio.h>
#include<sys/file.h>
#include<sys/types.h>
#include<string.h>
#include<signal.h>
#include<math.h>
#include<malloc.h>
#include"mmatrix.h"

#ifndef BADSIG
#define BADSIG SIG_ERR		/* SysV compatibility */
#endif

/*#define DEBUG /* */
#define MAX(a, b)  ((a > b) ? (a) : (b))
#define MAT             ".mat"	/* extension for Matlab files   */
#define NUL "\0"

#define NAME_LEN	100	/* default length for all filenames	 */


/* functions for signal handling	*/
void sighandle(), HUPhandle(), test_signal();

/* functions for reading and writing	*/
int read_input(), write_output();

MMatrix *p,			/* pattern matrix				 */
 *t,				/* target matrix				 */
 *P,				/* test pattern matrix				 */
 *T,				/* test target matrix				 */
 *M,				/* subspace matrix for all classes		 */
 *L,				/* pattern vector labels			 */
 *C;				/* class labels					 */

int Nclasses,			/* number of classes	 */
  Dimen,			/* dimension of subspaces	 */
  output,			/* if true then write weight matrices and
				 * parameters into file	 */
  logi,				/* if true then write logfile	 */
  do_test;			/* if true then do some testing (signal
				 * handling)	 */

char output_file[NAME_LEN],	/* output file name	 */
  input_file[NAME_LEN],		/* input file name	 */
  log_file[NAME_LEN];		/* log file name	 */

usage(s)
char *s;
{
  fprintf(stderr, "Error: %s\n", s);
  fprintf(stderr, "USAGE: lsmlearn -p <pattern file> -t <target file> [-P <test pattern file>]\n");
  fprintf(stderr, "       [-T <test target file>] [-i <input file>] [-o <output file>]\n");
  fprintf(stderr, "       [l <logfile>] [-g <gamma>] [-d <dimension>] [-e <epoch>]\n\n");
  fprintf(stderr, "    Learning Subspace Method (LSM)\n");
  fprintf(stderr, "    All input and output files are in Matlab (4.0) format. Pattern file must\n");
  fprintf(stderr, "    be in column order e.g., 100 samples in 61 dimensional space is stored\n");
  fprintf(stderr, "    as a 61x100 matrix.\n\n");
  fprintf(stderr, "PARAMETERS\n");
  fprintf(stderr, "    -g <gamma>  						[0.002]\n");
  fprintf(stderr, "    -d <dimension>  	sets the dimension of subspaces 	[1]\n");
  fprintf(stderr, "    -e <epoch>  	number of epochs			[50]\n");
  fprintf(stderr, "    -i <input file>  	if given then only test pattern and\n");
  fprintf(stderr, "			target files are used.\n");
  fprintf(stderr, "    -l <logfile>  	classification results are stored in\n");
  fprintf(stderr, "			this file after each epoch.\n");
  fprintf(stderr, "SIGNALS\n");
  fprintf(stderr, "    SIGINT signal (sent by CTRL-C) will break the iteration.\n");
  fprintf(stderr, "    SIGHUP signal will show some results and continue processing.\n");
  fprintf(stderr, "	   If output file is given then also subspaces are saved.\n");
  fprintf(stderr, "\n");
  exit(1);
}

/* INIT_LSM(XSIZE)						*/
/* INITIALIZES SUBSPACES WITH ORTHONORMAL BASEVECTORS	 	*/
/* PARAMETERS:							*/
/*		XSIZE 		PATTERN VECTOR DIMENSION 	*/
void init_lsm(Xsize)
int Xsize;
{
  int i;
  srand48(getpid());
  printf("Initializing subspaces\n");
  for (i = 0; i < Nclasses; i++) {
    if (i == 0)
      M = Morth(Mrando(Xsize, Dimen, "M"));
    else
      M = Mcolapp(M, Morth(Mrando(Xsize, Dimen, "M")));
  }
  M = Mnormc(M, M);
}


/* TEST_LSM(PAT,TAR,VERBOSE)					*/
/* TESTS TEST PATTERN VECTORS AND OUTPUTS RESULTS IN A	 	*/
/* CONFUSION MATRIX						*/
/* PARAMETERS:							*/
/*		PAT 		PATTERN MATRIX		 	*/
/*		TAR 		TARGET MATRIX		 	*/
/*		VERBOSE 	IF 1 THEN PRINT CONFUSION MATRIX*/
/* RETURNS:							*/
/*		DOUBLE VALUE (%)				*/
double test_lsm(pat, tar, verbose)
MMatrix *pat, *tar;
int verbose;
{
  MMatrix *D, *X;
  int correct, class, **conf;
  register i;
  double pros = 0.0;

  /* INITIALIZE CONFUSION MATRIX	 */
  conf = init_confusion_matrix(Nclasses);

  /* INITIALIZE DISCRIMINANT MATRIX */
  D = Mmatinit(Nclasses, 1, 0.0, "D", NULL);

  X = Mmatinit(MxGetM(pat), 1, 0.0, "X", NULL);

  /* EVALUATE THE DISCRIMINANT FUNCTIONS FOR ALL CLASSES */
  for (i = 0; i < MxGetN(pat); i++) {
    X = Mmatext(pat, i, 1, X);
    Mdiscriminant(X, D, M, Nclasses, Dimen);

    /* GET THE CORRECT CLASS FOR PATTERN VECTOR FROM TARGET MATRIX	 */
    correct = Mcompet(tar, i);

    /* FIND THE CLASS FOR A PATTERN VECTOR BY CLASSIFYING IT	 */
    class = Mcompet(D, 0);

    /* UPDATE CONFUSION MATRIX	 */
    update_confusion_matrix(conf, correct, class, Nclasses);
  }

  /* PRINT CONFUSION MATRIX	 */
  pros = print_confusion_matrix(conf, Nclasses);

  MxFreeMatrix(D);
  MxFree(D);
  MxFreeMatrix(X);
  MxFree(X);

  return (pros);
}

/* LEARN_LSM(XSIZE,GAMMA,EPOCH)					*/
/* LEARNS PATTERN VECTORS				 	*/
learn_lsm(Xsize, gamma, epoch)
int Xsize,			/* dimension of data vectors		 */
  epoch;			/* number of training epochs		 */
double gamma;			/* constant				 */
{
  MMatrix *D, *X, *MS, *M1, *v, *z;
  FILE *fp;
  double scalar = 0.0, tmp = 0.0;
  int vec, correct, best, e, err = 0, total = 0;
  register i, j, k;

  X = Mmatinit(Xsize, 1, 0.0, "X", NULL);
  D = Mmatinit(Nclasses, 1, 0.0, "D", NULL);
  v = Mmatinit(Dimen, 1, 0.0, "v", NULL);
  z = Mmatinit(Xsize, 1, 0.0, "z", NULL);
  MS = Mmatinit(Xsize, Dimen, 0.0, "MS", NULL);
  M1 = Mmatinit(Xsize, Dimen, 0.0, "M1", NULL);

  /* GET THE NUMBER OF LEARNING VECTORS	 */
  total = MxGetN(p);

  /* USING LOGFILE	 */
  if (logi)
    if ((fp = fopen(log_file, "w")) == NULL)
      errmsg("Cannot open logfile");

  /* ALL PATTERN VECTORS ARE LEARNED EPOCH TIMES */
  for (e = 0; e < epoch; e++) {

    /* ALL PATTERN VECTORS ARE HANDLED ONE BY ONE */
    for (vec = 0; vec < MxGetN(p); vec++) {

      /* EXTRACT CURRENT PATTERN VECTOR FROM PATTERN MATRIX */
      X = Mmatext(p, vec, 1, X);

      /* COMPUTE DISCRIMINANT FUNCTION 	 */
      Mdiscriminant(X, D, M, Nclasses, Dimen);

      /* GET THE CORRECT CLASS FOR PATTERN VECTOR FROM TARGET MATRIX	 */
      correct = Mcompet(t, vec);

      /* FIND THE BEST CLASS FOR PATTERN VECTOR BY CLASSIFYING IT	 */
      best = Mcompet(D, 0);

      /* ROTATE SUBSPACE (CORRECT CLASS)	 */
      MS = Mmatext(M, correct, Dimen, MS);
      v = Mmatmul(Mtransp(MS, NULL), X, v);
      z = Mmatmul(MS, v, z);
      M1 = Mmatadd(MS, Mtransp(Mscalmul(gamma, Mmatmul(v, Mtransp(Mmatsub(X, z, NULL),
					    NULL), NULL), NULL), NULL), M1);
      M = Mmatmerge(M, M1, correct);

      /* IF PATTERN IS MISCLASSIFIED THEN ALSO THE SUBSPACE OF THE BEST 	 */
      /* CLASS MUST BE ROTATED	 */
      if (correct != best) {
	err++;

	/* ROTATE SUBSPACE (BEST CLASS)	 */
	MS = Mmatext(M, best, Dimen, MS);
	v = Mmatmul(Mtransp(MS, NULL), X, v);
	z = Mmatmul(MS, v, z);
	M1 = Mmatadd(MS, Mtransp(Mscalmul(-gamma, Mmatmul(v, Mtransp(Mmatsub(X, z, NULL),
					    NULL), NULL), NULL), NULL), M1);
	M = Mmatmerge(M, M1, best);
      }
    }


    /* CREATE LOGFILE IF NEEDED	 */
    if (logi) {
      fprintf(fp, "%f %f\n", 100 * (1.0 - (double) err / total), test_lsm(P, T, 0));
      fflush(fp);
    }
    printf("epoch %d\r", e);
    fflush(stdout);
    err = 0;
    if (do_test)
      test_signal();
  }
  if (logi)
    fclose(fp);
  printf("\n");

}

main(argc, argv)
int argc;
char **argv;
{
  int pok = 0, tok = 0, tpok = 0, ttok = 0, input = 0, c, Xsize = 0, epoch = 50;
  double gamma = 0.002;
  extern char *optarg;
  extern int optind;
  char pattern_file[NAME_LEN], target_file[NAME_LEN];
  FILE *fp1;
  p = t = P = T = M = NULL;
  Nclasses = Dimen = 1;
  output = logi = do_test = 0;
  while ((c = getopt(argc, argv, "p:t:P:T:d:g:e:i:o:l:")) != -1) {
    switch (c) {
    case 'p':
      strcpy(pattern_file, optarg);
      fp1 = fopen(check_ext(pattern_file, MAT), "r");
      if ((p = MatGetMatrix(fp1, NULL)) == NULL)
	errmsg("No pattern matrix");
      fclose(fp1);
      pok = 1;
      break;
    case 't':
      strcpy(target_file, optarg);
      fp1 = fopen(check_ext(target_file, MAT), "r");
      if ((t = MatGetMatrix(fp1, NULL)) == NULL)
	errmsg("No target matrix");
      fclose(fp1);
      tok = 1;
      break;
    case 'P':
      strcpy(pattern_file, optarg);
      fp1 = fopen(check_ext(pattern_file, MAT), "r");
      if ((P = MatGetMatrix(fp1, NULL)) == NULL)
	errmsg("No test pattern matrix");
      fclose(fp1);
      tpok = 1;
      break;
    case 'T':
      strcpy(target_file, optarg);
      fp1 = fopen(check_ext(target_file, MAT), "r");
      if ((T = MatGetMatrix(fp1, NULL)) == NULL)
	errmsg("No test target matrix");
      fclose(fp1);
      ttok = 1;
      break;
    case 'i':
      strcpy(input_file, optarg);
      input = 1;
      break;
    case 'l':
      strcpy(log_file, optarg);
      logi = 1;
      break;
    case 'o':
      strcpy(output_file, optarg);
      output = 1;
      break;
    case 'd':
      Dimen = atoi(optarg);
      if (Dimen <= 0)
	errmsg("Illegal parameter value for d");
      break;
    case 'g':
      gamma = atof(optarg);
      if (gamma <= 0)
	errmsg("Illegal parameter value for g");
      break;
    case 'e':
      epoch = atoi(optarg);
      if (epoch <= 0)
	errmsg("Illegal parameter value for e");
      break;
    }
  }

  /* IF READY WEIGHT MATRIX IS GIVEN THEN DO SOME TESTING	 */
  if (input) {
    if (!(tpok && ttok)) {
      usage("Both test pattern and test target files must exist...");
      exit(1);
    }
    read_input();
    printf("%d %d\n", MxGetM(M), MxGetN(M));
    test_lsm(P, T);
    exit(1);
  }
  /* CHECK IF PATTERN AND TEST MATRICES EXIST	 */
  if (!(pok && tok)) {
    usage("Both pattern and target files must exist...");
    exit(1);
  }
  if ((!tpok && ttok) || (tpok && !ttok)) {
    usage("Both test pattern and test target files must exist...");
    exit(1);
  }
  /* PATTERN VECTOR DIMENSION	 */
  Xsize = MxGetM(p);

  /* NUMBER OF DIFFERENT CLASSES	 */
  Nclasses = MxGetM(t);

  if (signal(SIGINT, sighandle) == BADSIG)
    errmsg("Cannot catch interrupt signal");
  if (signal(SIGHUP, HUPhandle) == BADSIG)
    errmsg("Cannot catch hang-up signal\n");

  /* INITIALIZE SUBSPACES		 */
  init_lsm(Xsize);

  /* LEARNING	 */
  learn_lsm(Xsize, gamma, epoch);

  if (tpok && ttok)
    test_lsm(P, T, 1);

  if (output)
    if (!write_output())
      fprintf(stderr, "Cannot write output file\n");
}

int write_output()
{
  FILE *wfp;
  if ((wfp = fopen(check_ext(output_file, MAT), "w")) == NULL)
    return 0;
  if (MatPutMatrix(wfp, M))
    return 0;
  if (MatPutMatrix(wfp, Mmatinit(1, 1, (double) Dimen, "DIMEN", NULL)))
    return 0;
  if (MatPutMatrix(wfp, Mmatinit(1, 1, (double) Nclasses, "NCLASSES", NULL)))
    return 0;
  fclose(wfp);
  return 1;
}

int read_input()
{
  FILE *rfp;
  if ((rfp = fopen(check_ext(input_file, MAT), "r")) == NULL)
    return 0;
  if ((M = MatGetMatrix(rfp, "M")) == NULL)
    return 0;
  Dimen = (int) MxGetScalar(MatGetMatrix(rfp, "DIMEN"));
  Nclasses = (int) MxGetScalar(MatGetMatrix(rfp, "NCLASSES"));
  fclose(rfp);
  return 1;
}

void sighandle()
{
  /* BLOODY SysV drops the signal handler out when the signal comes */
  if (signal(SIGINT, sighandle) == BADSIG) {
    fprintf(stderr, "Cannot catch INT signal any more, next one will kill me!\n");
  }
  exit(1);
}

void HUPhandle()
{
  /* BLOODY SysV drops the signal handler out when the signal comes */
  if (signal(SIGHUP, SIG_IGN) == BADSIG) {
    fprintf(stderr, "Cannot block hang-up signal\n");
  }
  if (signal(SIGHUP, HUPhandle) == BADSIG) {
    fprintf(stderr, "Cannot recatch hang-up signal\n");
  }
  do_test = 1;
}

void test_signal()
{

  test_lsm(p, t, 1);
  test_lsm(P, T, 1);
  if (output)
    if (!write_output())
      fprintf(stderr, "Cannot write output file\n");
  do_test = 0;
}
