/***********************************************************************
*                                                                      *
*                              SPECLASS                                *
*                                                                      *
*                                The                                   *
*                                                                      *
*                      Color Spectra Classifier                        *
*                                                                      *
*                          Program  Package                            *
*                                                                      *
*                           Programmed by			       *
*								       *
*			     J. Haanpalo		               *
*                                                                      *
*                      Copyright (c) 1993-1995                         *
*                                                                      *
************************************************************************
*                                                                      *
*  NOTE: This program package is copyrighted in the sense that it      *
*  may be used for scientific purposes. The package as a whole, or     *
*  parts thereof, cannot be included or used in any commercial         *
*  application without written permission granted by its producents.   *
*  No programs contained in this package may be copied for commercial  *
*  distribution. This program package comes 'as it is' and with no     *
*  warranty.                                                           *
*                                                                      *
*  All comments concerning this program package may be sent to the     *
*  e-mail address 'haanpalo@lut.fi'.                                   *
*                                                                      *
***********************************************************************/

#include <math.h>
#include "xglobal.h"
#include "global.h"
#include "matfile.h"

Panel_item lsinput_item, lsoutput_item, lstpattern_item, lsttarget_item,
  lslog_item, lsepoch_item, lsgamma_item, lsdimen_item, lstest_item, lslearn_item;

void exec_lsmlearn(), lsm_test();

void init_lsm_panel()
{
  if (lsm_panel) {
    xv_set(current_panel, XV_SHOW, FALSE, NULL);
    xv_set(lsm_panel, XV_SHOW, TRUE, NULL);
    current_panel = lsm_panel;
    return;
  }
  current_panel = lsm_panel = xv_create(frame, PANEL,
					XV_X, PANEL_X,
					XV_Y, PANEL_Y,
					XV_WIDTH, PANEL_WIDTH,
					XV_HEIGHT, PANEL_HEIGHT,
					NULL);

  lsgamma_item = xv_create(lsm_panel, PANEL_TEXT,
			   XV_X, 0,
			   XV_Y, 0,
			   PANEL_LAYOUT, PANEL_VERTICAL,
			   PANEL_LABEL_STRING, "gamma",
			   PANEL_VALUE_DISPLAY_LENGTH, DISPLAY_LENGTH,
			   PANEL_VALUE_STORED_LENGTH, DISPLAY_LENGTH,
			   PANEL_VALUE, "0.002",
			   NULL);

  lsdimen_item = xv_create(lsm_panel, PANEL_TEXT,
			   XV_X, 100,
			   XV_Y, 0,
			   PANEL_LAYOUT, PANEL_VERTICAL,
			   PANEL_LABEL_STRING, "dimension",
			   PANEL_VALUE_DISPLAY_LENGTH, DISPLAY_LENGTH,
			   PANEL_VALUE_STORED_LENGTH, DISPLAY_LENGTH,
			   PANEL_VALUE, "1",
			   NULL);

  lsepoch_item = xv_create(lsm_panel, PANEL_TEXT,
			   XV_X, 200,
			   XV_Y, 0,
			   PANEL_LAYOUT, PANEL_VERTICAL,
			   PANEL_LABEL_STRING, "epoch",
			   PANEL_VALUE_DISPLAY_LENGTH, DISPLAY_LENGTH,
			   PANEL_VALUE_STORED_LENGTH, DISPLAY_LENGTH,
			   PANEL_VALUE, "50",
			   NULL);

  lslearn_item = xv_create(lsm_panel, PANEL_BUTTON,
			   XV_X, 0,
			   XV_Y, 70,
			   PANEL_LABEL_STRING, "learn",
			   PANEL_NOTIFY_PROC, exec_lsmlearn,
			   NULL);

  lstest_item = xv_create(lsm_panel, PANEL_BUTTON,
			  XV_X, 0,
			  XV_Y, 100,
			  PANEL_LABEL_STRING, " test ",
			  PANEL_NOTIFY_PROC, lsm_test,
			  NULL);

  lsinput_item = xv_create(lsm_panel, PANEL_TEXT,
			   XV_X, 2,
			   XV_Y, 120,
			   PANEL_LAYOUT, PANEL_VERTICAL,
			   PANEL_LABEL_STRING, "Input file:",
			   PANEL_VALUE_DISPLAY_LENGTH, 20,
			   PANEL_VALUE, "",
			   NULL);

  lsoutput_item = xv_create(lsm_panel, PANEL_TEXT,
			    XV_X, 2,
			    XV_Y, 155,
			    PANEL_LAYOUT, PANEL_VERTICAL,
			    PANEL_LABEL_STRING, "Output file:",
			    PANEL_VALUE_DISPLAY_LENGTH, 20,
			    PANEL_VALUE, "",
			    NULL);

  lslog_item = xv_create(lsm_panel, PANEL_TEXT,
			 XV_X, 2,
			 XV_Y, 190,
			 PANEL_LAYOUT, PANEL_VERTICAL,
			 PANEL_LABEL_STRING, "Logfile:",
			 PANEL_VALUE_DISPLAY_LENGTH, 20,
			 PANEL_VALUE, "",
			 NULL);

  lstpattern_item = xv_create(lsm_panel, PANEL_TEXT,
			      XV_X, 175,
			      XV_Y, 155,
			      PANEL_LAYOUT, PANEL_VERTICAL,
			      PANEL_LABEL_STRING, "Test pattern file:",
			      PANEL_VALUE_DISPLAY_LENGTH, 20,
			      PANEL_VALUE, "",
			      NULL);

  lsttarget_item = xv_create(lsm_panel, PANEL_TEXT,
			     XV_X, 175,
			     XV_Y, 190,
			     PANEL_LAYOUT, PANEL_VERTICAL,
			     PANEL_LABEL_STRING, "Test target file:",
			     PANEL_VALUE_DISPLAY_LENGTH, 20,
			     PANEL_VALUE, "",
			     NULL);
}

void exec_lsmlearn(item, event)
Panel_item item;
Event *event;
{
  double gam, seed;
  int dimen, epochs, test = 0;
  char cmd_buf[500], patternfile[NAME_LEN], targetfile[NAME_LEN], ifile[NAME_LEN],
    ofile[NAME_LEN], logfile[NAME_LEN], tmp_buf[NAME_LEN], tpatternfile[NAME_LEN],
    ttargetfile[NAME_LEN];

  /* not checked!!! */
  epochs = atoi((char *) xv_get(lsepoch_item, PANEL_VALUE));
  dimen = atoi((char *) xv_get(lsdimen_item, PANEL_VALUE));

  gam = atof((char *) xv_get(lsgamma_item, PANEL_VALUE));

  /* check pattern file name (required)   */
  strcpy(patternfile, (char *) xv_get(pattern_item, PANEL_VALUE));
  if (strcmp(patternfile, "\0") == 0) {
    show_notice(current_panel, item, event, "No pattern file name");
    return;
  }
  /* check target file name (required)   */
  strcpy(targetfile, (char *) xv_get(target_item, PANEL_VALUE));
  if (strcmp(targetfile, "\0") == 0) {
    show_notice(current_panel, item, event, "No target file name");
    return;
  }
  /* check output file name (required)    */
  strcpy(ofile, (char *) xv_get(lsoutput_item, PANEL_VALUE));
  if (strcmp(ofile, "\0") == 0) {
    show_notice(current_panel, item, event, "No output file name");
    return;
  }
  /* check test pattern file name (optional)   */
  strcpy(tpatternfile, (char *) xv_get(lstpattern_item, PANEL_VALUE));
  if (strcmp(tpatternfile, "\0") != 0)
    test++;

  /* check test target file name (optional)   */
  strcpy(ttargetfile, (char *) xv_get(lsttarget_item, PANEL_VALUE));
  if (strcmp(ttargetfile, "\0") != 0)
    test++;
  /* default command for lsmlearn */
  sprintf(cmd_buf, "%slsmlearn -p %s -t %s -o %s -e %d -d %d -g %f",
	  BINDIR, patternfile, targetfile, ofile, epochs, dimen, gam);

  /* test set is used 	 */
  if (test == 2) {
    sprintf(tmp_buf, " -P %s -T %s", tpatternfile, ttargetfile);
    strcat(cmd_buf, tmp_buf);
  }
  /* check logfile name (optional)        */
  strcpy(logfile, (char *) xv_get(lslog_item, PANEL_VALUE));
  if (strcmp(logfile, "\0") != 0) {
    sprintf(tmp_buf, " -l %s", logfile);
    strcat(cmd_buf, tmp_buf);
  }
  printf("%s\n", cmd_buf);

  show_time(RUSAGE_CHILDREN);

  system(cmd_buf);

  printf("User time used in learning %d s\n", show_time(RUSAGE_CHILDREN));
}

void lsm_test(item, event)
Panel_item item;
Event *event;
{
  MMatrix *D, *X, *tp, *tt, *M;
  int correct, class, **conf, Dimen, Nclasses;
  register i, j;
  double pros = 0.0;
  char pfile[NAME_LEN], tfile[NAME_LEN], ifile[NAME_LEN];

  /* check test pattern file name (required)   */
  strcpy(pfile, (char *) xv_get(lstpattern_item, PANEL_VALUE));
  if (strcmp(pfile, "\0") == 0) {
    show_notice(current_panel, item, event, "No test pattern file name");
    return;
  }
  /* check test target file name (required)   */
  strcpy(tfile, (char *) xv_get(lsttarget_item, PANEL_VALUE));
  if (strcmp(tfile, "\0") == 0) {
    show_notice(current_panel, item, event, "No test target file name");
    return;
  }
  /* check input file name (required)  */
  strcpy(ifile, (char *) xv_get(lsinput_item, PANEL_VALUE));
  if (strcmp(ifile, "\0") == 0) {
    show_notice(current_panel, item, event, "No input file name");
    return;
  }
  /* LOAD WEIGHT FILE   */
  if ((M = load_matrix(ifile, "M")) == NULL) {
    show_notice(current_panel, item, event, "Check input file");
    return;
  }
  /* LOAD TEST PATTERN AND TARGET FILES   */
  if ((tp = read_pattern_matrix(pfile, 2)) == NULL) {
    show_notice(current_panel, item, event, "Check test pattern file");
    return;
  }
  if ((tt = read_target_matrix(tfile)) == NULL) {
    show_notice(current_panel, item, event, "Check test target file");
    return;
  }
  Dimen = (int) MxGetScalar(load_matrix(ifile, "DIMEN"));
  Nclasses = (int) MxGetScalar(load_matrix(ifile, "NCLASSES"));

  /* INITIALIZE CONFUSION MATRIX	 */
  conf = init_confusion_matrix(Nclasses);

  /* INITIALIZE DISCRIMINANT MATRIX */
  D = Mmatinit(Nclasses, 1, 0.0, "D", NULL);

  X = Mmatinit(MxGetM(tp), 1, 0.0, "X", NULL);

  show_time(RUSAGE_SELF);

  /* EVALUATE DISCRIMINANT FUNCTIONS FOR ALL CLASSES */
  for (i = 0; i < MxGetN(tp); i++) {
    X = Mmatext(tp, i, 1, X);
    Mdiscriminant(X, D, M, Nclasses, Dimen);

    /* GET THE CORRECT CLASS FOR PATTERN VECTOR FROM TARGET MATRIX	 */
    correct = Mcompet(tt, i);

    /* FIND THE CLASS FOR A PATTERN VECTOR BY CLASSIFYING IT	 */
    class = Mcompet(D, 0);
    mark_class(i, class, correct, 2);

    /* UPDATE CONFUSION MATRIX	 */
    update_confusion_matrix(conf, correct, class, Nclasses);
  }

  printf("User time used in classification %d s\n", show_time(RUSAGE_SELF));

  /* PRINT CONFUSION MATRIX	 */
  xprint_confusion_matrix(conf, Nclasses, "LSM");

  MxFreeMatrix(M);
  MxFree(M);
  MxFreeMatrix(D);
  MxFree(D);
  MxFreeMatrix(X);
  MxFree(X);
  MxFreeMatrix(tp);
  MxFree(tp);
  MxFreeMatrix(tt);
  MxFree(tt);
}
