/***********************************************************************
*                                                                      *
*                              SPECLASS                                *
*                                                                      *
*                                The                                   *
*                                                                      *
*                      Color Spectra Classifier                        *
*                                                                      *
*                          Program  Package                            *
*                                                                      *
*                  Version 0.8 (October 17, 1994)                      *
*                                                                      *
*                           Programmed by			       *
*								       *
*			     J. Haanpalo		               *
*                                                                      *
*                      Copyright (c) 1993-1994                         *
*                                                                      *
************************************************************************
*                                                                      *
*  NOTE: This program package is copyrighted in the sense that it      *
*  may be used for scientific purposes. The package as a whole, or     *
*  parts thereof, cannot be included or used in any commercial         *
*  application without written permission granted by its producents.   *
*  No programs contained in this package may be copied for commercial  *
*  distribution. This program package comes 'as it is' and with no     *
*  warranty.                                                           *
*                                                                      *
*  All comments concerning this program package may be sent to the     *
*  e-mail address 'haanpalo@lut.fi'.                                   *
*                                                                      *
***********************************************************************/

#include <sys/file.h>
#include <sys/wait.h>
#include <string.h>
#include <math.h>
#include <stdio.h>
#include <malloc.h>
#include "global.h"
#include "xglobal.h"

#define index(mat, a, b) (*(mxGetPr(mat)+(a)*mxGetM(mat)+(b)))

extern void draw_spek();
extern int destroy_list();

void write_subwindow();

/*int check_status(status)
int status;
{
	int sig;
	if(WIFEXITED(status))
		return(0);
	if(WIFSTOPPED(status))
		return(WSTOPSIG(status));
	if(WIFSIGNALED(status))
		return(WTERMSIG(status));
}*/

/* FREE_GLOBALS()							*/
/* FREES GLOBAL MATLAB MATRICES 					*/
/* PARAMETERS:								*/
/*              NONE							*/
/* RETURNS:								*/
/*              NOTHING							*/
free_globals()
{
	MxFreeMatrix(pattern_matrix);
	MxFree(pattern_matrix);
	pattern_matrix = NULL;
	MxFreeMatrix(target_matrix);
	MxFree(target_matrix);
	target_matrix = NULL;
	free(cntable);
	cntable = NULL;
	free(class_table);
	class_table = NULL;
}

/* MAKE_DOUBLE_VECTOR(LBUF, MAXVAL, DIM)				*/
/* CONVERTS ASCII LINE INTO DOUBLE NUMBERS AND STORES THEM INTO 	*/
/* DOUBLE VECTOR.							*/
/* PARAMETERS:								*/
/*              LBUF    VECTOR OF ASCII	NUMBERS				*/
/*              MAXVAL  LARGEST VALUE (LOWPASS FILTER)	 		*/
/*              DIM   	VECTOR DIMENSION				*/
/* RETURNS:								*/
/*              DOUBLE VECTOR						*/
double *make_double_vector(lbuf, maxval, dim)
char *lbuf;
int maxval, *dim;
{
	static double dbuffer[300] = {0};
	double dp;
	char *p;
	int lkm = 0;

	if(p = strtok(lbuf, SEPARATOR)) {
		dp = atof(p);
		if(dp > maxval) dp = maxval;
		if(dp < 0) dp = 0;
                dbuffer[lkm++] = dp;
        }
	while(p = strtok(NULL, SEPARATOR)) {
		dp = atof(p);
		if(dp > maxval) dp = maxval;
		if(dp < 0) dp = 0;
                dbuffer[lkm++] = dp;
        }
	*dim = lkm;

	return dbuffer;
}

/* MAKE_ASCII_VECTOR(DBUF,DIM,CLASS)					*/
/* CONVERTS DOUBLE VECTOR INTO ASCII NUMBERS AND STORES THEM INTO 	*/
/* ASCII VECTOR.							*/
/* PARAMETERS:								*/
/*              LBUF    VECTOR OF ASCII	NUMBERS				*/
/*              DIM   	VECTOR DIMENSION				*/
/*              CLASS   CLASS OF THE VECTOR		 		*/
/* RETURNS:								*/
/*              ASCII VECTOR						*/
char *make_ascii_vector(dbuf,dim,class)
double *dbuf;
int dim,class;
{
	static char lbuf[LINE_LEN];
	char tmp[10];
	double *ptr;
	register i;

	ptr = dbuf;
	sprintf(lbuf,"\0");
	for(i=0;i<dim;i++) {
		sprintf(tmp,"%f ",*ptr++);
		strcat(lbuf,tmp);
	}
	sprintf(tmp,"%d",class);
	strcat(lbuf,tmp);
	return lbuf;
}

/* CREATE_PATTERN(DBUF,DIM)						*/
/* CREATES NEW PATTERN_MATRIX OR APPENDS ONE PATTERN VECTOR INTO MATRIX	*/
/* PARAMETERS:								*/
/*              DBUF    VECTOR OF DOUBLE NUMBERS			*/
/*              DIM   	VECTOR DIMENSION				*/
/* RETURNS:								*/
/*              1 ON SUCCES :)						*/
int create_pattern(dbuf,dim)
double *dbuf;
int dim;
{
	MMatrix *new_matrix;
	int rows, cols;
	register i;

	/* IF FIRST CALL THEN LET'S CREATE A NEW PATTERN_MATRIX */
        if (pattern_matrix==NULL) {
	       	pattern_matrix = MxCreateFull(dim,1,0);
        	memcpy(MxGetPr(pattern_matrix)[0], dbuf, dim*sizeof(double));
	}
	/* ELSE APPEND DBUF AT THE END OF PATTERN_MATRIX */
	else {
		rows = MxGetM(pattern_matrix);
		cols = MxGetN(pattern_matrix);
		if(dim!=rows)
			errmsg("create_pattern: wrong pattern dimension");
		new_matrix = MxCreateFull(rows, cols+1, 0);
		for (i=0;i<cols;i++)
        		memcpy(MxGetPr(new_matrix)[i], MxGetPr(pattern_matrix)[i],
			rows*sizeof(double));
        	memcpy(MxGetPr(new_matrix)[cols], dbuf, dim*sizeof(double));
		MxFreeMatrix(pattern_matrix);
		MxFree(pattern_matrix);
		pattern_matrix=new_matrix;
	}
        return 1;
}

/* CREATE_TABLE(TABLE,MESSU)						*/
/* CREATES NEW LABEL TABLE OR APPENDS ONE LABEL STRING INTO TABLE 	*/
/* PARAMETERS:								*/
/*              TABLE  	POINTER TO THE LABEL TABLE			*/
/*              MESSU   LABEL STRING TO BE ADDED			*/
/* RETURNS:								*/
/*              1 ON SUCCES :)						*/
int create_table(table, messu)
char **table, *messu;
{
	char *buf;

	/* CREATE NEW TABLE (FIRST CALL) */
        if (*table==NULL) {
		if((*table=(char*)malloc((strlen(messu)+1)*sizeof(char)))==NULL)
			errmsg("create_table: Table allocation failed\n");
		strcpy(*table,messu);
	}
	else {
		buf=(char*)malloc((strlen(*table)+strlen(messu)+1)*sizeof(char));
		if(buf==NULL)
			errmsg("create_table: Buffer allocation failed\n");
		strcpy(buf,*table);
		strcat(buf,messu);
		free(*table);
	       	*table = buf;
	}
        return 1;
}

/* CREATE_TARGET(CLASS,ROWS)						*/
/* CREATES TARGET_MATRIX					 	*/
/* PARAMETERS:								*/
/*              CLASS	CLASS OF THE CURRENT VECTOR			*/
/*              ROWS	NUMBER OF ROWS IN TARGET MATRIX			*/
/* RETURNS:								*/
/*              1 ON SUCCES :)						*/
int create_target(class,rows)
int class,rows;
{
	MMatrix *new_matrix;
	double cbuf[MAXCLASS];
	int cols;
	register i;

	for(i=0;i<rows;i++)
		cbuf[i]=0.0;
	cbuf[class]=1.0;

	/* CREATE TARGET_MATRIX (FIRST CALL) */
        if (target_matrix==NULL) { 
	       	target_matrix = MxCreateFull(rows,1,0);
        	memcpy(MxGetPr(target_matrix)[0],cbuf,rows*sizeof(double));
	}
	else {
		rows = MxGetM(target_matrix);
		cols = MxGetN(target_matrix);
		new_matrix = MxCreateFull(rows, cols+1, 0);
		for (i=0;i<cols;i++)
        		memcpy(MxGetPr(new_matrix)[i], MxGetPr(target_matrix)[i],
			rows*sizeof(double));
        	memcpy(MxGetPr(new_matrix)[cols],cbuf,rows*sizeof(double));
		MxFreeMatrix(target_matrix);
		MxFree(target_matrix);
		target_matrix=new_matrix;
	}
        return 1;
}

/* RESIZE_TARGET(COUNT)							*/
/* RESIZES TARGET_MATRIX BY CUTTING OFF UNUSED ROWS AT THE END OF MATRIX*/
/* PARAMETERS:								*/
/*              COUNT	NUMBER OF ROWS USED				*/
/* RETURNS:								*/
/*              1 ON SUCCES :)						*/
int resize_target(count)
int count;
{
	MMatrix *new_matrix;
	int rows, cols;
	register i, j;
	rows = MxGetM(target_matrix);
	cols = MxGetN(target_matrix);
	new_matrix = MxCreateFull(count, cols, 0);
	for(i=0;i<cols;i++)
		for(j=0;j<count;j++)
			MxGetPr(new_matrix)[i][j] = MxGetPr(target_matrix)[i][j];
	MxFreeMatrix(target_matrix);
	MxFree(target_matrix);
	target_matrix=new_matrix;
	return 1;
}

/* WRITE_PATTERN_MATRIX(FNAME)						*/
/* STORES PATTERN_MATRIX AND LABELS MATRIX INTO FILE FNAME		*/
/* PARAMETERS:								*/
/*              FNAME	FILENAME					*/
/* RETURNS:								*/
/*              1 ON SUCCES :)						*/
int write_pattern_matrix(fname)
char *fname;
{
	FILE *wfp;
	if((wfp = fopen(check_ext(fname,MAT), "w"))==NULL) {
	  printf("Error in opening file %s\n",fname);
	  return 0;
	}
	MxSetName(pattern_matrix,"P");
	if(MatPutMatrix(wfp,pattern_matrix)) {
	  printf("Error in writing pattern_matrix into file\n");
	  return 0;
	}
	if(MatPutString(wfp, cntable, 20, MxGetN(pattern_matrix), "LABELS")==0) {
	  printf("Couldn't create and save cntable matrix\n");
	  return 0;
	}
	fclose(wfp);
	return 1;
}

/* WRITE_TARGET_MATRIX(FNAME)						*/
/* STORES TARGET_MATRIX INTO FILE FNAME					*/
/* PARAMETERS:								*/
/*              FNAME	FILENAME					*/
/* RETURNS:								*/
/*              1 ON SUCCES :)						*/
int write_target_matrix(fname)
char *fname;
{
	FILE *wfp;
	if((wfp = fopen(check_ext(fname,MAT), "w"))==NULL) {
	  fprintf(stderr,"Error in opening file %s\n",fname);
	  return 0;
	}
	MxSetName(target_matrix,"T");
	if(MatPutMatrix(wfp,target_matrix)) {
	  fprintf(stderr,"Error in writing target_matrix into file\n");
	  return 0;
	}
	fclose(wfp);
	return 1;
}

/* READ_PATTERN_MATRIX(FNAME, LIST)					*/
/* READS PATTERN_MATRIX FROM MATLAB FILE FNAME AND STORES DATA INTO LIST*/
/* PARAMETERS:								*/
/*              FNAME	FILENAME					*/
/*              LIST	POINTER TO THE START OF LINKED LIST		*/
/* RETURNS:								*/
/*              MATLAB MATRIX						*/
MMatrix *read_pattern_matrix(fname, list)
char *fname;
int list;
{
	MMatrix *p_mat, *s_mat;
	Spec *next;
	char *label_buf, *file, *label, messu[20];
	int i, dim=0, total=0, string_length = 0;

	/* READ MATLAB MATRIX P FROM FILE FNAME */
	if((p_mat = load_matrix(fname,"P"))==NULL)
		return NULL;

	/* LOAD LABEL MATRIX FROM THE SAME FILE IF IT EXISTS ELSE USE NUMBERS AS LABELS */
	if((s_mat = load_matrix(fname,"LABELS"))!=NULL) {
		string_length = MxGetM(s_mat)*MxGetN(s_mat);
		if((label_buf=(char*)malloc((string_length+1)*sizeof(char)))==NULL)
			errmsg("read_pattern_matrix: Cannot allocate memory for label_buf");
		MatGetString(s_mat,label_buf);
	}
	else
		printf("Number labels used for spectra\n");

	dim = MxGetM(p_mat);

	/* HANDLE TEST LIST */
	if(list == 2) {
		list2_total = MxGetN(p_mat);

		/* OLD TEST LIST IS DESTROYED ALWAYS WHEN NEW FILE IS READ */
		destroy_list(&lstart);

		/* USE LOADED LABELS */
		if(s_mat != NULL) {
			file = strtok(label_buf, SEPARATOR);
			label = strtok(NULL, SEPARATOR);
			make_spectrum(0, dim, 0, file, label, MxGetPr(p_mat)[0], &lstart);
			for(i=1;i<MxGetN(s_mat);i++) {
				file = strtok(NULL, SEPARATOR);
				label = strtok(NULL, SEPARATOR);
				make_spectrum(0, dim, 0, file, label, MxGetPr(p_mat)[i], &lstart);
			}
		}
		else
			/* GENERATE NEW LABELS (NUMBERS) */
			for(i=0;i<list2_total;i++) {
				sprintf(messu, "%d", i);
				make_spectrum(0, dim, 0, fname, messu, MxGetPr(p_mat)[i], &lstart);
			}
	}
	/* HANDLE EDITOR LIST */
	else {

		/* USE LOADED LABELS */
		if(s_mat != NULL) {
			file = strtok(label_buf, SEPARATOR);
			label = strtok(NULL, SEPARATOR);
			make_spectrum(0, dim, 0, file, label, MxGetPr(p_mat)[0], &start);
			for(i=1;i<MxGetN(s_mat);i++) {
				file = strtok(NULL, SEPARATOR);
				label = strtok(NULL, SEPARATOR);
				make_spectrum(0, dim, 0, file, label, MxGetPr(p_mat)[i], &start);
			}
		}
		else
			/* GENERATE NEW LABELS (NUMBERS) */
			for(i=0;i<MxGetN(p_mat);i++) {
				sprintf(messu, "%d", list1_total + i);
				make_spectrum(0, dim, 0, fname, messu, MxGetPr(p_mat)[i], &start);
			}
		list1_total += MxGetN(p_mat);
	}

	MxFreeMatrix(s_mat);
	MxFree(s_mat);
	return(p_mat);
}

/* READ_TARGET_MATRIX(FNAME)						*/
/* READS TARGET_MATRIX FROM MATLAB FILE FNAME				*/
/* PARAMETERS:								*/
/*              FNAME	FILENAME					*/
/* RETURNS:								*/
/*              MATLAB MATRIX						*/
MMatrix *read_target_matrix(fname)
char *fname;
{
	MMatrix *t_mat;
	int i=0;
	if((t_mat = load_matrix(fname,"T"))==NULL)
		return NULL;
	return(t_mat);
}

/* MAKE_SPECTRUM(CLASS,DIM,TYPE,FILE,LABEL,DATA,LIST)			*/
/* FILLS THE SPEC STRUCTURE FIELDS WITH CORRECT DATA AND APPENDS	*/
/* STRUCTURE TO LINKED LIST						*/
/* PARAMETERS:								*/
/*              CLASS	CLASS OF DATA VECTOR				*/
/*              DIM	DIMENSION OF DATA VECTOR			*/
/*              TYPE	TYPE OF DATA VECTOR (NOT USED ANYWHERE)		*/
/*              FILE	FILENAME					*/
/*              LABEL	LABEL OF DATA VECTOR				*/
/*              DATA	DOUBLE POINTER TO DATA VECTOR			*/
/*              LIST	POINTER TO THE START OF CORRECT LIST		*/
/* RETURNS:								*/
/*            	NOTHING							*/
int make_spectrum(class,dim,type,file,label,data,list)
int class,dim,type;
char *file,*label;
double *data;
Spec **list;
{
	Spec *spec, *next;

	/* ALLOCATE MEMORY FOR THE STRUCTURE AND ITS FIELDS */
	if((spec = (Spec*)malloc(sizeof(Spec)))==NULL)
		errmsg("make_spectrum: Cannot allocate memory for struct");
	if((spec->file = (char*)malloc((strlen(file)+1)*sizeof(char)))==NULL)
		errmsg("make_spectrum: Cannot allocate memory for spec->file");
	if((spec->label = (char*)malloc((strlen(label)+1)*sizeof(char)))==NULL)
		errmsg("make_spectrum: Cannot allocate memory for spec->label");
	if((spec->data = (double*)malloc((dim+1)*sizeof(double)))==NULL)
		errmsg("make_spectrum: Cannot allocate memory for spec->data");

	/* FILL FIELDS */
	spec->class=class;
	spec->classed=-1;
	spec->dim=dim;
	spec->type=type;
	strcpy(spec->file,file);
	strcpy(spec->label,label);
	memcpy(spec->data,data,dim*sizeof(double));
	spec->next=NULL;

	/* APPEND STRUCTURE INTO LINKED LIST */
	if(*list)
		next = *list;
	else {
		*list = spec;
		return;
	}
	while(next->next)
		next=next->next;
	next->next=spec;
}

/* DELETE_SPECTRUM(FILE,LABEL,LIST)					*/
/* DELETE SPECTRUM FROM LINKED LIST					*/
/* PARAMETERS:								*/
/*              FILE	FILENAME					*/
/*              LABEL	LABEL OF DATA (SPECTRUM) VECTOR			*/
/*              LIST	POINTER TO THE START OF CORRECT LIST		*/
/* RETURNS:								*/
/*              NOTHING							*/
int delete_spectrum(file,label,list)
char *file,*label;
Spec **list;
{
	Spec *next, *prev;
	next=*list;
	prev=*list;
	if((*list)->next==NULL) {
		if((*list==start&&!visible)||(*list==lstart&&visible))
			draw_spek(*list,0);
		free((*list)->file);
		free((*list)->label);
		free((*list)->data);
		free(*list);
		*list=NULL;
		return;
	}
	while(next) {
		if(!strcmp(next->file,file)&&!strcmp(next->label,label)) {
			if((*list==start&&!visible)||(*list==lstart&&visible))
				draw_spek(next,0);
			free(next->file);
			free(next->label);
			free(next->data);
			if(next==*list)
				*list=(*list)->next;
			else if(next->next)
				prev->next=next->next;
			else
				prev->next=NULL;
			free(next);
			break;
		}
		prev=next;
		next=next->next;
	}
}

/* DESTROY_LIST(LIST)							*/
/* DESTROY WHOLE LINKED LIST POINTED BY LIST				*/
/* PARAMETERS:								*/
/*              LIST	POINTER TO THE START OF CORRECT LIST		*/
/* RETURNS:								*/
/*              NOTHING							*/
int destroy_list(list)
Spec **list;
{
	Spec *this,*next;
	next=*list;
	while(next) {
		this=next;
		if((*list==start&&!visible)||(*list==lstart&&visible))
			draw_spek(this,0);
		free(this->file);
		free(this->label);
		free(this->data);
		next=this->next;
		free(this);
	}
	*list=NULL;
}

/* DRAW_LIST(LIST)							*/
/* DRAWS ALL DATA VECTORS STORED IN LIST				*/
/* PARAMETERS:								*/
/*              LIST	POINTER TO THE START OF CORRECT LIST		*/
/* RETURNS:								*/
/*              NOTHING							*/
int draw_list(list)
Spec *list;
{
	Spec *next;
	if(list==NULL)
		return;
        next = list;
        while(next) {
                draw_spek(next,1);
                next = next->next;
        }
}

/* UNDRAW_LIST(LIST)							*/
/* UNDRAWS ALL DATA VECTORS STORED IN LIST				*/
/* PARAMETERS:								*/
/*              LIST	POINTER TO THE START OF CORRECT LIST		*/
/* RETURNS:								*/
/*              NOTHING							*/
int undraw_list(list)
Spec *list;
{
	Spec *next;
	if(list==NULL)
		return;
        next = list;
        while(next) {
		draw_spek(next,0);
                next = next->next;
        }
}

/* GET_SPEC(LIST,N)							*/
/* GET POINTER TO N'TH SPECTRUM IN LINKED LIST				*/
/* PARAMETERS:								*/
/*		LIST	POINTER TO THE START OF A CORRECT LIST		*/
/*		N	NUMBER OF SPECTRUM				*/
/* RETURNS:								*/
/*              POINTER TO SPEC STRUCTURE				*/
Spec *get_spec(list,n)
Spec *list;
int n;
{
	Spec *next;
	int counter=0;
        next = list;
        while(next) {
		if(counter==n)
			return(next);
                next = next->next;
		counter++;
        }
}

/* DRAW_CLASS(LIST,CNO)							*/
/* DRAW ONE SPECTRUM CLASS FROM A SET POINTED BY LIST			*/
/* PARAMETERS:								*/
/*		LIST	POINTER TO THE START OF A CORRECT LIST		*/
/*		CNO	NUMBER OF CLASS WHICH MEMBERS ARE TO BE SHOWN	*/
/* RETURNS:								*/
/*              NOTHING							*/
void draw_class(list,cno)
Spec *list;
int cno;
{
	Spec *next;
	int ctotal=0,cmissed=0,total=0;
	char cbuf[50];
	next=list;
	while(next) {
		/* FIND RIGHT SPECTRA	*/
		if(next->class==cno) {
			if(next->class==next->classed)
				draw_spek(next,1);	/* CORRECTLY CLASSIFIED */
			else {
				draw_spek(next,2);	/* MISCLASSIFIED	*/
				cmissed++;
			}
			ctotal++;	
		}
		next=next->next;
		total++;
	}
	/* SHOW THE NUMBER OF SELECTIONS MADE	*/
	sprintf(cbuf,"Class %d: Total %d  Missed %d   All %d",cno,ctotal,cmissed,total);
	xv_set(frame, FRAME_LEFT_FOOTER, cbuf, NULL);
}

/* XPRINT_CONFUSION_MATRIX(CONF,NCLASSES)				*/
/* PRINTS CONFUSION MATRIX INTO X-SUBWINDOW				*/
/* PARAMETERS:								*/
/*		CONF		CONFUSION MATRIX			*/
/*		NCLASSES	NUMBER OF CLASSES			*/
/* RETURNS:								*/
/*              NOTHING							*/
void xprint_confusion_matrix(conf,Nclasses,method)
int **conf,Nclasses;
char *method;
{        
	char buf[100];
	double pros=0.0;
	register i,j;

        /* CALCULATE CLASSIFICATION %   */
        pros = (double)conf[Nclasses][Nclasses]/conf[Nclasses+1][Nclasses+1]*100.0;

        sprintf(buf,"%s\n\n",method);
	write_subwindow(buf);

        /* PRINT CONFUSION MATRIX       */
        for(i=0;i<Nclasses;i++) {
                for(j=0;j<Nclasses;j++) {
                        sprintf(buf,"%3d ",conf[i][j]);
			write_subwindow(buf);
		}
		sprintf(buf," | %3d  %3d\n",conf[i][Nclasses],conf[i][Nclasses+1]);
		write_subwindow(buf);
        }
        for(i=0;i<Nclasses+2;i++) {
                sprintf(buf,"----");
		write_subwindow(buf);
	}

        sprintf(buf,"\n");
	write_subwindow(buf);

        for(i=Nclasses;i<Nclasses+2;i++) {
                for(j=0;j<Nclasses;j++) {
                        sprintf(buf,"%3d ",conf[i][j]);
			write_subwindow(buf);
		}
                sprintf(buf," | %3d  %3d\n",conf[i][Nclasses],conf[i][Nclasses+1]);
		write_subwindow(buf);
        }
        sprintf(buf,"\n%3.2f%%\n\n",pros);
	write_subwindow(buf);

        /* FREE MEMORY USED BY CONFUSION MATRIX */
        for(i=0;i<Nclasses+2;i++)
                free(conf[i]);
        free(conf);
}

/* WRITE_SUBWINDOW(BUF)							*/
/* WRITES TEXT INTO SUBWINDOW						*/
/* PARAMETERS:								*/
/*		BUF		POINTER TO TEXT BUFFER			*/
/* RETURNS:								*/
/*              NOTHING							*/
void write_subwindow(buf)
char *buf;
{
	int len;
	len = strlen(buf);
	if(textsw_insert(textsw,buf,len)!=len)
		fprintf(stderr,"write_subwindow: Can't write all into subwindow\n");
}

/* MARK_CLASS(NUMBER,CLASSED,CLASS,LIST)				*/
/* CHANGES CLASS AND CLASSED FIELDS OF SPEC STRUCTURE			*/
/* PARAMETERS:								*/
/*		NUMBER		NUMBER OF SPECTRUM FROM THE BEGINNING OF*/
/*				LIST					*/
/*		CLASSED		CLASSIFICATION RESULT			*/
/*		CLASS		CORRECT CLASS				*/
/*		LIST		POINTER TO THE START OF A CORRECT LIST	*/
/* RETURNS:								*/
/*              NOTHING							*/
void mark_class(number,classed,class,list)
int number, classed, class, list;
{
	Spec *next;
	int counter = 0;

	/* SELECT RIGHT LIST */
	if(list==1)
		next = start;
	else 
		next = lstart;

	/* IF EMPTY THEN RETURN */
	if(next==NULL)
		return;

	/* FIND RIGHT SPECTRUM FROM THE LIST */
        while(next) {
		if(counter==number) {
			/* CHANGE VALUES */
			next->class = class;
			next->classed = classed;
			return;
		}
                next = next->next;
		counter++;
        }
}

/* SHOW_TIME(WHO)							*/
/* READS TIME USED BY A PROCESS						*/
/* PARAMETERS:								*/
/*		WHO		PROCESS NUMBER				*/
/* RETURNS:								*/
/*              USED TIME IN SECONDS					*/
int show_time(who)
int who;
{
	struct rusage rage;
	int result;

	if(getrusage(who, &rage)==-1) {
		fprintf(stderr, "getrusage: No process info");
		return;
	}
	result = (int)rage.ru_utime.tv_sec-timem;
	timem = (int)rage.ru_utime.tv_sec;

	return(result);
}
