/***********************************************************************
*                                                                      *
*                              SPECLASS                                *
*                                                                      *
*                                The                                   *
*                                                                      *
*                      Color Spectra Classifier                        *
*                                                                      *
*                          Program  Package                            *
*                                                                      *
*                           Programmed by			       *
*								       *
*			     J. Haanpalo		               *
*                                                                      *
*                      Copyright (c) 1993-1995                         *
*                                                                      *
************************************************************************
*                                                                      *
*  NOTE: This program package is copyrighted in the sense that it      *
*  may be used for scientific purposes. The package as a whole, or     *
*  parts thereof, cannot be included or used in any commercial         *
*  application without written permission granted by its producents.   *
*  No programs contained in this package may be copied for commercial  *
*  distribution. This program package comes 'as it is' and with no     *
*  warranty.                                                           *
*                                                                      *
*  All comments concerning this program package may be sent to the     *
*  e-mail address 'haanpalo@lut.fi'.                                   *
*                                                                      *
***********************************************************************/

#include<stdio.h>
#include<errno.h>
#include<sys/unistd.h>
#include<strings.h>
#include<stddef.h>
#include<malloc.h>
#include<math.h>
#include<values.h>
#ifdef GCC
#include<float.h>
#endif
#include"global.h"

#define NUL "\0"
#define MxGetM(mat) ((mat->rows))
#define MxGetN(mat) ((mat->cols))
#define MxGetPr(mat) ((mat->data))
/* #define DEBUG /* */
/* #define CHECK_ERR		/* */

typedef struct {
  long type;
  long mrows;
  long ncols;
  long imagf;
  long namlen;
} Fmatrix;

extern double drand48();

/* ERRMSG(S)							*/
/* DISPLAYS ERROR MESSAGE POINTED BY STRING S AND QUITS PROGRAM	*/
void errmsg(s)
char *s;
{
  fprintf(stderr, "Error:: %s\n", s);
  exit(1);
}

/* MXFREEMATRIX(MAT)						*/
/* FREES MATRISES DATA AND NAME					*/
/* PARAMETERS:							*/
/*		MAT	OWN MATRIX				*/
/* RETURNS:							*/
/*		NUTHING						*/
void MxFreeMatrix(mat)
MMatrix *mat;
{
  register i;

  if (mat == NULL)
    return;

  for (i = 0; i < MxGetN(mat); i++)
    free(mat->data[i]);

  free(mat->data);
  mat->data = NULL;

  if (mat->name != NULL) {
    free(mat->name);
    mat->name = NULL;
  }
  return;
}

/* MXFREE(MAT)							*/
/* FREES OWN MATRIX DATA STRUCTURE				*/
/* PARAMETERS:							*/
/*		MAT	OWN MATRIX				*/
/* RETURNS:							*/
/*		NUTHING						*/
void MxFree(mat)
MMatrix *mat;
{
  if (mat == NULL)
    return;

  free(mat);
  mat = NULL;

  return;
}

/* MXCREATEFULL(ROWS, COLS, TYPE)				*/
/* INITIALIZES MATRIX						*/
/* PARAMETERS:							*/
/*		ROWS	NUMBER OF ROWS				*/
/*		COLS	NUMBER OF COLUMNS			*/
/*		TYPE	TYPE OF DATA (0 REAL   5 TEXT)		*/
/* RETURNS:							*/
/*		POINTER TO A CREATED MATRIX			*/
MMatrix *MxCreateFull(rows, cols, type)
int rows, cols, type;
{
  MMatrix *result = NULL;
  unsigned char **data;
  register i;
  double **p;

  result = (MMatrix *) malloc(sizeof(MMatrix));
  if (result == NULL)
    errmsg("MxCreateFull: Memory allocation failed for struct Matrix");

  /* SET MATRIX DIMENSIONS AND NAME */
  result->rows = rows;
  result->cols = cols;
  result->type = type;
  result->name = NULL;

  /* ALLOCATE MEMORY FOR A DOUBLE MATRIX */
  result->data = (double **) malloc(cols * sizeof(result->data));
  p = result->data;

  for (i = 0; i < cols; i++)
    *p++ = (double *) malloc(rows * sizeof(double));

  if (result->data == NULL)
    errmsg("MxCreateFull: Memory allocation failed for data");

  return result;
}

/* MXGETNAME(MAT)						*/
/* INITIALIZES MATRIX						*/
/* PARAMETERS:							*/
/*		ROWS	NUMBER OF ROWS				*/
/*		COLS	NUMBER OF COLUMNS			*/
/*		REAL	TYPE OF DATA (NOT USED IN THIS VERSION)	*/
/* RETURNS:							*/
/*		POINTER TO A CREATED MATRIX			*/
char *MxGetName(mat)
MMatrix *mat;
{
  if ((mat == NULL) || (mat->name == NULL))
    return NULL;
  else
    return mat->name;
}

/* MPRINTMAT(MAT) PRINTS MATRIX GIVEN AS A PARAMETER (USED FOR DEBUGGING) */
void Mprintmat(mat)
MMatrix *mat;
{
  register i, j;
  char *name;
  for (i = 0; i < MxGetM(mat); i++) {
    for (j = 0; j < MxGetN(mat); j++) {
      printf("%f ", MxGetPr(mat)[j][i]);
    }
    printf("\n");
  }
  printf("\n");
  if (MxGetName(mat) != NULL)
    printf("%s rows = %d cols = %d\n\n", MxGetName(mat), MxGetM(mat), MxGetN(mat));
  else
    printf("NULL rows = %d cols = %d\n\n", MxGetM(mat), MxGetN(mat));

}

/* MXSETNAME(ROWS, COLS, REAL)					*/
/* INITIALIZES MATRIX						*/
/* PARAMETERS:							*/
/*		ROWS	NUMBER OF ROWS				*/
/*		COLS	NUMBER OF COLUMNS			*/
/*		REAL	TYPE OF DATA (NOT USED IN THIS VERSION)	*/
/* RETURNS:							*/
/*		POINTER TO A CREATED MATRIX			*/
void MxSetName(mat, name)
MMatrix *mat;
char *name;
{

#ifdef CHECK_ERR
  if (mat == NULL)
    errmsg("MxSetName: NULL input matrix");
#endif

  if (MxGetName(mat) != NULL)
    free(MxGetName(mat));
  if (name == NULL) {
    mat->name = NULL;
    return;
  }
  /* SET MATRIX NAME */
  mat->name = (char *) malloc((strlen(name) + 1) * sizeof(char));

#ifdef CHECK_ERR
  if (mat->name == NULL)
    errmsg("MxSetName: Memory allocation failed");
#endif

  strcpy(mat->name, name);
  return;
}

/* MXCHECKREUSE(ROWS, COLS, MAT)				*/
/* CHECKS IF MATRIX CAN BE REUSED 				*/
/* PARAMETERS:							*/
/*		ROWS	NUMBER OF ROWS				*/
/*		COLS	NUMBER OF COLUMNS			*/
/*		MAT	TEST MATRIX				*/
/* RETURNS:							*/
/*		POINTER TO TEST MATRIX IF IT CAN BE REUSED,	*/
/*		NULL IF NOT					*/
MMatrix *MxCheckReuse(rows, cols, mat)
int rows, cols;
MMatrix *mat;
{
  /* CHECK MATRIX STRUCTURE	 */
  if ((mat != NULL) && (MxGetM(mat) == rows) && (MxGetN(mat) == cols))
    return mat;
  else {
    MxFreeMatrix(mat);
    MxFree(mat);
    return NULL;
  }
}

/* MXCHECKTEMP(MAT)						*/
/* CHECKS IF MATRIX IS A TEMPORARY RESULT WHICH CAN BE DESTOYED	*/
/* PARAMETERS:							*/
/*		MAT	TEST MATRIX				*/
/* RETURNS:							*/
/*		NOTHING						*/
void MxCheckTemp(mat)
MMatrix *mat;
{
  if (mat == NULL)
    return;

  if ((MxGetName(mat) == NULL) && (MxGetPr(mat) != NULL)) {
    MxFreeMatrix(mat);
    MxFree(mat);
  }
}

/* MXGETSCALAR(MAT)						*/
/* GETS THE FIRST ELEMENT FROM DOUBLE MATRIX			*/
/* PARAMETERS:							*/
/*		MAT	DOUBLE MATRIX				*/
/* RETURNS:							*/
/*		DOUBLE VALUE					*/
double MxGetScalar(mat)
MMatrix *mat;
{
  double result;

#ifdef CHECK_ERR
  if (mat == NULL)
    errmsg("MxGetScalar: NULL input matrix");
#endif

  result = MxGetPr(mat)[0][0];

  MxFreeMatrix(mat);
  MxFree(mat);

  return result;
}

/*
MMatrix *matlab2own(mat)
Matrix *mat;
{
	MMatrix *result;
	register i, j;
	int rows, cols;
	double *pr;

	rows = mxGetM(mat);
	cols = mxGetN(mat);

	result = MxCreateFull(rows, cols, 0);

	pr = mxGetPr(mat);

	for(j=0;j<cols;j++)
		for(i=0;i<rows;i++)
			MxGetPr(result)[j][i] = *pr++;

	MxSetName(result, mxGetName(mat));

#ifdef DEBUG
	printf("matlab2own r=%d c=%d\n",mxGetM(mat),mxGetN(mat));
	printf("matlab2own r=%d c=%d\n",MxGetM(result),MxGetN(result));
	Mprintmat(result);
#endif

	mxFreeMatrix(mat);
	mxFree(mat);
	mat = NULL;

	return result;
}

Matrix *own2matlab(mat)
MMatrix *mat;
{
	Matrix *result;
	register i, j;
	int rows, cols;
	double *pr;

	rows = MxGetM(mat);
	cols = MxGetN(mat);

	result = mxCreateFull(rows, cols, REAL);

	pr = mxGetPr(result);

	for(j=0;j<cols;j++)
		for(i=0;i<rows;i++)
			*pr++ = MxGetPr(mat)[j][i];

	mxSetName(result, MxGetName(mat));

#ifdef DEBUG
	printf("own2matlab x=%d y=%d\n",MxGetM(mat),MxGetM(mat));
	printf("own2matlab x=%d y=%d\n",mxGetM(result),mxGetM(result));
#endif

	MxFreeMatrix(mat);
	MxFree(mat);

	return result;
}
*/

/* MMATINIT(ROWS,COLS,VALUE,S,OLDMATRIX)			*/
/* INITIALIZES ROWSxCOLS OWN MATRIX WITH VALUE			*/
/* PARAMETERS:							*/
/*		ROWS 		NUMBER OF ROWS IN MATRIX	*/
/*		COLS 		NUMBER OF COLS IN MATRIX	*/
/*		VALUE		INITIAL VALUE FOR MATRIX	*/
/*		S		NAME STRING FOR MATRIX		*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A MATRIX				*/
MMatrix *Mmatinit(rows, cols, value, s, oldmatrix)
MMatrix *oldmatrix;
int rows, cols;
double value;
char *s;
{
  MMatrix *result;
  register i, j;
  double *p;

  /* CHECK OLD MATRIX STRUCTURE	 */
  result = MxCheckReuse(rows, cols, oldmatrix);
  if (result == NULL)
    result = MxCreateFull(rows, cols, 0);

  /* INITIALIZE MATRIX WITH VALUE	 */
  for (j = 0; j < cols; j++) {
    p = MxGetPr(result)[j];
    for (i = 0; i < rows; i++)
      *p++ = value;
  }
  MxSetName(result, s);

#ifdef DEBUG
  Mprintmat(result);
  printf("Mmatinit x=%d y=%d\n", MxGetM(result), MxGetN(result));
#endif
  return result;
}

/* MATPUTMATRIX(FP, MAT)					*/
/* PUTS A MATRIX MAT INTO A FILE POINTED BY FILE POINTER FP	*/
/* PARAMETERS:							*/
/*		FP	FILE POINTER FROM FOPEN			*/
/*		MAT	DOUBLE MATRIX				*/
/* RETURNS:							*/
/*		INT VALUE 0 SUCCESS 1 FAIL			*/
int MatPutMatrix(fp, mat)
FILE *fp;
MMatrix *mat;
{
  Fmatrix head;
  register i;

  /* FILL MATLAB FILE HEADER STRUCT */
  head.type = (long) 1000;	/* SPARC machine, double precision, numeric,
				 * other */
  head.mrows = (long) MxGetM(mat);
  head.ncols = (long) MxGetN(mat);
  head.imagf = (long) 0;
  head.namlen = (long) (strlen(MxGetName(mat)) + 1);

  /* FIND THE END OF FILE */
  if (fseek(fp, (long int) 0, SEEK_END) == -1) {
    perror("fseek");
    return 1;
  }
  /* WRITE HEADER STRUCT INTO FILE */
  fwrite(&head, sizeof(Fmatrix), 1, fp);

  /* WRITE MATRIX NAME INTO FILE */
  fwrite(MxGetName(mat), sizeof(char), (int) head.namlen, fp);

  /* WRITE MATRIX DATA INTO FILE */
  for (i = 0; i < MxGetN(mat); i++)
    fwrite(MxGetPr(mat)[i], sizeof(double), head.mrows, fp);

  return 0;
}

/* MATGETMATRIX(FP, NAME)						*/
/* GETS A MATRIX NAMED NAME FROM A FILE POINTED BY FILE POINTER FP	*/
/* PARAMETERS:								*/
/*		FP	FILE POINTER FROM FOPEN				*/
/*		NAME	MATRIXNAME					*/
/* RETURNS:								*/
/*		POINTER TO A MATRIX					*/
MMatrix *MatGetMatrix(fp, name)
FILE *fp;
char *name;
{
  Fmatrix head;
  MMatrix *result;
  register i;
  char matname[50];

  while (1) {

    /* READ HEADER STRUCT FROM FILE */
    if (fread(&head, sizeof(Fmatrix), 1, fp) != 1)
      return NULL;

    /* READ MATRIX NAME FROM FILE */
    if (fread(matname, sizeof(char), (int) head.namlen, fp) != (int) head.namlen)
      errmsg("MatGetMatrix: Cannot read matrix name from file");

    if ((name == NULL) || (strcmp(name, matname) == 0)) {	/* MATRIX FOUND */

      switch ((int) head.type) {
      case 1100:
	result = MxCreateFull((int) head.ncols, (int) head.mrows, 0);

	/* READ MATRIX DATA FROM FILE */
	for (i = 0; i < (int) head.mrows; i++)
	  if ((fread(MxGetPr(result)[i], sizeof(double), (int) head.ncols,
		     fp)) != (int) head.ncols)
	    errmsg("MatGetMatrix: Cannot read matrix (transp) from file");
	MxSetName(result, matname);
	result = Mtransp(result, result);
	break;

      default:			/* DOUBLE MATRIX */
	result = MxCreateFull((int) head.mrows, (int) head.ncols, 0);

	/* READ MATRIX DATA FROM FILE */
	for (i = 0; i < (int) head.ncols; i++)
	  if ((fread(MxGetPr(result)[i], sizeof(double),
		     (int) head.mrows, fp)) != (int) head.mrows)
	    errmsg("MatGetMatrix: Cannot read matrix from file");
	MxSetName(result, matname);
	break;
      }

      /* END OF MATRIX SEARCH */
      return result;
    } else
      /* FIND NEXT MATRIX FROM FILE */
    if (fseek(fp, (int) head.ncols * (int) head.mrows * sizeof(double), SEEK_CUR) == -1) {
      perror("fseek");
      return NULL;
    }
  }
}

/* MATPUTSTRING(FP, STRBUF, ROWS, COLS, NAME)				*/
/* PUTS A STRING INTO A MATRIX AND STORES THE MATRIX INTO A FILE	*/
/* PARAMETERS:								*/
/*		FP	FILE POINTER FROM FOPEN				*/
/*		STRBUF	POINTER TO THE START OF A STRING		*/
/*		ROWS	NUMBER OF ROWS					*/
/*		COLS	NUMBER OF COLS					*/
/*		NAME	MATRIXNAME					*/
/* RETURNS:								*/
/*		INT VALUE 1 SUCCES 0 FAIL				*/
int MatPutString(fp, strbuf, rows, cols, name)
FILE *fp;
char *strbuf;
int rows, cols;
char *name;
{
  MMatrix *result;
  Fmatrix head;
  register i;
  double d;

  /* FILL MATLAB FILE HEADER STRUCT */
  head.type = (long) 1001;	/* SPARC machine, double precision, text,
				 * other */
  head.mrows = (long) rows;
  head.ncols = (long) cols;
  head.imagf = (long) 0;
  head.namlen = (long) (strlen(name) + 1);

  /* FIND THE END OF FILE */
  if (fseek(fp, (long int) 0, SEEK_END) == -1) {
    perror("fseek");
    return 0;
  }
  /* WRITE HEADER STRUCT INTO FILE */
  if (fwrite(&head, sizeof(Fmatrix), 1, fp) != 1) {
    perror("fwrite");
    return 0;
  }
  /* WRITE MATRIX NAME INTO FILE */
  if (fwrite(name, sizeof(char), (int) head.namlen, fp) != (int) head.namlen) {
    perror("fwrite");
    return 0;
  }
  /* WRITE MATRIX DATA INTO FILE */
  for (i = 0; i < rows * cols; i++) {
    d = (double) strbuf[i];
    if (fwrite((char *) (&d), sizeof(double), 1, fp) != 1) {
      perror("fwrite");
      return 0;
    }
  }
  return 1;
}

/* MATGETSTRING(MAT, STRBUF)						*/
/* GETS A STRING FROM THE MATRIX MAT AND STORES IT INTO STRBUF		*/
/* PARAMETERS:								*/
/*		MAT	FILE POINTER FROM FOPEN				*/
/*		STRBUF	POINTER TO THE START OF THE STRING BUFFER	*/
/* RETURNS:								*/
/*		INT VALUE 1 SUCCES					*/
int MatGetString(mat, strbuf)
MMatrix *mat;
char *strbuf;
{
  register i, j;

  for (i = 0; i < MxGetN(mat); i++)
    for (j = 0; j < MxGetM(mat); j++)
      *strbuf++ = (char) MxGetPr(mat)[i][j];

  return 1;
}

/* MMATCOPY(MAT,OLDMATRIX)					*/
/* COPIES MATRIX MAT					 	*/
/* PARAMETERS:							*/
/*		MAT 		MATRIX			 	*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A MATRIX				*/
MMatrix *Mmatcopy(mat, oldmatrix)
MMatrix *mat, *oldmatrix;
{
  MMatrix *result;
  register i;
  int rows, cols;

#ifdef CHECK_ERR
  if (mat == NULL)
    errmsg("Mmatcopy: NULL input matrix");
  if (MxGetPr(mat) == NULL)
    errmsg("Mmatcopy: NULL data pointer");
#endif

  rows = MxGetM(mat);
  cols = MxGetN(mat);

  result = MxCreateFull(rows, cols, 0);

  for (i = 0; i < cols; i++)
    memcpy(MxGetPr(result)[i], MxGetPr(mat)[i], rows * sizeof(double));

  if (oldmatrix != NULL) {
    MxSetName(result, MxGetName(oldmatrix));
    MxFreeMatrix(oldmatrix);
    MxFree(oldmatrix);
  }
#ifdef DEBUG
  Mprintmat(result);
  printf("Mmatcopy x=%d y=%d\n\n", MxGetM(result), MxGetN(result));
#endif
  return result;
}

/* MMATDIAG(SIZE,S,OLDMATRIX)					*/
/* CREATES SIZExSIZE DIAGONAL MATRIX				*/
/* PARAMETERS:							*/
/*		SIZE 		NUMBER OF ROWS AND COLS		*/
/*		S		NAME STRING FOR MATRIX		*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A MATRIX				*/
MMatrix *Mmatdiag(size, s, oldmatrix)
MMatrix *oldmatrix;
int size;
char *s;
{
  MMatrix *result;
  register i;
  double *pr;

  /* CHECK OLD MATRIX STRUCTURE	 */
  result = MxCheckReuse(size, size, oldmatrix);
  if (result == NULL)
    result = MxCreateFull(size, size, 0);

  for (i = 0; i < size; i++)
    MxGetPr(result)[i][i] = 1;

#ifdef DEBUG
  printf("Mmatdiag x=%d y=%d\n", MxGetM(result), MxGetN(result));
#endif
  return result;
}

/* MTRANSP(MAT,OLDMATRIX)					*/
/* CREATES TRANSPOSE OF A MATRIX				*/
/* PARAMETERS:							*/
/*		MAT 		MATRIX WHICH IS GOING TO BE 	*/
/*				TRANSPOSED			*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A MATRIX				*/
MMatrix *Mtransp(mat, oldmatrix)
MMatrix *mat, *oldmatrix;
{
  MMatrix *result;
  double *p;
  register i, j;

  result = MxCreateFull(MxGetN(mat), MxGetM(mat), 0);

  for (i = 0; i < MxGetN(mat); i++) {
    p = MxGetPr(mat)[i];
    for (j = 0; j < MxGetM(mat); j++)
      MxGetPr(result)[j][i] = *p++;
  }

#ifdef DEBUG
  printf("Mtransp x=%d y=%d\n", MxGetM(mat), MxGetN(mat));
  Mprintmat(result);
  printf("Mtransp x=%d y=%d\n\n", MxGetM(result), MxGetN(result));
#endif

  if (oldmatrix != NULL) {
    MxSetName(result, MxGetName(oldmatrix));
    MxFreeMatrix(oldmatrix);
    MxFree(oldmatrix);
  }
  /* FREE mat IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat);

  return result;
}

/* MMATADD(MAT1,MAT2,OLDMATRIX)					*/
/* ADDS TWO MATRICES 						*/
/* PARAMETERS:							*/
/*		MAT1 		MATRIX			 	*/
/*		MAT2 		MATRIX			 	*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A MATRIX				*/
MMatrix *Mmatadd(mat1, mat2, oldmatrix)
MMatrix *mat1, *mat2, *oldmatrix;
{
  MMatrix *result;
  double *p1, *p2, *p3;
  register i, j;

#ifdef CHECK_ERR
  if (mat1 == NULL)
    errmsg("Mmatadd: NULL input matrix");
  if (mat2 == NULL)
    errmsg("Mmatadd: NULL input matrix");
  if (MxGetM(mat1) != MxGetM(mat2))
    errmsg("Mmatadd: input matrix dimension");
  if (MxGetN(mat1) != MxGetN(mat2))
    errmsg("Mmatadd: input matrix dimension");
  if ((MxGetPr(mat1) == NULL) || (MxGetPr(mat2) == NULL))
    errmsg("Mmatadd: NULL data pointer");
#endif

  result = MxCreateFull(MxGetM(mat1), MxGetN(mat2), 0);

  for (j = 0; j < MxGetN(mat1); j++) {
    p1 = MxGetPr(result)[j];
    p2 = MxGetPr(mat1)[j];
    p3 = MxGetPr(mat2)[j];
    for (i = 0; i < MxGetM(mat1); i++)
      *p1++ = *p2++ + *p3++;
  }

  if (oldmatrix != NULL) {
    MxSetName(result, MxGetName(oldmatrix));
    MxFreeMatrix(oldmatrix);
    MxFree(oldmatrix);
  }
#ifdef DEBUG
  printf("Mmatadd mat1 x=%d y=%d\n", MxGetM(mat1), MxGetN(mat1));
  printf("Mmatadd mat2 x=%d y=%d\n", MxGetM(mat2), MxGetN(mat2));
  Mprintmat(result);
#endif

  /* FREE mat1 IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat1);

  /* FREE mat2 IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat2);

  return result;
}

/* MMATSUB(MAT1,MAT2,OLDMATRIX)					*/
/* SUBSTRACTS MATRIX MAT2 FROM MATRIX MAT1 TWO MATRICES 	*/
/* PARAMETERS:							*/
/*		MAT1 		MATRIX 1		 	*/
/*		MAT2 		MATRIX 2			*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A MATRIX				*/
MMatrix *Mmatsub(mat1, mat2, oldmatrix)
MMatrix *mat1, *mat2, *oldmatrix;
{
  MMatrix *result;
  register i, j;
  double *p1, *p2, *p3;

#ifdef CHECK_ERR
  if (mat1 == NULL)
    errmsg("Mmatsub: NULL input matrix");
  if (mat2 == NULL)
    errmsg("Mmatsub: NULL input matrix");
  if (MxGetM(mat1) != MxGetM(mat2))
    errmsg("Mmatsub: input matrix dimension");
  if (MxGetN(mat1) != MxGetN(mat2))
    errmsg("Mmatsub: input matrix dimension");
  if ((MxGetPr(mat1) == NULL) || (MxGetPr(mat2) == NULL))
    errmsg("Mmatsub: NULL data pointer");
#endif

  result = MxCreateFull(MxGetM(mat1), MxGetN(mat2), 0);

  for (j = 0; j < MxGetN(mat1); j++) {
    p1 = MxGetPr(result)[j];
    p2 = MxGetPr(mat1)[j];
    p3 = MxGetPr(mat2)[j];
    for (i = 0; i < MxGetM(mat1); i++)
      *p1++ = *p2++ - *p3++;
  }
  if (oldmatrix != NULL) {
    MxSetName(result, MxGetName(oldmatrix));
    MxFreeMatrix(oldmatrix);
    MxFree(oldmatrix);
  }
#ifdef DEBUG
  printf("Mmatsub mat1 x=%d y=%d\n", MxGetM(mat1), MxGetN(mat1));
  printf("Mmatsub mat2 x=%d y=%d\n", MxGetM(mat2), MxGetN(mat2));
  Mprintmat(result);
#endif

  /* FREE mat1 IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat1);

  /* FREE mat2 IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat2);

  return result;
}

/* MMATMUL(MAT1,MAT2,OLDMATRIX)					*/
/* MULTIPLIES MATRIX MAT1 WITH MATRIX MAT2		 	*/
/* PARAMETERS:							*/
/*		MAT1 		MATRIX 1		 	*/
/*		MAT2 		MATRIX 2			*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A MATRIX				*/
MMatrix *Mmatmul(mat1, mat2, oldmatrix)
MMatrix *mat1, *mat2, *oldmatrix;
{
  MMatrix *result;
  double s, *p2;
  register i, j, k;

#ifdef CHECK_ERR
  if (mat1 == NULL)
    errmsg("Mmatmul: NULL input matrix");
  if (mat2 == NULL)
    errmsg("Mmatmul: NULL input matrix");
  if (MxGetN(mat1) != MxGetM(mat2))
    errmsg("Mmatmul: input matrix dimension");
  if ((MxGetPr(mat1) == NULL) || (MxGetPr(mat2) == NULL))
    errmsg("Mmatmul: NULL data pointer");
#endif

  result = MxCreateFull(MxGetM(mat1), MxGetN(mat2), 0);

  for (i = 0; i < MxGetM(mat1); i++) {
    for (j = 0; j < MxGetN(mat2); j++) {
      s = 0.0;
      p2 = MxGetPr(mat2)[j];
      for (k = 0; k < MxGetN(mat1); k++) {
	s += MxGetPr(mat1)[k][i] * *p2++;
      }
      MxGetPr(result)[j][i] = s;
    }
  }

#ifdef DEBUG
  Mprintmat(mat1);
  printf("matmul mat1 x=%d y=%d\n", MxGetM(mat1), MxGetN(mat1));
  Mprintmat(mat2);
  printf("matmul mat2 x=%d y=%d\n", MxGetM(mat2), MxGetN(mat2));
  Mprintmat(result);
#endif

  if (oldmatrix != NULL) {
    MxSetName(result, MxGetName(oldmatrix));
    MxFreeMatrix(oldmatrix);
    MxFree(oldmatrix);
  }
  /* FREE mat1 IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat1);

  /* FREE mat2 IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat2);

  return result;
}

/* MSCALMUL(SCALAR,MAT,OLDMATRIX)				*/
/* MULTIPLIES MATRIX MAT WITH SCALAR			 	*/
/* PARAMETERS:							*/
/*		SCALAR		SCALAR MULTIPLIER		*/
/*		MAT 		MATRIX			 	*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A MATRIX				*/
MMatrix *Mscalmul(scalar, mat, oldmatrix)
double scalar;
MMatrix *mat, *oldmatrix;
{
  MMatrix *result;
  register i, j;
  double *p1, *p2;

#ifdef CHECK_ERR
  if (mat == NULL)
    errmsg("Mscalmul: NULL input matrix");
  if (MxGetPr(mat) == NULL)
    errmsg("Mscalmul: NULL data pointer");
#endif

  result = MxCreateFull(MxGetM(mat), MxGetN(mat), 0);

  for (i = 0; i < MxGetN(mat); i++) {
    p1 = MxGetPr(result)[i];
    p2 = MxGetPr(mat)[i];
    for (j = 0; j < MxGetM(mat); j++)
      *p1++ = scalar * *p2++;
  }
#ifdef DEBUG
  printf("scalmul input\n");
  Mprintmat(mat);
  printf("scalmul output\n");
  Mprintmat(result);
#endif

  if (oldmatrix != NULL) {
    MxSetName(result, MxGetName(oldmatrix));
    MxFreeMatrix(oldmatrix);
    MxFree(oldmatrix);
  }
  /* FREE mat IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat);

  return result;
}

/* MCOLMUL(SCALAR,MAT,COL,OLDMATRIX)				*/
/* MULTIPLIES MATRIX COLUMN WITH SCALAR			 	*/
/* PARAMETERS:							*/
/*		SCALAR		SCALAR MULTIPLIER		*/
/*		MAT 		MATRIX			 	*/
/*		COL 		MATRIX COLUMN WHICH WILL BE 	*/
/*				MULTIPLIED			*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A MATRIX				*/
MMatrix *Mcolmul(scalar, mat, col, oldmatrix)
double scalar;
int col;
MMatrix *mat, *oldmatrix;
{
  MMatrix *result;
  register j;
  double *p, tmp;

#ifdef CHECK_ERR
  if (mat == NULL)
    errmsg("Mcolmul: NULL input matrix");
  if (col >= MxGetN(mat))
    errmsg("Mcolmul: no such column");
  if (MxGetPr(mat) == NULL)
    errmsg("Mcolmul: NULL data pointer");
#endif

  result = Mmatcopy(mat, NULL);
  if (result == NULL)
    errmsg("Mcolmul: NULL output matrix");

  p = MxGetPr(result)[col];

  for (j = 0; j < MxGetM(mat); j++) {
    tmp = *p;
    *p++ = scalar * tmp;
  }

  if (oldmatrix != NULL) {
    MxSetName(result, MxGetName(oldmatrix));
    MxFreeMatrix(oldmatrix);
    MxFree(oldmatrix);
  }
#ifdef DEBUG
  printf("colmul input\n");
  Mprintmat(mat);
  printf("colmul output x=%d y=%d col=%d\n\n", MxGetM(result), MxGetN(result), col);
  Mprintmat(result);
#endif

  /* FREE mat IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat);

  return result;
}

/* MCOLADD(MAT1,MAT2,COL,OLDMATRIX)				*/
/* ADDS COLUMN VECTOR MAT2 TO COLUMN COL OF MATRIX MAT1		*/
/* PARAMETERS:							*/
/*		MAT1 		MATRIX 			 	*/
/*		MAT2 		VECTOR			 	*/
/*		COL 		MATRIX COLUMN WHICH WILL BE 	*/
/*				USED				*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A MATRIX				*/
MMatrix *Mcoladd(mat1, mat2, col, oldmatrix)
int col;
MMatrix *mat1, *mat2, *oldmatrix;
{
  MMatrix *result;
  register j;
  double *p1, *p2, tmp;

#ifdef CHECK_ERR
  if (mat1 == NULL)
    errmsg("Mcoladd: NULL input matrix");
  if (mat2 == NULL)
    errmsg("Mcoladd: NULL input matrix");
  if (MxGetM(mat1) != MxGetM(mat2))
    errmsg("Mcoladd: input matrix dimension");
  if ((MxGetPr(mat1) == NULL) || (MxGetPr(mat2) == NULL))
    errmsg("Mcoladd: NULL data pointer");
#endif

  result = Mmatcopy(mat1, NULL);

  p1 = MxGetPr(result)[col];
  p2 = MxGetPr(mat2)[0];

  for (j = 0; j < MxGetM(mat2); j++) {
    tmp = *p1;
    *p1++ = tmp + *p2++;
  }

  if (oldmatrix != NULL) {
    MxSetName(result, MxGetName(oldmatrix));
    MxFreeMatrix(oldmatrix);
    MxFree(oldmatrix);
  }
#ifdef DEBUG
  printf("coladd input\n");
  Mprintmat(mat1);
  Mprintmat(mat2);
  printf("coladd output x=%d y=%d col=%d\n", MxGetM(result), MxGetN(result), col);
  Mprintmat(result);
#endif

  /* FREE mat1 IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat1);

  /* FREE mat2 IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat2);

  return result;
}

/* MCOLSUB(MAT1,COL1,MAT2,COL2,OLDMATRIX)			*/
/* SUBSTRACTS COLUMN COL2 OF MATRIX MAT2 FROM COLUMN COL1 OF	*/
/* MATRIX MAT1							*/
/* PARAMETERS:							*/
/*		MAT1 		MATRIX 1		 	*/
/*		COL1 		COLUMN OF MATRIX 1	 	*/
/*		MAT2		MATRIX 2		 	*/
/*		COL2 		COLUMN OF MATRIX 2	 	*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A MATRIX				*/
MMatrix *Mcolsub(mat1, col1, mat2, col2, oldmatrix)
int col1, col2;
MMatrix *mat1, *mat2, *oldmatrix;
{
  MMatrix *result;
  double *p1, *p2, tmp;
  register j;

#ifdef CHECK_ERR
  if (mat1 == NULL)
    errmsg("Mcolsub: NULL input matrix");
  if (mat2 == NULL)
    errmsg("Mcolsub: NULL input matrix");
  if (MxGetM(mat1) != MxGetM(mat2))
    errmsg("Mcolsub: input matrix dimension");
  if ((MxGetPr(mat1) == NULL) || (MxGetPr(mat2) == NULL))
    errmsg("Mcolsub: NULL data pointer");
#endif

  result = Mmatcopy(mat1, NULL);

  p1 = MxGetPr(result)[col1];
  p2 = MxGetPr(mat2)[col2];

  for (j = 0; j < MxGetM(mat2); j++) {
    tmp = *p1;
    *p1++ = tmp + *p2++;
  }

  if (oldmatrix != NULL) {
    MxSetName(result, MxGetName(oldmatrix));
    MxFreeMatrix(oldmatrix);
    MxFree(oldmatrix);
  }
#ifdef DEBUG
  printf("Mcolsub input col1=%d col2=%d\n", col1, col2);
  Mprintmat(mat1);
  Mprintmat(mat2);
  printf("Mcolsub output\n\n");
  Mprintmat(result);
#endif

  /* FREE mat1 IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat1);

  /* FREE mat2 IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat2);

  return result;
}

/* MVECMUL(SCALAR,MAT,COL,OLDMATRIX)				*/
/* MULTIPLIES COLUMN COL OF MATRIX MAT WITH SCALAR		*/
/* PARAMETERS:							*/
/*		SCALAR		MULTIPLIER			*/
/*		MAT 		MATRIX			 	*/
/*		COL 		COLUMN OF MATRIX	 	*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A COLUMN VECTOR			*/
MMatrix *Mvecmul(scalar, mat, col, oldmatrix)
double scalar;
MMatrix *mat, *oldmatrix;
int col;
{
  MMatrix *result;
  register j;
  double *p1, *p2;

#ifdef CHECK_ERR
  if (mat == NULL)
    errmsg("Mvecmul: NULL input matrix");
  if (MxGetPr(mat) == NULL)
    errmsg("Mvecmul: NULL data pointer");
#endif

  result = MxCreateFull(MxGetM(mat), 1, 0);

  p1 = MxGetPr(result)[0];
  p2 = MxGetPr(mat)[col];

  for (j = 0; j < MxGetM(mat); j++)
    *p1++ = scalar * *p2++;

  if (oldmatrix != NULL) {
    MxSetName(result, MxGetName(oldmatrix));
    MxFreeMatrix(oldmatrix);
    MxFree(oldmatrix);
  }
#ifdef DEBUG
  printf("Mvecmul input\n");
  Mprintmat(mat);
  printf("Mvecmul output x=%d y=%d col=%d\n", MxGetM(result), MxGetN(result), col);
  Mprintmat(result);
#endif

  /* FREE mat IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat);

  return result;
}

/* MVECADD(MAT1,COL1,MAT2,COL2,OLDMATRIX)			*/
/* ADDS COLUMN COL2 OF MATRIX MAT2 TO COLUMN COL1 OF		*/
/* MATRIX MAT1							*/
/* PARAMETERS:							*/
/*		MAT1 		MATRIX 1		 	*/
/*		COL1 		COLUMN OF MATRIX 1	 	*/
/*		MAT2		MATRIX 2		 	*/
/*		COL2 		COLUMN OF MATRIX 2	 	*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A COLUMN VECTOR			*/
MMatrix *Mvecadd(mat1, col1, mat2, col2, oldmatrix)
int col1, col2;
MMatrix *mat1, *mat2, *oldmatrix;
{
  MMatrix *result;
  register j;
  double *p1, *p2, *p3;

#ifdef CHECK_ERR
  if (mat1 == NULL)
    errmsg("Mvecadd: NULL input matrix");
  if (mat2 == NULL)
    errmsg("Mvecadd: NULL input matrix");
  if (MxGetM(mat1) != MxGetM(mat2))
    errmsg("Mvecadd: input matrix dimension");
  if ((MxGetPr(mat1) == NULL) || (MxGetPr(mat2) == NULL))
    errmsg("Mvecadd: NULL data pointer");
#endif

  result = MxCreateFull(MxGetM(mat2), MxGetN(mat2), 0);

  p1 = MxGetPr(result)[0];
  p2 = MxGetPr(mat1)[col1];
  p3 = MxGetPr(mat2)[col2];

  for (j = 0; j < MxGetM(mat2); j++)
    *p1++ = *p2++ + *p3++;

  if (oldmatrix != NULL) {
    MxSetName(result, MxGetName(oldmatrix));
    MxFreeMatrix(oldmatrix);
    MxFree(oldmatrix);
  }
#ifdef DEBUG
  printf("Mvecadd input\n");
  /* Mprintmat(mat1); */
  Mprintmat(mat2);
  printf("Mvecadd output x=%d y=%d col1=%d col2=%d\n",
	 MxGetM(result), MxGetN(result), col1, col2);
  Mprintmat(result);
#endif

  /* FREE mat1 IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat1);

  /* FREE mat2 IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat2);

  return result;
}

/* MVECSUB(MAT1,COL1,MAT2,COL2,OLDMATRIX)			*/
/* SUBSTRACTS COLUMN COL2 OF MATRIX MAT2 FROM COLUMN COL1 OF	*/
/* MATRIX MAT1							*/
/* PARAMETERS:							*/
/*		MAT1 		MATRIX 1		 	*/
/*		COL1 		COLUMN OF MATRIX 1	 	*/
/*		MAT2		MATRIX 2		 	*/
/*		COL2 		COLUMN OF MATRIX 2	 	*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A COLUMN VECTOR			*/
MMatrix *Mvecsub(mat1, col1, mat2, col2, oldmatrix)
int col1, col2;
MMatrix *mat1, *mat2, *oldmatrix;
{
  MMatrix *result;
  register j;
  double *p1, *p2, *p3;

#ifdef CHECK_ERR
  if (mat1 == NULL)
    errmsg("Mvecsub: NULL input matrix");
  if (mat2 == NULL)
    errmsg("Mvecsub: NULL input matrix");
  if (MxGetM(mat1) != MxGetM(mat2))
    errmsg("Mvecsub: input matrix dimension");
  if ((MxGetPr(mat1) == NULL) || (MxGetPr(mat2) == NULL))
    errmsg("Mvecsub: NULL data pointer");
#endif

  result = MxCreateFull(MxGetM(mat1), 1, 0);

  p1 = MxGetPr(result)[0];
  p2 = MxGetPr(mat1)[col1];
  p3 = MxGetPr(mat2)[col2];

  for (j = 0; j < MxGetM(mat2); j++)
    *p1++ = *p2++ - *p3++;

  if (oldmatrix != NULL) {
    MxSetName(result, MxGetName(oldmatrix));
    MxFreeMatrix(oldmatrix);
    MxFree(oldmatrix);
  }
#ifdef DEBUG
  printf("Mvecsub input\n");
  Mprintmat(mat1);
  Mprintmat(mat2);
  printf("Mvecsub output x=%d y=%d col1=%d col2=%d\n",
	 MxGetM(result), MxGetN(result), col1, col2);
  Mprintmat(result);
#endif

  /* FREE mat1 IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat1);

  /* FREE mat2 IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat2);

  return result;
}


/* MCOLAPP(MAT1,MAT2)					*/
/* APPEND MATRIX MAT2 AT THE END OF MATRIX MAT1			*/
/* PARAMETERS:							*/
/*		MAT1 		MATRIX 1		 	*/
/*		MAT2 		MATRIX 2		 	*/
/* RETURNS:							*/
/*		POINTER TO A MATRIX				*/
MMatrix *Mcolapp(mat1, mat2)
MMatrix *mat1, *mat2;
{
  MMatrix *result = NULL;
  register i;
  int rows1 = 0, cols1 = 0, rows2 = 0, cols2 = 0;

#ifdef CHECK_ERR
  if (mat2 == NULL)
    errmsg("Mcolapp: NULL input matrix");
  if (MxGetPr(mat2) == NULL)
    errmsg("Mcolapp: NULL data pointer");
#endif

  rows2 = MxGetM(mat2);
  cols2 = MxGetN(mat2);

  if (mat1 != NULL) {
    rows1 = MxGetM(mat1);
    cols1 = MxGetN(mat1);
    result = MxCreateFull(rows1, cols1 + cols2, 0);

    for (i = 0; i < cols1; i++)
      memcpy(MxGetPr(result)[i], MxGetPr(mat1)[i], rows1 * sizeof(double));

    /* FREE mat1 IF IT IS A TEMPORARY RESULT	 */
    MxCheckTemp(mat1);
  } else {
    result = MxCreateFull(rows2, cols2, 0);
  }

  for (i = 0; i < cols2; i++)
    memcpy(MxGetPr(result)[cols1 + i], MxGetPr(mat2)[i], rows2 * sizeof(double));

  MxSetName(result, MxGetName(mat1));

#ifdef DEBUG
  printf("Mcolapp input rows1 %d cols1 %d rows2 %d cols2 %d\n",
	 rows1, cols1, rows2, cols2);
  Mprintmat(mat2);
  printf("Mcolapp output\n");
  Mprintmat(result);
#endif
  MxFreeMatrix(mat2);
  MxFree(mat2);

  return result;
}

/* MMATEXT(MAT,OFF,DIM,OLDMATRIX)				*/
/* EXTRACT SUBMATRIX FROM MATRIX MAT				*/
/* PARAMETERS:							*/
/*		MAT 		MATRIX			 	*/
/*		OFF		OFFSET FROM THE BEGINNING OF	*/
/*				MATRIX MAT			*/
/*		DIM		NUMBER OF COLUMNS WHICH ARE	*/
/*				GOING TO BE EXTRACTED		*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A MATRIX				*/
MMatrix *Mmatext(mat, off, dim, oldmatrix)
MMatrix *mat, *oldmatrix;
int off, dim;
{
  MMatrix *result;
  register i;
  int rows, cols;

#ifdef CHECK_ERR
  if (mat == NULL)
    errmsg("Mmatext: NULL input matrix");
  if (MxGetPr(mat) == NULL)
    errmsg("Mmatext: NULL data pointer");
#endif

  rows = MxGetM(mat);
  cols = MxGetN(mat);
  result = MxCreateFull(rows, dim, 0);

  /* copy right colums to new matrix	 */
  for (i = 0; i < dim; i++)
    memcpy(MxGetPr(result)[i], MxGetPr(mat)[off + i], rows * sizeof(double));

  if (oldmatrix != NULL) {
    MxSetName(result, MxGetName(oldmatrix));
    MxFreeMatrix(oldmatrix);
    MxFree(oldmatrix);
  }
#ifdef DEBUG
  printf("Mmatext output\n");
  Mprintmat(result);
  printf("Mmatext offset=%d dim=%d cols %d-%d\n\n", off, dim, dim * off, dim * off + dim - 1);
#endif

  return result;
}

/* MMATMERGE(MAT1,MAT2,OFF,OLDMATRIX)				*/
/* MERGE SUBMATRIX MAT2 INTO MATRIX MAT1			*/
/* PARAMETERS:							*/
/*		MAT1 		MATRIX 1		 	*/
/*		MAT2 		MATRIX 2		 	*/
/*		OFF		OFFSET FROM THE BEGINNING OF	*/
/*				MATRIX MAT			*/
/* RETURNS:							*/
/*		POINTER TO A MATRIX				*/
MMatrix *Mmatmerge(mat1, mat2, off)
MMatrix *mat1, *mat2;
int off;
{
  int dim;
  register i;

#ifdef CHECK_ERR
  if (mat1 == NULL)
    errmsg("Mmatmerge: NULL input matrix");
  if (mat2 == NULL)
    errmsg("Mmatmerge: NULL input matrix");
  if (MxGetPr(mat1) == NULL)
    errmsg("Mmatmerge: NULL data pointer");
  if (MxGetPr(mat2) == NULL)
    errmsg("Mmatmerge: NULL data pointer");
#endif

  dim = MxGetN(mat2);

  /* copy right colums to the first matrix	 */
  for (i = 0; i < dim; i++)
    memcpy(MxGetPr(mat1)[off + i], MxGetPr(mat2)[i], MxGetM(mat1) * sizeof(double));

#ifdef DEBUG
  Mprintmat(result);
  printf("Mmatmerge offset=%d dim=%d cols %d-%d\n\n", off, dim, dim * off, dim * off + dim - 1);
#endif

  /* FREE mat2 IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat2);

  return mat1;
}

/* MNORM(MAT)							*/
/* CALCULATES THE NORM OF VECTOR MAT				*/
/* PARAMETERS:							*/
/*		MAT 		VECTOR			 	*/
/* RETURNS:							*/
/*		NORM						*/
double Mnorm(mat)
MMatrix *mat;
{
  double result = 0.0, *p, tmp;
  register i;

#ifdef CHECK_ERR
  if (mat == NULL)
    errmsg("Mnorm: NULL input matrix");
  if (MxGetPr(mat) == NULL)
    errmsg("Mnorm: NULL data pointer");
#endif

  p = MxGetPr(mat)[0];
  for (i = 0; i < MxGetM(mat); i++) {
    tmp = *p++;
    result += tmp * tmp;
  }

  /* FREE mat IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat);

#ifdef DEBUG
  printf("Mnorm input\n");
  Mprintmat(mat);
  printf("Mnorm result =%f\n\n", sqrt(result));
#endif
  return (sqrt(result));
}

/* MNORMC(MAT,OLDMATRIX)					*/
/* NORMALIZE COLUMNS OF MATRIX MAT				*/
/* PARAMETERS:							*/
/*		MAT		MATRIX			 	*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A MATRIX				*/
MMatrix *Mnormc(mat, oldmatrix)
MMatrix *mat, *oldmatrix;
{
  MMatrix *result;
  double norm = 0.0, *p1, *p2, tmp;
  register i, j;

#ifdef CHECK_ERR
  if (mat == NULL)
    errmsg("Mnormc: NULL input matrix");
#endif

  result = MxCreateFull(MxGetM(mat), MxGetN(mat), 0);

  for (j = 0; j < MxGetN(mat); j++) {
    p1 = MxGetPr(result)[j];
    p2 = MxGetPr(mat)[j];
    for (i = 0; i < MxGetM(mat); i++) {
      tmp = *p2++;
      norm += tmp * tmp;
    }
    p2 = MxGetPr(mat)[j];
    for (i = 0; i < MxGetM(result); i++)
      *p1++ = *p2++ / sqrt(norm);
    norm = 0.0;
  }

  if (oldmatrix != NULL) {
    MxSetName(result, MxGetName(oldmatrix));
    MxFreeMatrix(oldmatrix);
    MxFree(oldmatrix);
  }
#ifdef DEBUG
  printf("Mnormc input\n");
  Mprintmat(mat);
  printf("Mnormc output\n");
  Mprintmat(result);
#endif

  /* FREE mat IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat);

  return result;
}

/* MLOGSIG(MAT,BIAS,OLDMATRIX)					*/
/* LOGSIG THRESHOLD FUNCTION					*/
/* PARAMETERS:							*/
/*		MAT		WEIGHT MATRIX		 	*/
/*		BIAS		BIAS VECTOR			*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A VECTOR				*/
MMatrix *Mlogsig(mat, bias, oldmatrix)
MMatrix *mat, *bias, *oldmatrix;
{
  MMatrix *result;
  double z, *p1, *p2, *p3;
  register i, j;

#ifdef CHECK_ERR
  if (mat == NULL)
    errmsg("Mlogsig: NULL input matrix mat");
  if (bias == NULL)
    errmsg("Mlogsig: NULL input matrix bias");
#endif

  result = MxCreateFull(MxGetM(mat), MxGetN(mat), 0);

  for (i = 0; i < MxGetN(mat); i++) {
    p1 = MxGetPr(result)[i];
    p2 = MxGetPr(mat)[i];
    p3 = MxGetPr(bias)[0];
    for (j = 0; j < MxGetM(mat); j++) {
      z = *p2++ + *p3++;
      *p1++ = 1.0 / (1.0 + exp(-z));
    }
  }
  if (oldmatrix != NULL) {
    MxSetName(result, MxGetName(oldmatrix));
    MxFreeMatrix(oldmatrix);
    MxFree(oldmatrix);
  }
  /* FREE mat IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat);

  return result;
}


/* MPURELIN(MAT,BIAS,OLDMATRIX)					*/
/* PURELIN THRESHOLD FUNCTION					*/
/* PARAMETERS:							*/
/*		MAT		WEIGHT MATRIX		 	*/
/*		BIAS		BIAS VECTOR			*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A VECTOR				*/
MMatrix *Mpurelin(mat, bias, oldmatrix)
MMatrix *mat, *bias, *oldmatrix;
{
  MMatrix *result;
  double z, x, *p1, *p2, *p3;
  register i, j;

#ifdef CHECK_ERR
  if (mat == NULL)
    errmsg("Mpurelin: NULL input matrix mat");
  if (bias == NULL)
    errmsg("Mpurelin: NULL input matrix bias");
#endif

  result = MxCreateFull(MxGetM(mat), MxGetN(mat), 0);

  for (i = 0; i < MxGetN(mat); i++) {
    p1 = MxGetPr(result)[i];
    p2 = MxGetPr(mat)[i];
    p3 = MxGetPr(bias)[0];
    for (j = 0; j < MxGetM(mat); j++)
      *p1++ = *p2++ + *p3++;
  }
  if (oldmatrix != NULL) {
    MxSetName(result, MxGetName(oldmatrix));
    MxFreeMatrix(oldmatrix);
    MxFree(oldmatrix);
  }
  /* FREE mat IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat);

  return result;
}

/* MTANSIG(MAT,BIAS,OLDMATRIX)					*/
/* TANSIG THRESHOLD FUNCTION					*/
/* PARAMETERS:							*/
/*		MAT		WEIGHT MATRIX		 	*/
/*		BIAS		BIAS VECTOR			*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A VECTOR				*/
MMatrix *Mtansig(mat, bias, oldmatrix)
MMatrix *mat, *bias, *oldmatrix;
{
  MMatrix *result;
  double z, x, *p1, *p2, *p3;
  register i, j;

#ifdef CHECK_ERR
  if (mat == NULL)
    errmsg("Mtansig: NULL input matrix mat");
  if (bias == NULL)
    errmsg("Mtansig: NULL input matrix bias");
#endif

  result = MxCreateFull(MxGetM(mat), MxGetN(mat), 0);

  for (i = 0; i < MxGetN(mat); i++) {
    p1 = MxGetPr(result)[i];
    p2 = MxGetPr(mat)[i];
    p3 = MxGetPr(bias)[0];
    for (j = 0; j < MxGetM(mat); j++) {
      z = *p2++ + *p3++;
      x = exp(2.0 * z);
      *p1++ = (x - 1) / (x + 1);
    }
  }
  if (oldmatrix != NULL) {
    MxSetName(result, MxGetName(oldmatrix));
    MxFreeMatrix(oldmatrix);
    MxFree(oldmatrix);
  }
  /* FREE mat IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat);

  return result;
}

/* MSIGMOID(MAT,THETA,OLDMATRIX)				*/
/* SIGMOID THRESHOLD FUNCTION					*/
/* PARAMETERS:							*/
/*		MAT		VECTOR			 	*/
/*		THETA		SCALAR CONSTANT			*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A VECTOR				*/
MMatrix *Msigmoid(mat, theta, oldmatrix)
MMatrix *mat, *oldmatrix;
double theta;
{
  MMatrix *result;
  double t, tmp, *p1, *p2;
  register i;

#ifdef CHECK_ERR
  if (mat == NULL)
    errmsg("Msigmoid: NULL input matrix");
  if (MxGetPr(mat) == NULL)
    errmsg("Msigmoid: NULL data pointer");
#endif

  result = MxCreateFull(MxGetM(mat), MxGetN(mat), 0);
  p1 = MxGetPr(result)[0];
  p2 = MxGetPr(mat)[0];
  for (i = 0; i < MxGetM(mat); i++) {
    t = *p2++;
    if ((t >= 0) && (t <= theta)) {
      tmp = t * t;
      *p1++ = (2.0 * theta * tmp) / (tmp + theta * theta);
    } else
      *p1++ = t;
  }

#ifdef DEBUG
  printf("sigmoid input\n");
  Mprintmat(mat);
  printf("sigmoid output\n");
  Mprintmat(result);
#endif

  if (oldmatrix != NULL) {
    MxSetName(result, MxGetName(oldmatrix));
    MxFreeMatrix(oldmatrix);
    MxFree(oldmatrix);
  }
  /* FREE mat IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat);

  return result;
}

/* MSIGMOID2(MAT,THETA,OLDMATRIX)				*/
/* SIGMOID THRESHOLD FUNCTION					*/
/* PARAMETERS:							*/
/*		MAT		VECTOR			 	*/
/*		THETA		SCALAR CONSTANT			*/
/*		OLDMATRIX	MATRIX WHICH IS REPLACED,	*/
/*				NULL IF NONE			*/
/* RETURNS:							*/
/*		POINTER TO A VECTOR				*/
MMatrix *Msigmoid2(mat, theta, oldmatrix)
MMatrix *mat, *oldmatrix;
double theta;
{
  MMatrix *result;
  register i;
  double t, *p1, *p2;

#ifdef CHECK_ERR
  if (mat == NULL)
    errmsg("Msigmoid2: NULL input matrix");
  if (MxGetPr(mat) == NULL)
    errmsg("Msigmoid2: NULL data pointer");
#endif

  result = MxCreateFull(MxGetM(mat), MxGetN(mat), 0);

  p1 = MxGetPr(result)[0];
  p2 = MxGetPr(mat)[0];

  for (i = 0; i < MxGetM(mat); i++) {
    t = *p2++;
    if ((t >= 0) && (t <= theta))
      *p1++ = 0;
    else
      *p1++ = t;
  }

  if (oldmatrix != NULL) {
    MxSetName(result, MxGetName(oldmatrix));
    MxFreeMatrix(oldmatrix);
    MxFree(oldmatrix);
  }
#ifdef DEBUG
  printf("sigmoid2 input\n");
  Mprintmat(mat);
  printf("sigmoid2 output\n");
  Mprintmat(result);
#endif

  /* FREE mat IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat);

  return result;
}


/* MCOMPET(MAT,COL)						*/
/* FIND THE GREATEST VALUE OF COLUMN COL OF MATRIX MAT		*/
/* PARAMETERS:							*/
/*		MAT		MATRIX			 	*/
/*		COL		COLUMN OF MATRIX MAT		*/
/* RETURNS:							*/
/*		INDEX OF GREATEST VALUE				*/
int Mcompet(mat, col)
MMatrix *mat;
int col;
{
  double tmp = MINDOUBLE, t, *p;
  int result = 0;
  register i;

#ifdef CHECK_ERR
  if (mat == NULL)
    errmsg("Mcompet: NULL input matrix");
  if (MxGetPr(mat) == NULL)
    errmsg("Mcompet: NULL data pointer");
#endif

  p = MxGetPr(mat)[col];

  for (i = 0; i < MxGetM(mat); i++) {
    t = *p++;
    if (t > tmp) {
      tmp = t;
      result = i;
    }
  }

#ifdef DEBUG
  printf("Mcompet mat x=%d y=%d\n", MxGetM(mat), MxGetN(mat));
  printf("Mcompet result =%d\n\n", result);
#endif

  /* FREE mat IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat);

  return result;
}

/* MCOMPET2(MAT,COL,P)						*/
/* FIND THE GREATEST VALUE OF COLUMN COL OF MATRIX MAT		*/
/* PARAMETERS:							*/
/*		MAT		MATRIX			 	*/
/*		COL		COLUMN OF MATRIX MAT		*/
/*		P		ROW WHICH IS NOT USED IN SEARCH	*/
/* RETURNS:							*/
/*		INDEX OF GREATEST VALUE				*/
int Mcompet2(mat, col, p)
MMatrix *mat;
int col, p;
{
  double tmp = MINDOUBLE, t, *p1;
  int result = 0;
  register i;

#ifdef CHECK_ERR
  if (mat == NULL)
    errmsg("Mcompet2: NULL input matrix");
  if (MxGetPr(mat) == NULL)
    errmsg("Mcompet2: NULL data pointer");
#endif

  p1 = MxGetPr(mat)[col];

  for (i = 0; i < MxGetM(mat); i++) {
    t = *p1++;
    if ((t > tmp) && (i != p)) {
      tmp = t;
      result = i;
    }
  }

#ifdef DEBUG
  printf("Mcompet2 mat x=%d y=%d\n", MxGetM(mat), MxGetN(mat));
  printf("Mcompet2 result =%d\n\n", result);
#endif

  /* FREE mat IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat);

  return result;
}

/* MALL(MAT,SCALAR)						*/
/* CHECKS IF MATRIX CONTAINS ONLY THE VALUE OF PARAMETER SCALAR	*/
/* PARAMETERS:							*/
/*		MAT		MATRIX			 	*/
/*		SCALAR		SCALAR CONSTANT			*/
/* RETURNS:							*/
/*		1 IF MATRIX CONTAINS ONLY VALUE SCALAR, ELSE 0	*/
int Mall(mat, scalar)
MMatrix *mat;
double scalar;
{
  register i, j;
  double *p;

#ifdef CHECK_ERR
  if (mat == NULL)
    errmsg("Mall: NULL input matrix");
  if (MxGetPr(mat) == NULL)
    errmsg("Mall: NULL data pointer");
#endif

  for (j = 0; j < MxGetN(mat); j++) {
    p = MxGetPr(mat)[j];
    for (i = 0; i < MxGetM(mat); i++)
      if (*p++ != scalar)
	return 0;
  }
#ifdef DEBUG
  printf("Mall mat x=%d y=%d\n", MxGetM(mat), MxGetN(mat));
  printf("Mall result = 1\n\n");
#endif

  /* FREE mat IF IT IS A TEMPORARY RESULT	 */
  MxCheckTemp(mat);

  return 1;
}

int hardlim(scalar)
double scalar;
{
#ifdef DEBUG
  printf("hardlim input=%f result 1 if scalar >=0 else 0\n\n", scalar);
#endif
  if (scalar >= 0.0)
    return 1;
  else
    return 0;
}

/* MRANDO(ROWS,COLS)						*/
/* CREATES ROWSxCOLS RANDOM MATRIX				*/
/* PARAMETERS:							*/
/*		ROWS		NUMBER OF ROWS		 	*/
/*		COLS		NUMBER OF COLS			*/
/* RETURNS:							*/
/*		POINTER TO A MATRIX				*/
MMatrix *Mrando(rows, cols, name)
int rows, cols;
char *name;
{
  MMatrix *result = NULL;
  register i, j;
  double *p;

  result = MxCreateFull(rows, cols, 0);

  MxSetName(result, name);

  for (j = 0; j < cols; j++) {
    p = MxGetPr(result)[j];
    for (i = 0; i < rows; i++)
      *p++ = drand48();
  }

#ifdef DEBUG
  printf("rando x=%d y=%d\n", rows, cols);
  Mprintmat(result);
#endif

  return result;
}

/* MINPROD(MAT1,COL1,MAT2,COL2)					*/
/* CALCULATES INNER PRODUCT OF COLUMN COL1 OF MATRIX MAT1 	*/
/* AND COLUMN COL2 OF MATRIX MAT2				*/
/* PARAMETERS:							*/
/*		MAT1 		MATRIX 1		 	*/
/*		COL1 		COLUMN OF MATRIX 1	 	*/
/*		MAT2		MATRIX 2		 	*/
/*		COL2 		COLUMN OF MATRIX 2	 	*/
/* RETURNS:							*/
/*		DOUBLE VALUE					*/
double Minprod(mat1, col1, mat2, col2)
MMatrix *mat1, *mat2;
int col1, col2;
{
  double result = 0.0, *p1, *p2;
  register i;

#ifdef CHECK_ERR
  if (mat1 == NULL)
    errmsg("Minprod: NULL input matrix");
  if (mat2 == NULL)
    errmsg("Minprod: NULL input matrix");
  if (MxGetM(mat1) != MxGetM(mat2))
    errmsg("Minprod: input matrix dimension");
#endif

  p1 = MxGetPr(mat1)[col1];
  p2 = MxGetPr(mat2)[col2];

  for (i = 0; i < MxGetM(mat1); i++)
    result += *p1++ * *p2++;

#ifdef DEBUG
  printf("Minprod mat1 x=%d y=%d col1 =%d mat2 x=%d y=%d col2 =%d\n",
	 MxGetM(mat1), MxGetN(mat1), col1, MxGetM(mat2), MxGetN(mat2), col2);
  Mprintmat(mat1);
  Mprintmat(mat2);
  printf("Minprod result %f\n", result);
#endif

  return result;
}

/* MORTH(MAT)							*/
/* ORTHOGONALIZES COLUMN VECTORS OF MATRIX MAT BY USING  	*/
/* GRAM-SCHMIDT ORTHOGONALIZATION (GSO) PROCEDURE		*/
/* PARAMETERS:							*/
/*		MAT 		MATRIX			 	*/
/* RETURNS:							*/
/*		POINTER TO A MATRIX				*/
MMatrix *Morth(mat)
MMatrix *mat;
{
  MMatrix *result = NULL, *tmp = NULL;
  double scalar;
  int rows, cols;
  register j, k;

#ifdef CHECK_ERR
  if (mat == NULL)
    errmsg("Morth: NULL input matrix");
#endif

  rows = MxGetM(mat);
  cols = MxGetN(mat);

  result = Mmatcopy(mat, result);
  MxSetName(result, MxGetName(mat));

  tmp = Mmatinit(rows, 1, 0.0, "tmp", NULL);

  for (j = 1; j < cols; j++) {
    for (k = 0; k < j; k++) {
      scalar = Minprod(mat, j, result, k) / Minprod(result, k, result, k);
      tmp = Mvecadd(Mcolmul(scalar, result, k, NULL), k, tmp, 0, tmp);
    }
    result = Mmatmerge(result, Mvecsub(mat, j, tmp, 0, NULL), j);
    tmp = Mmatinit(rows, 1, 0.0, "tmp", tmp);
  }

  MxFreeMatrix(tmp);
  MxFree(tmp);
  MxFreeMatrix(mat);
  MxFree(mat);

#ifdef DEBUG
  printf("Morth result x=%d y=%d\n", MxGetM(result), MxGetN(result));
  Mprintmat(result);
#endif
  return result;
}

/* MDISCRIMINANT(PATTERN,DISC,MAT,NC,DIM)			*/
/* CALCULATES DISCRIMINANT FUNCTION FOR A PATTERN VECTOR 	*/
/* PARAMETERS:							*/
/*		PATTERN 	PATTERN VECTOR		 	*/
/*		DISC 		DISCRIMINANT VECTOR	 	*/
/*		MAT 		SUBSPACE MATRIX		 	*/
/*		NC 		NUMBER OF CLASSES	 	*/
/*		DISC 		DIMENSION OF SUBSPACE	 	*/
/* RETURNS:							*/
/*		DOUBLE VALUE STORED IN DISC VECTOR		*/
void Mdiscriminant(pattern, disc, mat, nc, dim)
MMatrix *pattern, *disc, *mat;
int nc, dim;
{
  register i, j;
  double tmp = 0.0, scalar = 0.0, *p;

  p = MxGetPr(disc)[0];

  /* EVALUATE DISCRIMINANT FUNCTIONS FOR ALL CLASSES */
  for (j = 0; j < nc; j++) {
    for (i = 0; i < dim; i++) {
      tmp = Minprod(pattern, 0, mat, j * dim + i);
      scalar += tmp * tmp;
    }
    *p++ = scalar;
    scalar = 0.0;
  }
  return;
}

/* CHECK_EXT(BUF,EXT)						*/
/* CHECKS IF THERE IS AN EXTENSION EXT AT THE END OF STRING BUF	*/
/* IF THERE IS NO EXTENSION THEN IT IS ADDED			*/
/* PARAMETERS:							*/
/*		BUF 		STRING			 	*/
/*		EXT		EXTENSION STRING		*/
/* RETURNS:							*/
/*		POINTER TO A STRING				*/
char *check_ext(buf, ext)
char *buf, *ext;
{
  int len;
  /* static char tmp[50]; */
  char *tmp;
  len = strlen(buf);
  if (len >= 4)
    if (strcmp((buf + len - 4), ext) == 0)
      return buf;
  if ((tmp = (char *) malloc((len + 5) * sizeof(char))) == NULL)
    errmsg("check_ext: Cannot handle extension\n");
  strcpy(tmp, buf);
  strcat(tmp, ext);
  free(buf);
  return tmp;
}

/* INIT_CONFUSION_MATRIX(NCLASSES)				*/
/* INITIALIZES CONFUSION MATRIX					*/
/* PARAMETERS:							*/
/*		NCLASSES	NUMBER OF CLASSES		*/
/* RETURNS:							*/
/*		POINTER TO A CONFUSION MATRIX			*/
int **init_confusion_matrix(Nclasses)
int Nclasses;
{
  int **conf;
  register i, j;

  /* ALLOCATE MEMORY FOR CONFUSION MATRIX */
  conf = (int **) malloc((Nclasses + 2) * sizeof(conf));
  for (i = 0; i < Nclasses + 2; i++)
    conf[i] = (int *) malloc((Nclasses + 2) * sizeof(int));
  if (conf == NULL)
    fprintf(stderr, "init_confusion_matrix: NULL confusion matrix");

  /* INITIALIZE CONFUSION MATRIX  */
  for (i = 0; i < Nclasses + 2; i++)
    for (j = 0; j < Nclasses + 2; j++)
      conf[i][j] = 0;
  return conf;
}

/* UPDATE_CONFUSION_MATRIX(CONF,CORRECT,CLASS,NCLASSES)		*/
/* UPDATES CONFUSION MATRIX					*/
/* PARAMETERS:							*/
/*		CONF		CONFUSION MATRIX		*/
/*		CORRECT		CORRECT CLASS FOR A VECTOR	*/
/*		CLASS		CLASSIFICATION RESULT		*/
/*		NCLASSES	NUMBER OF CLASSES		*/
int update_confusion_matrix(conf, correct, class, Nclasses)
int **conf, correct, class, Nclasses;
{
  static int total;
  /* UPDATE CONFUSION MATRIX      */
  conf[correct][class] = conf[correct][class] + 1;
  if (class == correct) {
    conf[correct][Nclasses] = conf[correct][Nclasses] + 1;
    conf[Nclasses][class] = conf[Nclasses][class] + 1;
    conf[Nclasses][Nclasses] = conf[Nclasses][Nclasses] + 1;
  } else {
    conf[correct][Nclasses + 1] = conf[correct][Nclasses + 1] + 1;
    conf[Nclasses + 1][class] = conf[Nclasses + 1][class] + 1;
    conf[Nclasses][Nclasses + 1] = conf[Nclasses][Nclasses + 1] + 1;
    conf[Nclasses + 1][Nclasses] = conf[Nclasses + 1][Nclasses] + 1;
  }
  if (conf[Nclasses + 1][Nclasses + 1] == 0)
    total = 0;
  conf[Nclasses + 1][Nclasses + 1] = ++total;
}

/* PRINT_CONFUSION_MATRIX(CONF,NCLASSES)			*/
/* PRINTS CONFUSION MATRIX					*/
/* PARAMETERS:							*/
/*		CONF		CONFUSION MATRIX		*/
/*		NCLASSES	NUMBER OF CLASSES		*/
double print_confusion_matrix(conf, Nclasses)
int **conf, Nclasses;
{
  double pros = 0.0;
  register i, j;

  /* CALCULATE CLASSIFICATION %   */
  pros = (double) conf[Nclasses][Nclasses] / conf[Nclasses + 1][Nclasses + 1] * 100.0;

  /* PRINT CONFUSION MATRIX       */
  for (i = 0; i < Nclasses; i++) {
    for (j = 0; j < Nclasses; j++)
      printf("%3d ", conf[i][j]);
    printf(" | %3d  %3d\n", conf[i][Nclasses], conf[i][Nclasses + 1]);
  }
  for (i = 0; i < Nclasses + 2; i++)
    printf("----");
  printf("\n");

  for (i = Nclasses; i < Nclasses + 2; i++) {
    for (j = 0; j < Nclasses; j++)
      printf("%3d ", conf[i][j]);
    printf(" | %3d  %3d\n", conf[i][Nclasses], conf[i][Nclasses + 1]);
  }
  printf("\n%3.2f%%\n\n", pros);

  /* FREE MEMORY USED BY CONFUSION MATRIX */
  for (i = 0; i < Nclasses + 2; i++)
    free(conf[i]);
  free(conf);
  return (pros);
}

/* LOAD_MATRIX(FNAME,MAT_NAME)					*/
/* LOADS MATRIX MAT_NAME FROM FILE FNAME			*/
/* PARAMETERS:							*/
/*		FNAME 		FILE NAME		 	*/
/*		MAT_NAME 	MATRIX NAME		 	*/
/* RETURNS:							*/
/*		POINTER TO A MATRIX				*/
MMatrix *load_matrix(fname, mat_name)
char *fname, *mat_name;
{
  MMatrix *mat;
  FILE *fp;
  if ((fp = fopen(check_ext(fname, MAT), "r")) == NULL) {
    fprintf(stderr, "Error in opening file %s\n", fname);
    return NULL;
  }
  if ((mat = MatGetMatrix(fp, mat_name)) == NULL) {
    fprintf(stderr, "Error in reading matrix %s from file %s\n", mat_name, fname);
    fclose(fp);
    return NULL;
  }
  fclose(fp);
  return (mat);
}

/* SAVE_MATRIX(FNAME,MAT)					*/
/* SAVES MATRIX MAT TO FILE FNAME				*/
/* PARAMETERS:							*/
/*		FNAME 		FILE NAME		 	*/
/*		MAT	 	POINTER TO A MATRIX		*/
/* RETURNS:							*/
/*		1 IF SUCCESFUL ELSE 0				*/
int save_matrix(fname, mat)
char *fname;
MMatrix *mat;
{
  FILE *fp;
  if ((fp = fopen(check_ext(fname, MAT), "w")) == NULL) {
    fprintf(stderr, "Error in opening file %s\n", fname);
    return 0;
  }
  if (MatPutMatrix(fp, mat)) {
    fprintf(stderr, "Error in saving matrix to file %s\n", fname);
    fclose(fp);
    return 0;
  }
  fclose(fp);
  return 1;
}

/* MBMU(MAT1,MAT2)						*/
/* SEARCH BEST MATCHING UNIT FOR COLUMN VECTOR MAT2 FROM COLUMN	*/
/* VECTORS STORED IN MATRIX MAT1				*/
/* MATRIX MAT1							*/
/* PARAMETERS:							*/
/*		MAT1 		MATRIX 1		 	*/
/*		P		DOUBLE VECTOR			*/
/*		L		LABEL VECTOR			*/
/* RETURNS:							*/
/*		INDEX OF BEST MATCHING VECTOR			*/
int Mbmu(mat1, p, l)
MMatrix *mat1;
double *p, *l;
{
  int result;
  double difference, diff, *p1, *p2;
  register i, j;

#ifdef FLT_MAX
  double diffsf = FLT_MAX;
#else
  double diffsf = MAXDOUBLE;
#endif

#ifdef CHECK_ERR
  if (mat1 == NULL)
    errmsg("Mbmu: NULL input matrix");
#endif

  for (i = 0; i < MxGetN(mat1); i++) {
    if (l[i] > -1.0) {
      difference = 0.0;
      p1 = MxGetPr(mat1)[i];
      p2 = p;
      for (j = 0; j < MxGetM(mat1); j++) {
	diff = *p2++ - *p1++;
	difference += diff * diff;
	if (difference > diffsf)
	  break;
      }
      if (difference < diffsf) {
	diffsf = difference;
	result = i;
      }
    }
  }

#ifdef DEBUG
  printf("Mbmu input\n");
  Mprintmat(mat1);
#endif
  return (result);
}

/* MDISTANCES(MAT, VEC, DIST)						*/
/* CALCULATES DISTANCES BETWEEN VECTOR VEC AND EACH COLUMN 	*/
/* VECTOR IN MATRIX MAT	(NOTE! USES THE SQUARE OF DISTANCE)	*/
/* MATRIX MAT1							*/
/* PARAMETERS:							*/
/*		MAT 		MATRIX			 	*/
/*		VEC		VECTOR			 	*/
/*		DIST		VECTOR			 	*/
/* RETURNS:							*/
/*		DISTANCE VECTOR	POINTED BY DIST			*/
MMatrix *Mdistances(mat, vec, dist)
MMatrix *mat, *dist;
double *vec;
{
  int result;
  double difference, diff, *p1, *p2, *p3;
  register i, j;

#ifdef FLT_MAX
  double diffsf = FLT_MAX;
#else
  double diffsf = MAXDOUBLE;
#endif

#ifdef CHECK_ERR
  if (mat == NULL)
    errmsg("Mdistances: NULL input matrix");
  if ((MxGetPr(mat) == NULL) || (vec == NULL))
    errmsg("Mdistances: NULL data pointer");
#endif

  p3 = MxGetPr(dist)[0];

  for (i = 0; i < MxGetN(mat); i++) {
    difference = 0.0;
    p1 = vec;
    p2 = MxGetPr(mat)[i];
    for (j = 0; j < MxGetM(mat); j++) {
      diff = *p1++ - *p2++;
      difference += diff * diff;
      if (difference > diffsf)
	break;
    }
    *p3++ = difference;
  }

#ifdef DEBUG
  printf("Mdistances input\n");
  Mprintmat(mat);
#endif
  return (dist);
}

/* MSORT(MAT, OLDMAT)						*/
/* SORTS THE FIRST COLUMN OF MATRIX MAT				*/
/* MATRIX MAT1							*/
/* PARAMETERS:							*/
/*		MAT 		INPUT MATRIX		 	*/
/*		OLDMAT		POINTER TO A MATRIX WHICH IS 	*/
/*				REPLACED (NULL IF NONE)		*/
/* RETURNS:							*/
/*		SORTED VECTOR					*/
MMatrix *Msort(vec, oldmat)
MMatrix *vec, *oldmat;
{
  MMatrix *result;

  double s, r, *p1;
  register i, j;

#ifdef CHECK_ERR
  if (vec == NULL)
    errmsg("Msort: NULL input matrix");
  if (MxGetPr(vec) == NULL)
    errmsg("Msort: NULL data pointer");
#endif

  result = Mmatinit(MxGetM(vec), 1, 0.0, "sorted", NULL);

  p1 = MxGetPr(result)[0];
  for (i = 0; i < MxGetM(vec); i++)
    *p1++ = (double) i;

  for (i = 0; i < MxGetM(vec); i++) {
    for (j = i; j < MxGetM(vec); j++) {
      if (MxGetPr(vec)[0][j] < MxGetPr(vec)[0][i]) {
	s = MxGetPr(vec)[0][i];
	MxGetPr(vec)[0][i] = MxGetPr(vec)[0][j];
	MxGetPr(vec)[0][j] = s;
	r = MxGetPr(result)[0][i];
	MxGetPr(result)[0][i] = MxGetPr(result)[0][j];
	MxGetPr(result)[0][j] = r;
      }
    }
  }

#ifdef DEBUG
  printf("Msort input\n");
  Mprintmat(vec);
#endif

  MxFreeMatrix(oldmat);
  MxFree(oldmat);

  return (result);
}

/* NORMALIZE(VEC,DIM)						*/
/* NORMALIZE VECTOR TO THE LENGTH OF 1				*/
/* PARAMETERS:							*/
/*		VEC		VECTOR			 	*/
/*		DIM		DIMENSION OF THE VECTOR	 	*/
/* RETURNS:							*/
/*		NORMALIZED VECTOR				*/
void normalize(vec, dim)
double *vec;
int dim;
{
  double norm = 0.0, tmp = 0.0, *p;
  register i;
  if (vec == NULL)
    errmsg("normalize: NULL input vector");
  p = vec;
  for (i = 0; i < dim; i++)
    norm += *p * *p++;
  norm = sqrt(norm);

  p = vec;
  for (i = 0; i < dim; i++)
    *p = *p++ / norm;
}

/* SCALE(VEC,DIM,VAL)						*/
/* SCALE VECTOR WITH MAXIMUM VALUE				*/
/* PARAMETERS:							*/
/*		VEC		VECTOR			 	*/
/*		DIM		DIMENSION OF THE VECTOR	 	*/
/*		VAL		MAXIMUM VALUE			*/
/* RETURNS:							*/
/*		SCALED VECTOR					*/
void scale(vec, dim, val)
double *vec;
int dim, val;
{
  register i;
  double *p;
  if (vec == NULL)
    errmsg("scale: NULL input vector");
  if (val == 0)
    errmsg("scale: Trying to divide by zero");
  p = vec;
  for (i = 0; i < dim; i++)
    *p = *p++ / (double) val;
}
