/***********************************************************************
*                                                                      *
*                              SPECLASS                                *
*                                                                      *
*                                The                                   *
*                                                                      *
*                      Color Spectra Classifier                        *
*                                                                      *
*                          Program  Package                            *
*                                                                      *
*                           Programmed by			       *
*								       *
*			     J. Haanpalo		               *
*                                                                      *
*                      Copyright (c) 1993-1995                         *
*                                                                      *
************************************************************************
*                                                                      *
*  NOTE: This program package is copyrighted in the sense that it      *
*  may be used for scientific purposes. The package as a whole, or     *
*  parts thereof, cannot be included or used in any commercial         *
*  application without written permission granted by its producents.   *
*  No programs contained in this package may be copied for commercial  *
*  distribution. This program package comes 'as it is' and with no     *
*  warranty.                                                           *
*                                                                      *
*  All comments concerning this program package may be sent to the     *
*  e-mail address 'haanpalo@lut.fi'.                                   *
*                                                                      *
***********************************************************************/

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <sys/stat.h>
#include <sys/dir.h>
#include "xglobal.h"
#include "global.h"
#include "matfile.h"

#ifndef MAXPATHLEN
#include <sys/param.h>
#endif

double *make_double_vector();

extern void draw_spek(), draw_empty_spec();
extern int draw_list();

char *spec_dir;

/* GETFILENAME(PATH)						*/
/* FINDS THE LAST ITEM IN A PATH				*/
/* PARAMETERS:							*/
/*		PATH	POINTER TO A PATH STRING		*/
/* RETURNS:							*/
/*		POINTER TO AN ALLOCATED STRING			*/
char *getfilename(path)
char *path;
{
  char *p;
  if (p = rindex(path, '/'))
    p++;
  else
    p = path;
  return strcpy(malloc(strlen(p) + 1), p);
}

/* CREATE_MENU(PATH)						*/
/* CREATES A PULLRIGHT MENU FOR MUNSELL BUTTON. MUNSELL SPECTRA	*/
/* FILES ARE PRESENTED AS MENUITEMS				*/
/* PARAMETERS:							*/
/*		PATH	POINTER TO A PATH STRING WHICH TELLS	*/
/*			THE LOCATION OF MUNSELL SPECTRA DATABASE*/
/* RETURNS:							*/
/*		MENUITEM					*/
Menu_item create_menu(path)
char *path;
{
  DIR *dirp;
  struct direct *dp;
  struct stat s_buf;
  Menu_item mi;
  Menu next_menu, create_submenu();
  char buf[MAXPATHLEN];
  static int recursion;

  if (stat(path, &s_buf) == -1 || !(s_buf.st_mode & S_IREAD))
    return NULL;
  if (s_buf.st_mode & S_IFDIR) {
    int cnt = 0;
    if (!(dirp = opendir(spec_dir)))
      return NULL;
    if (recursion)
      return (Menu_item) - 1;
    recursion++;
    next_menu = (Menu) xv_create(XV_NULL, MENU,
				 MENU_TITLE_ITEM, "spectra",
				 MENU_NROWS, 20,
				 MENU_NCOLS, 2,
				 NULL);
    while (dp = readdir(dirp))
      if (strcmp(dp->d_name, ".") && strcmp(dp->d_name, "..")) {
	(void) sprintf(buf, "%s/%s", path, dp->d_name);
	if (!(mi = create_menu(buf)))
	  mi = xv_create(XV_NULL, MENUITEM,
			 MENU_STRING, getfilename(dp->d_name),
			 MENU_INACTIVE, TRUE,
			 NULL);
	else
	  xv_set(mi, MENU_GEN_PULLRIGHT, create_submenu, NULL);
	xv_set(next_menu, MENU_APPEND_ITEM, mi, NULL);
	cnt++;
      }
    closedir(dirp);
    mi = xv_create(XV_NULL, MENUITEM,
		   MENU_STRING, getfilename(path),
		   MENU_RELEASE,
		   MENU_RELEASE_IMAGE,
		   NULL);
    if (!cnt) {
      xv_destroy(next_menu);
      xv_set(mi, MENU_INACTIVE, TRUE, NULL);
    } else
      xv_set(mi, MENU_PULLRIGHT, next_menu, NULL);
    recursion--;
    return mi;
  }
  return (Menu_item) xv_create(NULL, MENUITEM,
			       MENU_STRING, getfilename(path),
			       MENU_RELEASE,
			       MENU_RELEASE_IMAGE,
			       NULL);
}

/* GET_SPECTRUM(MENU, ITEM)					*/
/* READS SELECTED SPECTRUM FROM FILE AND PUTS IT INTO		*/
/* LINKED LIST							*/
/* 								*/
/* PARAMETERS:							*/
/*		MENU	MENU POINTER				*/
/*			ITEM PONTER				*/
void get_spectrum(menu, item)
Menu menu;
Menu_item item;
{
  FILE *fp;
  Spec *next;
  int spec_count = 0, class, dim = 0, type, veclen = 0, view_mode = 0,
    cno, i, maxval;
  char linebuf[LINE_LEN], filename[NAME_LEN], file[NAME_LEN], label[NAME_LEN],
    messu[100], *ptr;
  double *db;
#ifdef GCC
  double dbuf[300] = {0};
#else
  double dbuf[300];
  register j;
  for(j=0;j<300;j++)
    dbuf[j] = 0.0;
#endif

  if ((ptr = (char *) xv_get(menu, MENU_CLIENT_DATA)) == NULL)
    return;

  strcpy(file, ptr);
  strcpy(label, (char *) xv_get(item, MENU_STRING));

  view_mode = (int) xv_get(edview_class_item, PANEL_VALUE);
  cno = atoi((char *) xv_get(edclass_item, PANEL_VALUE));

  /* REMOVE UNSELECTED SPECTRUM FROM THE LIST */
  if (!xv_get(item, MENU_SELECTED)) {
    delete_spectrum(file, label, &start);

    /* CLEAR CANVAS AND DRAW COORDINATE AXIS */
    draw_empty_spec();

    /* DRAW SPECTRA FROM CORRECT LIST */
    if (view_mode == 1)
      draw_class(start, cno);
    else
      draw_list(start);

    list1_total--;

    /* SHOW THE NUMBER OF SPECTRA	 */
    sprintf(messu, "%d spectra", list1_total);
    xv_set(frame, FRAME_LEFT_FOOTER, messu, NULL);

    return;
  }
  strcpy(filename, spec_dir);
  strcat(filename, "/");
  strcat(filename, file);

  if ((fp = fopen(filename, "r")) == NULL) {
    perror("fopen");
    exit(1);
  }
  /* GET MAXIMUM VALUE FOR SPECTRUM COMPONENTS	 */
  maxval = atoi((char *) xv_get(maxval_item, PANEL_VALUE));

  /* READ SPECTRUM SELECTED FROM A MENU */
  switch ((int) xv_get(load_format_item, PANEL_VALUE)) {

    /* READ MUNSELL TYPE SPECTRUM			 */
    /* FIRST LINE IS A COMMENT LINE, SECOND LINE IS	 */
    /* DATA LINE					 */
  case MUNSELL_DATA:

    /* FIND CORRECT SPECTRUM FROM SELECTED FILE */
    while (strcmp(linebuf, label) != 0)
      if (fgets(linebuf, LINE_LEN, fp) == NULL)
	break;

    /* READ DATA LINE	 */
    fgets(linebuf, LINE_LEN, fp);
    db = make_double_vector(linebuf, maxval, &veclen);
    for (i = 0; i < veclen; i++)
      dbuf[i] = db[i];
    dim = veclen - 1;
    break;

    /* READ NATURAL TYPE SPECTRUM			 */
    /* FIRST LINE IS A COMMENT LINE, 5 FOLLOWING    */
    /* LINES ARE DATA LINES				 */
  case NATURAL_DATA:

    /* FIND CORRECT SPECTRUM FROM SELECTED FILE */
    while (strcmp(linebuf, label) != 0)
      if (fgets(linebuf, LINE_LEN, fp) == NULL)
	break;

    /* READ FIVE DATA LINES	 */
    fgets(linebuf, LINE_LEN, fp);
    while (isdigit(linebuf[4])) {
      db = make_double_vector(linebuf, maxval, &veclen);
      for (i = 0; i < veclen; i++)
	dbuf[i + dim] = db[i];
      dim += veclen;
      fgets(linebuf, LINE_LEN, fp);
    }
    break;

    /*
     * case MINOLTA_DATA: if(fscanf(fp, "%s", trivi)==EOF) return 0; for(i=0;
     * i < 61; i++){ fscanf(fp, "%d" , &ti); sp->sp_data[i] = (double)ti; }
     * break;
     */

  default:
    return;

  }

  fclose(fp);

  /* GET PARAMETERS FOR SPECTRUM	 */
  class = atoi((char *) xv_get(edclass_item, PANEL_VALUE));
  type = (int) xv_get(type_item, PANEL_VALUE);

  /* PROCESS SPECTRUM IF NEEDED	 */
  switch (type) {
  case SCALED:
    scale(dbuf, dim, maxval);
    break;
  case NORMALIZED:
    normalize(dbuf, dim);
    break;
  case RAW:
    break;
  }

  /* ADD SPECTRUM WITH ITS PARAMETERS TO LINKED LIST	 */
  make_spectrum(class, dim, type, file, label, dbuf, &start);
  list1_total++;

  /* SHOW THE NUMBER OF SPECTRA	 */
  sprintf(messu, "%d spectra", list1_total);
  xv_set(frame, FRAME_LEFT_FOOTER, messu, NULL);

  /* SHOW SPECTRUM ON THE SCREEN IF CORRECT VIEW MODE IS SELECTED	 */
  view_mode = (int) xv_get(edview_class_item, PANEL_VALUE);
  if (view_mode == 0)
    draw_spek(get_spec(start, list1_total - 1), 1);
  else {
    cno = atoi((char *) xv_get(edclass_item, PANEL_VALUE));
    draw_spek(get_spec(start, list1_total - 1), 1);

    /*
     * CLEAR CANVAS AND DRAW COORDINATE AXIS draw_empty_spec();
     * draw_class(start, cno);
     */
  }
}


/* CREATE_SUBMENU(MENU_ITEM,OP)					*/
/* CREATES A PULLRIGHT SUBMENU FOR SPECTRA FILE MENUITEMS.	*/
/* EACH CREATED SUBMENU CONSISTS OF THE LABELS OF SPECTRA	*/
/* STORED IN SPECTRA FILE RESPECTIVELY. A SUBMENU IS CREATED	*/
/* WHEN IT IS NEEDED FOR THE FIRST TIME OR PRESSING CLEAR 	*/
/* BUTTON HAS BEEN DESTROYED IT. OTHERWISE OLD SUBMENU IS	*/
/* RETURNED.							*/
/* PARAMETERS:							*/
/*		MENU_ITEM	PARENT MENUITEM			*/
/*		OP		MENU OPERATION			*/
/* RETURNS:							*/
/*		SUBMENU	WITH TOGGLE BUTTONS			*/
Menu create_submenu(menu_item, op)
Menu_item menu_item;
Menu_generate op;
{
  Menu_item mi;
  Menu next_menu, menu, menu1 = NULL, menu2, pmenu, testmenu[32];
  FILE *fp;
  char linebuf[LINE_LEN], fname[NAME_LEN], data[NAME_LEN], *ptr;
  int counter = 1, subcounter = 0, j;

  if (op == MENU_DISPLAY) {

    /* IF MENU EXISTS THEN RETURN IT ELSE DESTROY AND CONTINUE */
    if (menu = (Menu) xv_get(menu_item, MENU_PULLRIGHT))
      if (xv_get(menu, MENU_CLIENT_DATA))
	return menu;
      else {
	for (j = 1; j <= (int) xv_get(menu, MENU_NITEMS); j++)
	  if (pmenu = (Menu) xv_get(xv_get(menu, MENU_NTH_ITEM, j),
				    MENU_PULLRIGHT)) {
	    xv_free(xv_get(pmenu, MENU_CLIENT_DATA));
	    xv_destroy(pmenu);
	  }
      }
    xv_destroy(menu);

    /* CREATE NEW MENU	 */
    next_menu = (Menu) xv_create(XV_NULL, MENU_TOGGLE_MENU,
				 MENU_GEN_PIN_WINDOW, frame,
				 xv_get(menu_item, MENU_STRING),
				 MENU_NROWS, 12,
				 MENU_NCOLS, 3,
				 NULL);

    strcpy(data, (char *) xv_get(menu_item, MENU_STRING));
    strcpy(fname, spec_dir);
    strcat(fname, "/");
    strcat(fname, data);
    if ((fp = fopen(fname, "r")) == NULL) {
      perror("fopen");
      exit(1);
    }
    /* READ MENUITEM NAMES FROM SPECTRA FILE */
    while (1) {
      if (fgets(linebuf, LINE_LEN, fp) == NULL)
	break;
      if (isalpha(linebuf[0])) {
	if (ptr = malloc(strlen(linebuf) + 1))
	  strcpy(ptr, linebuf);
	if (counter % 33 == 0) {
	  testmenu[subcounter] = next_menu;
	  subcounter++;
	  /* CREATE NEW MENU	 */
	  next_menu = (Menu) xv_create(XV_NULL,
				       MENU_TOGGLE_MENU,
				       MENU_GEN_PIN_WINDOW, frame,
				       xv_get(menu_item, MENU_STRING),
				       NULL);
	}
	mi = xv_create(XV_NULL, MENUITEM,
		       MENU_RELEASE,
		       MENU_RELEASE_IMAGE,
		       MENU_NOTIFY_PROC, get_spectrum,
		       MENU_STRING, ptr,
		       NULL);
	xv_set(next_menu, MENU_APPEND_ITEM, mi, NULL);
	counter++;
      }
    }
    fclose(fp);


    xv_set(next_menu, MENU_CLIENT_DATA, strcpy(malloc(strlen(data) + 1),
					       data), NULL);

    if (subcounter > 0) {
      menu2 = (Menu) xv_create(XV_NULL, MENU, NULL);
      for (j = 0; j < subcounter; j++) {
	xv_set(testmenu[j], MENU_CLIENT_DATA, strcpy(malloc(strlen(data) + 1), data), NULL);
	mi = (Menu_item) xv_create(NULL, MENUITEM,
			MENU_STRING, strcpy(malloc(strlen(data) + 1), data),
				   MENU_PULLRIGHT, testmenu[j],
				   MENU_RELEASE,
				   MENU_RELEASE_IMAGE,
				   NULL);
	xv_set(menu2, MENU_APPEND_ITEM, mi, NULL);
      }

      mi = (Menu_item) xv_create(NULL, MENUITEM,
			MENU_STRING, strcpy(malloc(strlen(data) + 1), data),
				 MENU_PULLRIGHT, next_menu,
				 MENU_RELEASE,
				 MENU_RELEASE_IMAGE,
				 NULL);

      xv_set(menu2, MENU_APPEND_ITEM, mi, NULL);
      xv_set(menu2, MENU_CLIENT_DATA, strcpy(malloc(strlen(data) + 1), data), NULL);
      return menu2;
    }
    return next_menu;
  }
  if (!(menu = (Menu) xv_get(menu_item, MENU_PULLRIGHT)))
    menu = xv_create(NULL, MENU, MENU_STRINGS, "?", NULL, NULL);
  return menu;
}

/* DESTROY_MENU(ITEM,EVENT)						*/
/* CLEARS ALL SPECTRA SELECTIONS WHEN CLEAR SET BUTTON IS PRESSED	*/
/* CLEARING IS DONE BY DESTROYING THOSE SUBMENUS IN WHICH THERE EXISTS	*/
/* SELECTED SPECTRA. THIS IS ONE WAY TO AVOID THE PROBLEMS WITH UPDATING*/
/* SUBFRAMES.								*/
void destroy_menu(menu)
Menu menu;
{
  Menu submenu, subsubmenu;
  register i, j;
  for (i = 2; i <= (int) xv_get(menu, MENU_NITEMS); i++)
    if (submenu = (Menu) xv_get(xv_get(menu, MENU_NTH_ITEM, i), MENU_PULLRIGHT)) {
      for (j = 1; j <= (int) xv_get(submenu, MENU_NITEMS); j++) {
	if (subsubmenu = (Menu) xv_get(xv_get(submenu, MENU_NTH_ITEM, j), MENU_PULLRIGHT)) {
	  xv_free(xv_get(subsubmenu, MENU_CLIENT_DATA));
	  xv_free(subsubmenu);
	}
	xv_free(xv_get(submenu, MENU_CLIENT_DATA));
	xv_free(submenu);
      }
    }
  xv_free(xv_get(menu, MENU_CLIENT_DATA));
  xv_free(menu);
}
