/***********************************************************************
*                                                                      *
*                              SPECLASS	                               *
*                                                                      *
*                                The                                   *
*                                                                      *
*                      Color Spectra Classifier                        *
*                                                                      *
*                          Program  Package                            *
*                                                                      *
*                           Programmed by			       *
*								       *
*			     J. Haanpalo		               *
*                                                                      *
*                      Copyright (c) 1993-1995                         *
*                                                                      *
************************************************************************
*                                                                      *
*  NOTE: This program package is copyrighted in the sense that it      *
*  may be used for scientific purposes. The package as a whole, or     *
*  parts thereof, cannot be included or used in any commercial         *
*  application without written permission granted by its producents.   *
*  No programs contained in this package may be copied for commercial  *
*  distribution. This program package comes 'as it is' and with no     *
*  warranty.                                                           *
*                                                                      *
*  All comments concerning this program package may be sent to the     *
*  e-mail address 'haanpalo@lut.fi'.                                   *
*                                                                      *
***********************************************************************/

#include<stdio.h>
#include<stdlib.h>
#include<sys/types.h>
#include<sys/file.h>
#include<string.h>
#ifdef GCC
#include<float.h>
#else
#include<values.h>
#endif
#include<malloc.h>
#include<math.h>
#ifdef MATLAB
#include<mat.h>
#endif
#include"mmatrix.h"

/* Sun version */
#define MAT             ".mat"	/* extension for Matlab files   	 */
#define NAME_LEN        100	/* default length for all filenames     */
#define minimum(a, b)  (((a) < (b)) ? (a) : (b))
#define maximum(a, b)  (((a) > (b)) ? (a) : (b))
#define MAPCOORD(x, y, c) ((x)+(c)*(y))
extern void srand48();
extern long lrand48();

usage(s)
char *s;
{
  fprintf(stderr, "%s\n", s);
  fprintf(stderr, "USAGE: -p <pattern file> [-t <target file>] -R <rows> -C <columns>\n");
  fprintf(stderr, "      [-o <output file>] [-e <epochs>] [-n <neighborhood size>]\n");
  fprintf(stderr, "      [-I <0|1>] [-H] [-v <1|2>] [-T] [-r <rand seed>] [-a <init alpha>]\n");
  fprintf(stderr, "      [-A <0|1>] [-f] [-g] [-m <old map>] [-x <bmu elements>]\n");
  fprintf(stderr, "      [-i]\n\n");
  fprintf(stderr, "    Self-Organizing-Map with Luttrell algorithm. (Sun version)\n\n");
  fprintf(stderr, "    Input matrices must be in MATLAB format and pattern vectors stored\n");
  fprintf(stderr, "    in column order. For example 1000 vectors each of which has 10 components\n");
  fprintf(stderr, "    should be stored as 10x1000 matrix.\n\n");
  fprintf(stderr, "    -e number of training epochs (default is the number of pattern vectors).\n");
  fprintf(stderr, "    -I specifies map initialization function, -I 0 initializes map by selecting\n");
  fprintf(stderr, "       weight vectors randomly from pattern vectors, -I 1 initializes weight\n");
  fprintf(stderr, "       vectors with random values (default is 0).\n");
  fprintf(stderr, "    -H specifies hierarchical training (growing map).\n");
  fprintf(stderr, "    -v allows verbose output, -v 1 gives less verbose and -v 2 a little bit more.\n");
  fprintf(stderr, "    -T transposes pattern matrix before map initialization and training.\n");
  fprintf(stderr, "    -n specifies neighborhood size (default is 1). This parameter is not really\n");
  fprintf(stderr, "       needed if hierarchical training is used.\n");
  fprintf(stderr, "    -r seed number for random number generators.\n");
  fprintf(stderr, "    -a initial value for alpha (learning rate) (default is 0.6). Alpha should be\n");
  fprintf(stderr, "       0 < alpha < 1.\n");
  fprintf(stderr, "    -A specifies learning rate function, -A 0 uses linear function to decrease alpha,\n");
  fprintf(stderr, "       -A 1 uses gaussian function (default is 0).\n");
  fprintf(stderr, "    -f turns on the fine tuning of the map.\n");
  fprintf(stderr, "    -g turns on map tuning by using small learning rates.\n");
  fprintf(stderr, "    -x uses only those first numbers for best match search that are\n");
  fprintf(stderr, "       defined by bmu element number.\n");
  fprintf(stderr, "    -i dumps initialized weight matrix into output file and quits program.\n");
  fprintf(stderr, "    -m uses old map file to initialize weight vectors.\n\n");
  exit(1);
}

int Sbmu(mat, vec, loop, quanterr)
MMatrix *mat;
double *vec, *quanterr;
int loop;
{
  int result;
  double difference, diff, *p1, *p2;
  register i, j;

#ifdef GCC
  double diffsf = FLT_MAX;
#else
  double diffsf = MAXDOUBLE;
#endif

  for (i = 0; i < MxGetN(mat); i++) {
    difference = 0.0;
    p1 = MxGetPr(mat)[i];
    p2 = vec;
    for (j = 0; j < loop; j++, p1++, p2++) {
      diff = *p2 - *p1;
      difference += diff * diff;
      if (difference > diffsf)
	break;
    }
    if (difference < diffsf) {
      diffsf = difference;
      result = i;
    }
  }
  *quanterr += diffsf;
  return (result);
}

MMatrix *paintmap(MAP, p, t, loop, verbose)
MMatrix *MAP, *p, *t;
int verbose, loop;
{
  register i, j;
  int neurons, nclass, bestmatch, cl, hit;
  double quanterr = 0.0;
  MMatrix *tmp, *result;

  if (verbose)
    fprintf(stderr, "Painting map with learn data\n");

  neurons = MxGetN(MAP);
  nclass = MxGetM(t);

  tmp = Mmatinit(nclass, neurons, 0.0, "TMP", NULL);
  result = Mmatinit(neurons, 1, -1.0, "MAPI", NULL);

  for (i = 0; i < MxGetN(p); i++) {
    bestmatch = Sbmu(MAP, MxGetPr(p)[i], loop, &quanterr);
    cl = Mcompet(t, i);
    MxGetPr(tmp)[bestmatch][cl] += 1.0;
  }
  for (i = 0; i < neurons; i++) {
    hit = 0;
    for (j = 0; j < nclass; j++) {
      if (MxGetPr(tmp)[i][j] > 0.0) {
	hit += 1;
      }
    }
    if (hit)
      MxGetPr(result)[0][i] = (double) Mcompet(tmp, i);
    else
      MxGetPr(result)[0][i] = -1.0;
  }

  MxFreeMatrix(tmp);
  MxFree(tmp);

  return (result);
}


MMatrix *interp_new_map(OLDMAP, oldrows, oldcols, rows, cols, verbose)
MMatrix *OLDMAP;
int *oldrows, *oldcols, rows, cols, verbose;
{
  MMatrix *result;
  double rint, cint, rf, cf, x1, x2, y1, y2, d11, d12, d21, d22, dsum,
   *pold1, *pold2, *pold3, *pold4, *pnew;
  int rnew, cnew, r, c, k;

  rnew = 2 * (*oldrows) - 1;
  if (rnew > rows)
    rnew = rows;
  cnew = 2 * (*oldcols) - 1;
  if (cnew > cols)
    cnew = cols;

  rint = (rnew == 1) ? 1 : (double) (*oldrows - 1) / (double) (rnew - 1);
  cint = (cnew == 1) ? 1 : (double) (*oldcols - 1) / (double) (cnew - 1);

  if ((*oldrows == rows) && (*oldcols == cols))
    return NULL;

  result = MxCreateFull(MxGetM(OLDMAP), rnew * cnew, 0);
  MxSetName(result, "MAP");

  for (r = 0; r < rnew; r++) {
    for (c = 0; c < cnew; c++) {
      rf = r * rint;
      cf = c * cint;
      y1 = floor(rf);
      y2 = ceil(rf);
      x1 = floor(cf);
      x2 = ceil(cf);
      if (verbose == 3)
	fprintf(stderr, "\nrow = %d col = %d :: y1 %d x1 %d  #  y2 %d x1 %d  #  y1 %d x2 %d  #  y2 %d x2 %d",
		r, c, (int) y1, (int) x1, (int) y2, (int) x1, (int) y1, (int) x2, (int) y2, (int) x2);

      d11 = hypot(cf - x1, rf - y1);
      if (d11 == 0.0) {
	memcpy(MxGetPr(result)[(c + cnew * r)], MxGetPr(OLDMAP)[((int) x1 + (*oldcols) * (int) y1)],
	       MxGetM(OLDMAP) * sizeof(double));
      } else {
	if (verbose == 3)
	  fprintf(stderr, "  Interpolated\n");

	d21 = hypot(cf - x2, rf - y1);
	d12 = hypot(cf - x1, rf - y2);
	d22 = hypot(cf - x2, rf - y2);
	dsum = d11 + d12 + d21 + d22;

	pold1 = MxGetPr(OLDMAP)[((int) x1 + (*oldcols) * (int) y1)];
	pold2 = MxGetPr(OLDMAP)[((int) x1 + (*oldcols) * (int) y2)];
	pold3 = MxGetPr(OLDMAP)[((int) x2 + (*oldcols) * (int) y1)];
	pold4 = MxGetPr(OLDMAP)[((int) x2 + (*oldcols) * (int) y2)];

	pnew = MxGetPr(result)[(c + cnew * r)];

	for (k = 0; k < MxGetM(OLDMAP); k++, pold1++, pold2++, pold3++, pold4++, pnew++)
	  *pnew = (1 - d11 / dsum) / 3.0 * (*pold1) + (1 - d12 / dsum) / 3.0 * (*pold2) +
	    (1 - d21 / dsum) / 3.0 * (*pold3) + (1 - d22 / dsum) / 3.0 * (*pold4);
      }

    }				/* END OF FOR c */
  }				/* END OF FOR r */

  MxFreeMatrix(OLDMAP);
  MxFree(OLDMAP);

  *oldrows = rnew;
  *oldcols = cnew;

  return (result);
}

MMatrix *initmap1(rows, cols, p, verbose)
int rows, cols, verbose;
MMatrix *p;
{
  MMatrix *result;
  register i;
  int dimen, pvecs, index;

  if (verbose)
    fprintf(stderr, "Initializing %dx%d map with randomly selected pattern vectors\n", rows, cols);

  dimen = MxGetM(p);
  pvecs = MxGetN(p);
  result = MxCreateFull(dimen, rows * cols, 0);
  MxSetName(result, "MAP");
  for (i = 0; i < rows * cols; i++) {
    index = (int) (lrand48() % (long) pvecs);
    memcpy(MxGetPr(result)[i], MxGetPr(p)[index], dimen * sizeof(double));
  }
  return (result);
}

MMatrix *initmap2(rows, cols, p, verbose)
int rows, cols, verbose;
MMatrix *p;
{
  MMatrix *result;
  int dimen;

  if (verbose)
    fprintf(stderr, "\nInitializing %d x %d map with random values\n", rows, cols);
  dimen = MxGetM(p);
  result = Mrando(dimen, rows * cols, "MAP");
  return (result);
}

int *nborhood(MAP, neuron, nbor, rows, cols, nborsize, nvector, nindex)
MMatrix *MAP;
int neuron, nbor, rows, cols, *nborsize, *nindex;
double **nvector;
{
  int colmin, rowmin, colmax, rowmax, row, col;
  register i, j, k = 0;

  row = neuron % rows;
  col = neuron / rows;

  colmin = maximum(0, col - nbor);
  rowmin = maximum(0, row - nbor);
  colmax = minimum(cols - 1, col + nbor);
  rowmax = minimum(rows - 1, row + nbor);

  /*
   * printf("n = %d  row = %d:  %d - %d   col = %d:  %d - %d  mapcoord %d\n",
   * neuron,row,rowmin, rowmax, col,colmin,colmax,MAPCOORD(row, col, rows));
   */

  *nborsize = (colmax - colmin + 1) * (rowmax - rowmin + 1);
  for (i = rowmin; i <= rowmax; i++)
    for (j = colmin; j <= colmax; j++) {
      /*
       * printf("n = %d  row = %d:  %d - %d   col = %d:  %d - %d  mapcoord
       * %d\n", neuron,i,rowmin, rowmax, j,colmin,colmax,MAPCOORD(i, j,
       * rows));
       */
      nvector[k] = MxGetPr(MAP)[MAPCOORD(i, j, rows)];
      nindex[k] = maximum(abs(j - col), abs(i - row));
      k++;
    }
}

double gaus_alpha(epochs, i, start, end)
int epochs, i;
double start, end;
{
  double x, n;
  x = sqrt(log(end / start) * (-2.0));
  n = (double) i *x / (double) epochs;
  return (start * exp(-n * n * 0.5));
}

double lin_alpha(epochs, i, start, end)
int epochs, i;
double start, end;
{
  double k;
  k = ((double) end - (double) start) / (double) epochs;
  return (k * i + (double) start);
}

double *neighbor_values(nbor, n)
int nbor;
double n;
{
  static double nh[1000];
  register i;
  nh[0] = 1.0;
  for (i = 1; i <= nbor; i++)
    nh[i] = n * nh[i - 1];
  return (nh);
}

int update_neuron(MAP, vec, neuron, nbor, alpha, cr, cc, nh, dimen)
MMatrix *MAP;
int dimen, neuron, nbor, cr, cc;
double *vec, *nh, alpha;
{
  register j, k;
  int nborsize;
  double *pmap, *tmpvec, n, gamma, *p;
  static int nindex[200];
  static double *nvector[200];

  if (nbor == 0) {
    pmap = MxGetPr(MAP)[neuron];
    for (k = 0; k < dimen; k++, pmap++, vec++)
      (*pmap) += alpha * ((*vec) - (*pmap));
  } else {
    nborhood(MAP, neuron, nbor, cr, cc, &nborsize, nvector, nindex);
    for (j = 0; j < nborsize; j++) {
      pmap = nvector[j];
      gamma = alpha * nh[nindex[j]];
      tmpvec = vec;
      for (k = 0; k < dimen; k++, pmap++, tmpvec++)
	(*pmap) += gamma * ((*tmpvec) - (*pmap));
    }
  }
}

MMatrix *teachsom(p, MAP, rows, cols, crows, ccols, epoch, Nbor, ialpha, falpha, hier, verbose, a, ft, at, loop)
MMatrix *p, *MAP;
int epoch, Nbor, rows, cols, crows, ccols, hier, verbose, a, ft, at, loop;
double ialpha, falpha;
{
  register i, j, k;		/* LOOP COUNTERS			 */
  int pvecs,			/* NUMBER OF PATTERN VECTORS		 */
    epoch_count,		/* NEIGHBORHOOD DECREASE POINT		 */
    bestmatch,			/* INDEX OF BESTMATCHING MAP NODE	 */
    too_small_map = 1,		/* MAP SIZE INDICATOR		 */
    nbor,			/* CURRENT NEIGHBORHOOD SIZE		 */
    dimen,			/* DIMENSION OF PATTERN VECTORS		 */
   *nlut;			/* LOOKUP TABLE FOR NEIGHBORHOOD SIZE	 */
  double *pvec,			/* POINTER TO CURRENT PATTERN VECTOR	 */
    dec_nbor,			/* NEIGHBORHOOD DECREASE POINT		 */
   *nh,				/* LOOKUP TABLE FOR NEIGHBORHOOD LEARNING
				 * RATES	 */
    tmpnh,			/* TEMP VARIABLE FOR NEIGHBORHOOD LR	 */
    quanterr = 0.0,		/* QUANTIZATION ERROR			 */
   *alut;			/* LOOKUP TABLE FOR ALPHA VALUES	 */
  MMatrix *TMPMAP;		/* POINTER TO TEMPORARY MAP		 */

  static double (*afun[]) () = {lin_alpha, gaus_alpha};	/* POINTERS TO ALUT
							 * INITIALIZING
							 * FUNCTIONS */


  /* INITIALIZE VARIABLES	 */
  nbor = Nbor;
  pvecs = MxGetN(p);
  dimen = MxGetM(MAP);
  nh = neighbor_values(nbor, 0.7);

  /* FILL LOOKUP TABLE FOR ALPHA VALUES	 */
  alut = (double *) malloc(epoch * sizeof(double));
  if (alut == NULL) {
    fprintf(stderr, "alut NULL pointer\n");
    exit(0);
  }
  for (i = 0; i < epoch; i++)
    alut[i] = afun[a] (epoch, i, ialpha, falpha);

  /* FILL LOOKUP TABLE FOR DECREASING NEIGHBORHOOD SIZE	 */
  nlut = (int *) malloc(epoch * sizeof(int));
  if (nlut == NULL) {
    fprintf(stderr, "nlut NULL pointer\n");
    exit(0);
  }
  dec_nbor = (double) epoch / ((double) Nbor + 1.0);
  epoch_count = dec_nbor;
  for (i = 0; i < epoch; i++) {
    if (nbor > 1) {
      if (i > epoch_count) {
	epoch_count += dec_nbor;
	nbor--;
      }
    }
    nlut[i] = nbor;
  }

  /* VERBOSE OUTPUT	 */
  if (verbose) {
    fprintf(stderr, "Nbor function [%d", (int) nh[0]);
    for (k = 1; k <= Nbor; k++)
      fprintf(stderr, " %f", nh[k]);
    fprintf(stderr, "]\n");
  }
  /* TEACH MAP(S)	 */
  while (too_small_map) {
    quanterr = 0.0;
    for (i = 0; i < epoch; i++) {
      pvec = MxGetPr(p)[(int) (lrand48() % (long) pvecs)];
      bestmatch = Sbmu(MAP, pvec, loop, &quanterr);
      update_neuron(MAP, pvec, bestmatch, nlut[i], alut[i], crows, ccols, nh, dimen);
      if (verbose == 2) {
	fprintf(stderr, "Epoch %d/%d  alpha = %f  nbor = %d  quanterr = %f\r",
		i + 1, epoch, alut[i], nlut[i], quanterr);
	fflush(stderr);
      }
    }
    if (verbose)
      fprintf(stderr, "Map %d x %d done (quanterr %f)\n", crows, ccols, quanterr);

    /* GROW MAP SIZE IF TOO SMALL	 */
    if (hier) {
      TMPMAP = interp_new_map(MAP, &crows, &ccols, rows, cols, verbose);
      if (TMPMAP == NULL)
	too_small_map = 0;
      else
	MAP = TMPMAP;
    } else
      too_small_map = 0;
  }
  /*
   * FINE TUNE MAP BY DECREASING THE EFFECT OF LEARNING RATE IN LARGER (>= 1)
   * NEIGHBORHOODS KEEPING THE NEIGHBORHOOD SIZE CONSTANT
   */
  if (ft) {
    tmpnh = 1.0;
    for (j = 1; j <= 4; j++) {
      nh = neighbor_values(Nbor, tmpnh / 2.0);
      tmpnh = nh[1];
      quanterr = 0.0;
      for (i = 0; i < epoch; i++) {
	pvec = MxGetPr(p)[(int) (lrand48() % (long) pvecs)];
	bestmatch = Sbmu(MAP, pvec, loop, &quanterr);
	update_neuron(MAP, pvec, bestmatch, Nbor, alut[i], crows, ccols, nh, dimen);
      }
      if (verbose) {
	fprintf(stderr, "Map %d x %d fine tuning neighborhoods [%d",
		rows, cols, (int) nh[0]);
	for (k = 1; k <= Nbor; k++)
	  fprintf(stderr, " %f", nh[k]);
	fprintf(stderr, "] done  (quanterr %f)\n", quanterr);
      }
    }
  }
  /*
   * FINE TUNE MAP BY USING SMALL LEARNING RATES WITH CHANGING NEIGHBORHOOD
   * SIZE
   */
  if (at) {
    ialpha = 0.1;
    for (j = 1; j <= 3; j++) {
      for (i = 0; i < epoch; i++)
	alut[i] = afun[a] (epoch, i, ialpha, ialpha / 50.0);
      quanterr = 0.0;
      for (i = 0; i < epoch; i++) {
	pvec = MxGetPr(p)[(int) (lrand48() % (long) pvecs)];
	bestmatch = Sbmu(MAP, pvec, loop, &quanterr);
	update_neuron(MAP, pvec, bestmatch, nlut[i], alut[i], crows, ccols, nh, dimen);
      }
      if (verbose)
	fprintf(stderr, "Map %d x %d alpha tuning [%f...%f] done (quanterr %f)\n",
		rows, cols, ialpha, alut[epoch - 1], quanterr);
      ialpha = alut[epoch - 1];
    }
  }
  free(alut);
  free(nlut);
  return (MAP);
}

/* MAIN PROGRAM FOR PARSING COMMAND LINE OPTIONS */
main(argc, argv)
int argc;
char **argv;
{
  int Nbor = 1,			/* DEFAULT NEIGHBORHOOD SIZE	 */
    rows = 1,			/* FINAL MAP SIZE		 */
    cols = 1,			/* FINAL MAP SIZE		 */
    epoch = 0,			/* NUMBER OF EPOCHS		 */
    hier = 0,			/* HIERACHICAL LEARNING FLAG	 */
    pn = 0,			/* PATTERN FLAG			 */
    tn = 0,			/* TARGET FLAG			 */
    verbose = 1,		/* VERBOSE OUTPUT (1,2,3,4)	 */
    ook = 0,			/* OUTPUT FLAG			 */
    ch,				/* OPTIONS PARSING		 */
    init_fun = 0,		/* DEFAULT INITIALIZE FUNCTION	 */
    ir,				/* INITIAL NUMBER OF ROWS	 */
    ic,				/* INITIAL NUMBER OF COLUMNS	 */
    transpose = 0,		/* TRANSPOSE FLAG		 */
    rand_seed,			/* VALUE OF RANDOM SEED NUMBER	 */
    use_seed = 0,		/* RANDOM SEED FLAG		 */
    alpha_fun = 0,		/* DEFAULT ALPHA FUNCTION	 */
    neighbor_fun = 0,		/* DEFAULT NEIGHBOR FUNCTION	 */
    ft = 0,			/* FINE TUNING FLAG		 */
    at = 0,			/* ALPHA FINE TUNING FLAG	 */
    oldmap = 0,			/* USE OLD MAP FLAG		 */
    loop = 0,			/* NUMBER OF VECTOR ELEMENTS TO	 */
  /* BE USED IN BMU SEARCH	 */
    initialize = 0;		/* SAVE INITIALIZED WEIGHTS AND */
  /* QUIT	:)			 */


  double ialpha = 0.6,		/* INITIAL ALPHA (LEARNING RATE) */
    falpha = 0.0001;		/* FINAL ALPHA			 */

  extern char *optarg;
  extern int optind;

  char pfile[NAME_LEN],		/* FILENAME BUFFERS		 */
    tfile[NAME_LEN], ofile[NAME_LEN], mfile[NAME_LEN];

  MMatrix *p, *t, *MAP, *MAPI;

#ifdef MATLAB			/* USES MATLAB'S OWN FUNCTIONS FOR FILE IO */
  MATFile *fp;
  Matrix *tmpp, *tmpt;
#else				/* IF MATLAB IS NOT AVAILABLE THEN TRY TO */
  FILE *fp;			/* USE OWN FUNCTIONS	 */
#endif

  static MMatrix *(*initf[]) () = {initmap1, initmap2};

  while ((ch = getopt(argc, argv, "p:t:o:R:C:e:n:I:v:r:a:A:N:HTfgd:m:x:i")) != -1) {
    switch (ch) {
    case 'p':
      strcpy(pfile, optarg);
#ifdef MATLAB
      if ((fp = matOpen(check_ext(pfile, MAT), "r")) == NULL)
	usage("Cannot open pattern file");
      if ((tmpp = matGetNextMatrix(fp)) == NULL)
	usage("No pattern matrix");
      matClose(fp);
      p = matlab2own(tmpp);
#else
      if ((fp = fopen(check_ext(pfile, MAT), "r")) == NULL)
	usage("Cannot open pattern file");
      if ((p = MatGetMatrix(fp, NULL)) == NULL)
	usage("No pattern matrix");
      fclose(fp);
#endif
      pn = 1;
      break;
    case 't':
      strcpy(tfile, optarg);
#ifdef MATLAB
      if ((fp = matOpen(check_ext(tfile, MAT), "r")) == NULL)
	usage("Cannot open target file");
      if ((tmpt = matGetNextMatrix(fp)) == NULL)
	usage("No target matrix");
      matClose(fp);
      t = matlab2own(tmpt);
#else
      if ((fp = fopen(check_ext(tfile, MAT), "r")) == NULL)
	usage("Cannot open target file");
      if ((t = MatGetMatrix(fp, NULL)) == NULL)
	usage("No target matrix");
      fclose(fp);
#endif
      tn = 1;
      break;
    case 'm':
      strcpy(mfile, optarg);
#ifdef MATLAB
      if ((fp = matOpen(check_ext(tfile, MAT), "r")) == NULL)
	usage("Cannot open map file");
      if ((tmpt = matGetNextMatrix(fp)) == NULL)
	usage("No matrix in map file");
      matClose(fp);
      MAP = matlab2own(tmpt);
#else
      if ((fp = fopen(check_ext(mfile, MAT), "r")) == NULL)
	usage("Cannot open map file");
      if ((MAP = MatGetMatrix(fp, NULL)) == NULL)
	usage("No matrix in map file");
      fclose(fp);
#endif
      oldmap = 1;
      break;
    case 'R':
      rows = atoi(optarg);
      if (rows < 1)
	usage("Illegal value for parameter R");
      break;
    case 'C':
      cols = atoi(optarg);
      if (cols < 1)
	usage("Illegal value for parameter C");
      break;
    case 'e':
      epoch = atoi(optarg);
      if (epoch < 1)
	usage("Illegal value for parameter e");
      break;
    case 'n':
      Nbor = atoi(optarg);
      if (Nbor < 0)
	usage("Illegal value for parameter n");
      break;
    case 'I':
      init_fun = atoi(optarg);
      if ((init_fun < 0) || (init_fun > 1))
	usage("Illegal value for parameter I");
      break;
    case 'H':
      hier = 1;
      break;
    case 'T':
      transpose = 1;
      break;
    case 'f':
      ft = 1;
      break;
    case 'v':
      verbose = atoi(optarg);
      if ((verbose < 1) || (verbose > 3))
	usage("Illegal value for parameter v");
      break;
    case 'r':
      rand_seed = atoi(optarg);
      use_seed = 1;
      break;
    case 'a':
      ialpha = atof(optarg);
      if (ialpha <= 0.0)
	usage("Illegal value for parameter a");
      break;
    case 'd':
      falpha = atof(optarg);
      if (falpha <= 0.0)
	usage("Illegal value for parameter d");
      break;
    case 'A':
      alpha_fun = atoi(optarg);
      if ((alpha_fun < 0) || (alpha_fun > 1))
	usage("Illegal value for parameter A");
      break;
    case 'g':
      at = 1;
      break;
    case 'o':
      strcpy(ofile, optarg);
      ook = 1;
      break;
    case 'x':
      loop = atoi(optarg);
      if (loop <= 0)
	usage("Illegal value for parameter x");
      break;
    case 'i':
      initialize = 1;
      break;
    }
  }
  if (!pn)
    usage("No pattern file");

  if (!ook)
    usage("No output filename");

  ir = rows;
  ic = cols;

  if (loop == 0)
    loop = MxGetM(p);

  if (hier) {
    if (rows >= 3)
      ir = 3;
    if (cols >= 3)
      ic = 3;
  }
  if (!epoch)
    epoch = MxGetN(p);

  if (transpose)
    p = Mtransp(p, p);

  if (!oldmap) {
    if (use_seed)
      srand48((long) rand_seed);
    else
      srand48((long) getpid());
    MAP = initf[init_fun] (ir, ic, p, verbose);
  } else {
    if (hier)
      usage("Cannot use hierachical learning with previously teached map!");
    if (MxGetN(MAP) != (rows * cols))
      usage("Wrong number of weight vectors in map! Should be rows x cols.");
  }

  if (!initialize) {
    MAP = teachsom(p, MAP, rows, cols, ir, ic, epoch, Nbor, ialpha, falpha, hier, verbose, alpha_fun, ft, at, loop);

    if (tn) {
      MAPI = paintmap(MAP, p, t, loop, verbose);
      MxFreeMatrix(t);
      MxFree(t);
    }
  }
#ifdef MATLAB
  if ((fp = matOpen(check_ext(ofile, MAT), "w")) == NULL) {
    fprintf(stderr, "Cannot open output file!");
    exit(1);
  }
  if (matPutMatrix(fp, own2matlab(MAP)) == 1) {
    fprintf(stderr, "Cannot write map into file %s!", ofile);
    matClose(fp);
    exit(1);
  }
  if (!initialize)
    if (tn) {
      if (matPutMatrix(fp, own2matlab(MAPI)) == 1) {
	fprintf(stderr, "Cannot write map indexes into file %s!", ofile);
	matClose(fp);
	exit(1);
      }
    }
  matClose(fp);
#else
  if ((fp = fopen(check_ext(ofile, MAT), "w")) == NULL) {
    fprintf(stderr, "Cannot open output file!");
    exit(1);
  }
  if (MatPutMatrix(fp, MAP) == 1) {
    fprintf(stderr, "Note! Cannot save map weights!\n");
    exit(1);
  }
  if (!initialize)
    if (tn) {
      if (MatPutMatrix(fp, MAPI) == 1) {
	fprintf(stderr, "Cannot write map labels into file %s!", ofile);
	fclose(fp);
	exit(1);
      }
    }
  fclose(fp);
#endif
  if (verbose)
    fprintf(stderr, "Map saved into file %s\n", ofile);

  printf("\n");

  MxFreeMatrix(p);
  MxFree(p);

  exit(1);
}
