/***********************************************************************
*                                                                      *
*                              SPECLASS                                *
*                                                                      *
*                                The                                   *
*                                                                      *
*                      Color Spectra Classifier                        *
*                                                                      *
*                          Program  Package                            *
*                                                                      *
*                           Programmed by			       *
*								       *
*			     J. Haanpalo		               *
*                                                                      *
*                      Copyright (c) 1993-1995                         *
*                                                                      *
************************************************************************
*                                                                      *
*  NOTE: This program package is copyrighted in the sense that it      *
*  may be used for scientific purposes. The package as a whole, or     *
*  parts thereof, cannot be included or used in any commercial         *
*  application without written permission granted by its producents.   *
*  No programs contained in this package may be copied for commercial  *
*  distribution. This program package comes 'as it is' and with no     *
*  warranty.                                                           *
*                                                                      *
*  All comments concerning this program package may be sent to the     *
*  e-mail address 'haanpalo@lut.fi'.                                   *
*                                                                      *
***********************************************************************/

#include <stdio.h>
#include <strings.h>
#include <math.h>
#include "xglobal.h"
#include "global.h"
#include "matfile.h"

#define index(mat, a, b) (*(mxGetPr(mat)+(a)*mxGetM(mat)+(b)))
#define max(a, b)  ((a) > (b) ? (a) : (b))
#define min(a, b)  ((a) < (b) ? (a) : (b))


Panel_item slog_item, sselect1_item, sselect2_item, sseed_item, srow_item,
  scol_item, soutput_map_item, sinput_map_item, snbors_item, siters_item,
  strials_item, stpattern_item, sgamma_item, sdecay_item, sge_item, stest_item,
  slearn_item, sttarget_item;

int nborhood();
MMatrix *read_map();
void exec_somlearn(), test_som(), generate_map();

void init_som_panel()
{
  if (som_panel) {
    xv_set(current_panel, XV_SHOW, FALSE, NULL);
    xv_set(som_panel, XV_SHOW, TRUE, NULL);
    current_panel = som_panel;
    return;
  }
  current_panel = som_panel = xv_create(frame, PANEL,
					XV_X, PANEL_X,
					XV_Y, PANEL_Y,
					XV_WIDTH, PANEL_WIDTH,
					XV_HEIGHT, PANEL_HEIGHT,
					NULL);


  siters_item = xv_create(som_panel, PANEL_TEXT,
			  XV_X, 0,
			  XV_Y, 0,
			  PANEL_LAYOUT, PANEL_VERTICAL,
			  PANEL_LABEL_STRING, "epochs",
			  PANEL_VALUE_DISPLAY_LENGTH, DISPLAY_LENGTH,
			  PANEL_VALUE_STORED_LENGTH, DISPLAY_LENGTH,
			  PANEL_VALUE, "1000",
			  NULL);

  srow_item = xv_create(som_panel, PANEL_TEXT,
			XV_X, 100,
			XV_Y, 0,
			PANEL_LAYOUT, PANEL_VERTICAL,
			PANEL_LABEL_STRING, "rows",
			PANEL_VALUE_DISPLAY_LENGTH, DISPLAY_LENGTH,
			PANEL_VALUE_STORED_LENGTH, DISPLAY_LENGTH,
			PANEL_VALUE, "10",
			NULL);

  scol_item = xv_create(som_panel, PANEL_TEXT,
			XV_X, 200,
			XV_Y, 0,
			PANEL_LAYOUT, PANEL_VERTICAL,
			PANEL_LABEL_STRING, "cols",
			PANEL_VALUE_DISPLAY_LENGTH, DISPLAY_LENGTH,
			PANEL_VALUE_STORED_LENGTH, DISPLAY_LENGTH,
			PANEL_VALUE, "10",
			NULL);

  snbors_item = xv_create(som_panel, PANEL_TEXT,
			  XV_X, 300,
			  XV_Y, 0,
			  PANEL_LAYOUT, PANEL_VERTICAL,
			  PANEL_LABEL_STRING, "nbors",
			  PANEL_VALUE_DISPLAY_LENGTH, DISPLAY_LENGTH,
			  PANEL_VALUE_STORED_LENGTH, DISPLAY_LENGTH,
			  PANEL_VALUE, "1",
			  NULL);

  slearn_item = xv_create(som_panel, PANEL_BUTTON,
			  XV_X, 0,
			  XV_Y, 70,
			  PANEL_LABEL_STRING, "learn",
			  PANEL_NOTIFY_PROC, exec_somlearn,
			  NULL);

  stest_item = xv_create(som_panel, PANEL_BUTTON,
			 XV_X, 0,
			 XV_Y, 100,
			 PANEL_LABEL_STRING, " test ",
			 PANEL_NOTIFY_PROC, test_som,
			 NULL);

  sselect1_item = xv_create(som_panel, PANEL_CHECK_BOX,
			    XV_X, 60,
			    XV_Y, 68,
			    PANEL_LAYOUT, PANEL_VERTICAL,
			    PANEL_CHOOSE_ONE, FALSE,
		     PANEL_CHOICE_STRINGS, "verbose", "hier training", NULL,
			    PANEL_VALUE, 3,
			    NULL);

  sselect2_item = xv_create(som_panel, PANEL_CHECK_BOX,
			    XV_X, 175,
			    XV_Y, 70,
			    PANEL_LAYOUT, PANEL_VERTICAL,
			    PANEL_CHOOSE_ONE, FALSE,
			    PANEL_CHOICE_STRINGS, "gamma tuning", NULL,
			    PANEL_VALUE, 0,
			    NULL);

  sinput_map_item = xv_create(som_panel, PANEL_TEXT,
			      XV_X, 2,
			      XV_Y, 120,
			      PANEL_LAYOUT, PANEL_VERTICAL,
			      PANEL_LABEL_STRING, "Input map file:",
			      PANEL_VALUE_DISPLAY_LENGTH, 20,
			      PANEL_VALUE, "",
			      NULL);

  soutput_map_item = xv_create(som_panel, PANEL_TEXT,
			       XV_X, 2,
			       XV_Y, 155,
			       PANEL_LAYOUT, PANEL_VERTICAL,
			       PANEL_LABEL_STRING, "Output map file:",
			       PANEL_VALUE_DISPLAY_LENGTH, 20,
			       PANEL_VALUE, "",
			       NULL);

  slog_item = xv_create(som_panel, PANEL_TEXT,
			XV_X, 2,
			XV_Y, 190,
			PANEL_LAYOUT, PANEL_VERTICAL,
			PANEL_LABEL_STRING, "Logfile:",
			PANEL_VALUE_DISPLAY_LENGTH, 20,
			PANEL_VALUE, "",
			NULL);

  stpattern_item = xv_create(som_panel, PANEL_TEXT,
			     XV_X, 175,
			     XV_Y, 155,
			     PANEL_LAYOUT, PANEL_VERTICAL,
			     PANEL_LABEL_STRING, "Test pattern file:",
			     PANEL_VALUE_DISPLAY_LENGTH, 20,
			     PANEL_VALUE, "",
			     NULL);

  sttarget_item = xv_create(som_panel, PANEL_TEXT,
			    XV_X, 175,
			    XV_Y, 190,
			    PANEL_LAYOUT, PANEL_VERTICAL,
			    PANEL_LABEL_STRING, "Test target file:",
			    PANEL_VALUE_DISPLAY_LENGTH, 20,
			    PANEL_VALUE, "",
			    NULL);
}

void exec_somlearn(item, event)
Panel_item item;
Event *event;
{
  double igamma, fgamma, decay, seed;
  int rows, cols, nbors, iters, trials;
  unsigned int match;
  char cmd_buf[500], pfile[NAME_LEN], imapfile[NAME_LEN], omapfile[NAME_LEN], logfile[NAME_LEN],
    tfile[NAME_LEN], tmp_buf[50];

  /* not checked!!! */
  rows = atoi((char *) xv_get(srow_item, PANEL_VALUE));
  cols = atoi((char *) xv_get(scol_item, PANEL_VALUE));
  nbors = atoi((char *) xv_get(snbors_item, PANEL_VALUE));
  iters = atoi((char *) xv_get(siters_item, PANEL_VALUE));

  /* These parameters are not needed?				 */
  /* trials = atoi((char*)xv_get(strials_item,PANEL_VALUE));	 */
  /* igamma = atof((char*)xv_get(sgamma_item,PANEL_VALUE));	 */
  /* fgamma = atof((char*)xv_get(sge_item,PANEL_VALUE));	 */
  /* decay = atof((char*)xv_get(sdecay_item,PANEL_VALUE));	 */
  /* seed = atof((char*)xv_get(sseed_item,PANEL_VALUE));	 */

  /* check pattern file name (required)	 */
  strcpy(pfile, (char *) xv_get(pattern_item, PANEL_VALUE));
  if (strcmp(pfile, "\0") == 0) {
    show_notice(current_panel, item, event, "No pattern file name");
    return;
  }
  /* check target file name (required)	 */
  strcpy(tfile, (char *) xv_get(target_item, PANEL_VALUE));
  if (strcmp(tfile, "\0") == 0) {
    show_notice(current_panel, item, event, "No target file name");
    return;
  }
  /* check output file name (required)	 */
  strcpy(omapfile, (char *) xv_get(soutput_map_item, PANEL_VALUE));
  if (strcmp(omapfile, "\0") == 0) {
    show_notice(current_panel, item, event, "No output file name");
    return;
  }
  /* default command for maplearn	 */
  sprintf(cmd_buf, "%ssom -p %s -t %s -o %s -e %d -R %d -C %d -n %d",
	  BINDIR, pfile, tfile, omapfile, iters, rows, cols, nbors);

  /* check input map file name (optional)	 */
  strcpy(imapfile, (char *) xv_get(sinput_map_item, PANEL_VALUE));
  if (strcmp(imapfile, "\0") != 0) {
    sprintf(tmp_buf, " -m %s", imapfile);
    strcat(cmd_buf, tmp_buf);
  }
  match = xv_get(sselect1_item, PANEL_VALUE);
  if (match & 1)
    strcat(cmd_buf, " -v1");	/* verbose output	 */
  if (match & 2)
    strcat(cmd_buf, " -H");	/* hierarchical training */

  match = xv_get(sselect2_item, PANEL_VALUE);
  if (match & 1)
    strcat(cmd_buf, " -g");	/* gamma fine tuning	 */

  printf("%s\n", cmd_buf);

  show_time(RUSAGE_CHILDREN);

  system(cmd_buf);

  printf("User time used in learning %d s\n", show_time(RUSAGE_CHILDREN));

}

void test_som(item, event)
Panel_item item;
Event *event;
{
  MMatrix *tp, *tt, *W, *MAP, *MAPI;
  int i, rows = 0, cols = 0, correct, class, neuron, **conf = NULL, Nclasses;
  char ifile[NAME_LEN], pfile[NAME_LEN], tfile[NAME_LEN];

  /* check pattern file name (required)	 */
  strcpy(pfile, (char *) xv_get(stpattern_item, PANEL_VALUE));
  if (strcmp(pfile, "\0") == 0) {
    show_notice(current_panel, item, event, "No test pattern file name");
    return;
  }
  /* check target file name (required)	 */
  strcpy(tfile, (char *) xv_get(sttarget_item, PANEL_VALUE));
  if (strcmp(tfile, "\0") == 0) {
    show_notice(current_panel, item, event, "No test target file name");
    return;
  }
  /* check input map file name (required)	 */
  strcpy(ifile, (char *) xv_get(sinput_map_item, PANEL_VALUE));
  if (strcmp(ifile, "\0") == 0) {
    show_notice(current_panel, item, event, "No input file name");
    return;
  }
  /* LOAD TEST PATTERN AND TARGET FILES   */
  if ((tp = read_pattern_matrix(pfile, 2)) == NULL) {
    show_notice(current_panel, item, event, "Check test pattern file");
    return;
  }
  if ((tt = read_target_matrix(tfile)) == NULL) {
    show_notice(current_panel, item, event, "Check test target file");
    return;
  }
  if ((MAP = load_matrix(ifile, "MAP")) == NULL) {
    show_notice(current_panel, item, event, "Check map file");
    return;
  }
  if ((MAPI = load_matrix(ifile, "MAPI")) == NULL) {
    show_notice(current_panel, item, event, "Map labels not found");
    return;
  }
  Nclasses = MxGetM(tt);

  conf = init_confusion_matrix(Nclasses);

  show_time(RUSAGE_SELF);

  /* CLASSIFY PATTERN VECTORS */
  for (i = 0; i < MxGetN(tp); i++) {
    correct = Mcompet(tt, i);
    neuron = Mbmu(MAP, MxGetPr(tp)[i], MxGetPr(MAPI)[0]);
    class = MxGetPr(MAPI)[0][neuron];
    mark_class(i, class, correct, 2);
    update_confusion_matrix(conf, correct, class, Nclasses);
  }

  printf("User time used in classification %d s\n", show_time(RUSAGE_SELF));

  /* PRINT CONFUSION MATRIX       */
  xprint_confusion_matrix(conf, Nclasses, "SOM");

  MxFreeMatrix(tp);
  MxFree(tp);
  MxFreeMatrix(tt);
  MxFree(tt);
  MxFreeMatrix(MAP);
  MxFree(MAP);
  MxFreeMatrix(MAPI);
  MxFree(MAPI);
}
