/***********************************************************************
*                                                                      *
*                              SPECLASS                                *
*                                                                      *
*                                The                                   *
*                                                                      *
*                      Color Spectra Classifier                        *
*                                                                      *
*                          Program  Package                            *
*                                                                      *
*                           Programmed by			       *
*								       *
*			     J. Haanpalo		               *
*                                                                      *
*                      Copyright (c) 1993-1995                         *
*                                                                      *
************************************************************************
*                                                                      *
*  NOTE: This program package is copyrighted in the sense that it      *
*  may be used for scientific purposes. The package as a whole, or     *
*  parts thereof, cannot be included or used in any commercial         *
*  application without written permission granted by its producents.   *
*  No programs contained in this package may be copied for commercial  *
*  distribution. This program package comes 'as it is' and with no     *
*  warranty.                                                           *
*                                                                      *
*  All comments concerning this program package may be sent to the     *
*  e-mail address 'haanpalo@lut.fi'.                                   *
*                                                                      *
***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <math.h>
#include "xglobal.h"
#include "global.h"
#include "matfile.h"

short icon_bits[] = {
#include "Icons/spec_icon"
};

/* EXTERNAL FUNCTIONS */
extern void init_canvas(), draw_empty_spec();
extern Menu_item create_menu();
extern Menu create_submenu();
extern int destroy_list();
extern void init_editor_panel(),
  init_help(),
  init_art_panel(),
  init_bp_panel(),
  init_som_panel(),
  init_lvq_panel(),
  init_lsm_panel(),
  init_knn_panel();

/* OTHER USEFUL FUNCTIONS */
Xv_opaque quit_proc();

void change_panel(),
  inc_class(),
  dec_class(),
  toggle_up(),
  toggle_down();

void init_main_panel();

void clear_proc(),
  clear_canvas(),
  canvas_event_proc(),
  read_spectrum(),
  save_files(),
  clear_set(),
  view_class(),
  spec_editor(),
  help_window();

void main(argc, argv)
int argc;
char *argv[];
{
  int i;
  char ver[30];
  art_panel = bp_panel = som_panel = lvq_panel = lsm_panel = knn_panel = NULL;

  start = lstart = NULL;
  visible = highlight = list1_total = list2_total = 0;

  xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);

  /* SET UP ICON */
  spec_icon = (Server_image) xv_create(NULL, SERVER_IMAGE,
				       XV_WIDTH, 64,
				       XV_HEIGHT, 64,
				       SERVER_IMAGE_BITS, icon_bits, NULL);

  icon = (Icon) xv_create(XV_NULL, ICON,
			  XV_VISUAL_CLASS, StaticColor,
			  ICON_IMAGE, spec_icon,
			  NULL);

  /* SET UP FRAME */
  strcpy(ver, "Speclass v. ");
  strcat(ver, VERSION);
  frame = xv_create(NULL, FRAME,
		    FRAME_SHOW_FOOTER, TRUE,
		    FRAME_ICON, icon,
		    XV_LABEL, ver,
		    NULL);

  /* SET UP EDITOR FRAME */
  editor_frame = xv_create(frame, FRAME,
			   XV_LABEL, "Spectrum editor",
			   NULL);

  xv_set(editor_frame, XV_SHOW, FALSE, NULL);
  xv_set(editor_frame, FRAME_SHOW_RESIZE_CORNER, FALSE, NULL);

  /* SETUP THE SIZE OF BASE FRAME */
  rect.r_top = rect.r_left = 20;
  rect.r_width = FRAME_W;
  rect.r_height = FRAME_H;
  frame_set_rect(frame, &rect);

  /* SETUP THE SIZE OF EDITOR FRAME */
  rect.r_top = rect.r_left = 20;
  rect.r_width = EDFRAME_W;
  rect.r_height = EDFRAME_H;
  frame_set_rect(editor_frame, &rect);

  /* CREATE TEXTSW AND SET THE COLORMAP SEGMENT FOR IT */
  textsw = xv_create(frame, TEXTSW,
		     XV_X, 0,
		     XV_Y, 550,
		     WIN_ROWS, 20,
		     NULL);

  /* INIT PANELS AND CANVAS, FIT THE WINDOWS TO EACH OTHER */
  init_main_panel();
  init_art_panel();
  init_canvas();
  init_editor_panel();
  init_help();

  window_fit(canvas);
  window_fit(textsw);
  window_fit(helpsw1);

  xv_main_loop(frame);
}

/* MAIN PANEL INITIALIZATION	*/
void init_main_panel()
{
  main_panel = xv_create(frame, PANEL,
			 XV_X, 0,
			 XV_Y, 0,
			 XV_WIDTH, 630,
			 XV_HEIGHT, 250,
			 NULL);

  method_item = xv_create(main_panel, PANEL_CHOICE_STACK,
			  XV_X, 190,
			  XV_Y, 4,
			  PANEL_CHOOSE_ONE, TRUE,
			  PANEL_LAYOUT, PANEL_VERTICAL,
			  PANEL_LABEL_STRING, "CLASSIFIER",
			  PANEL_CHOICE_STRINGS,
#ifdef ALSM
			 " ALSM ",
#endif
#ifdef ART
			 " ART ",
#endif
#ifdef BP
			 " BP ",
#endif
#ifdef SOM
			 " SOM ",
#endif
#ifdef LVQ
			 " LVQ ",
#endif
#ifdef LSM
			 " LSM ",
#endif
#ifdef KNN
			 " KNN ",
#endif
#ifdef UPA
			 " UPA ",
#endif
			 NULL,
			  PANEL_VALUE, 0,
			  PANEL_NOTIFY_PROC, change_panel,
			  PANEL_CLIENT_DATA, frame,
			  NULL);

  editor_item = xv_create(main_panel, PANEL_BUTTON,
			  XV_X, 95,
			  XV_Y, 4,
			  PANEL_LABEL_STRING, "   Editor   ",
			  PANEL_NOTIFY_PROC, spec_editor,
			  NULL);

  help_item = xv_create(main_panel, PANEL_BUTTON,
			  XV_X, 190,
			  XV_Y, 110,
			  PANEL_LABEL_STRING, "Help",
			  PANEL_NOTIFY_PROC, help_window,
			  NULL);

  quit_item = xv_create(main_panel, PANEL_BUTTON,
			XV_X, 4,
			XV_Y, 4,
			PANEL_LABEL_STRING, "   QUIT    ",
			PANEL_NOTIFY_PROC, quit_proc,
			PANEL_CLIENT_DATA, frame,
			NULL);

  (void) xv_create(main_panel, PANEL_MESSAGE,
		   XV_X, 4,
		   XV_Y, 80,
		   PANEL_LABEL_STRING, "Toggle class",
		   PANEL_LABEL_BOLD, TRUE,
		   NULL);

  dec_item = xv_create(main_panel, PANEL_BUTTON,
		       XV_X, 4,
		       XV_Y, 110,
		       PANEL_LABEL_STRING, "Dec",
		       PANEL_NOTIFY_PROC, dec_class,
		       PANEL_CLIENT_DATA, frame,
		       NULL);

  inc_item = xv_create(main_panel, PANEL_BUTTON,
		       XV_X, 110,
		       XV_Y, 110,
		       PANEL_LABEL_STRING, "Inc",
		       PANEL_NOTIFY_PROC, inc_class,
		       PANEL_CLIENT_DATA, frame,
		       NULL);

  class_item = xv_create(main_panel, PANEL_TEXT,
			 XV_X, 58,
			 XV_Y, 110,
			 PANEL_VALUE_DISPLAY_LENGTH, 4,
			 PANEL_READ_ONLY, TRUE,
			 PANEL_VALUE, "0",
			 NULL);

  (void) xv_create(main_panel, PANEL_MESSAGE,
		   XV_X, 4,
		   XV_Y, 175,
		   PANEL_LABEL_STRING, "Toggle spectrum",
		   PANEL_LABEL_BOLD, TRUE,
		   NULL);

  down_item = xv_create(main_panel, PANEL_BUTTON,
			XV_X, 4,
			XV_Y, 200,
			PANEL_LABEL_STRING, " << ",
			PANEL_NOTIFY_PROC, toggle_down,
			PANEL_CLIENT_DATA, frame,
			NULL);

  up_item = xv_create(main_panel, PANEL_BUTTON,
		      XV_X, 65,
		      XV_Y, 200,
		      PANEL_LABEL_STRING, " >> ",
		      PANEL_NOTIFY_PROC, toggle_up,
		      PANEL_CLIENT_DATA, frame,
		      NULL);

  max_class_item = xv_create(main_panel, PANEL_TEXT,
			     XV_X, 4,
			     XV_Y, 40,
			     PANEL_LABEL_STRING, "Max class",
			     PANEL_VALUE_DISPLAY_LENGTH, 4,
			     PANEL_VALUE, "10",
			     NULL);

  view_class_item = xv_create(main_panel, PANEL_CHOICE,
			      XV_X, 130,
			      XV_Y, 60,
		   PANEL_CHOICE_STRINGS, "  View all  ", "View class", NULL,
			      PANEL_VALUE, 0,
			      PANEL_NOTIFY_PROC, view_class,
			      PANEL_CLIENT_DATA, frame,
			      NULL);

  pattern_item = xv_create(main_panel, PANEL_TEXT,
			   XV_X, 130,
			   XV_Y, 155,
			   PANEL_LAYOUT, PANEL_VERTICAL,
			   PANEL_LABEL_STRING, "Pattern file:",
			   PANEL_VALUE_DISPLAY_LENGTH, 20,
			   PANEL_VALUE, "",
			   NULL);

  target_item = xv_create(main_panel, PANEL_TEXT,
			  XV_X, 130,
			  XV_Y, 190,
			  PANEL_LAYOUT, PANEL_VERTICAL,
			  PANEL_LABEL_STRING, "Target file:",
			  PANEL_VALUE_DISPLAY_LENGTH, 20,
			  PANEL_VALUE, "",
			  NULL);

}

/* SPEC_EDITOR(ITEM,EVENT)						*/
/* SHOW FILE OPTIONS SUBFRAME						*/
/* PARAMETERS:								*/
/*		ITEM	ITEM (BUTTON) WHICH CAUSES ALL THIS		*/
/*		EVENT							*/
void spec_editor(item, event)
Panel_item item;
Event *event;
{
  xv_set(editor_frame, XV_SHOW, TRUE, NULL);
}

/* HELP_WINDOW(ITEM,EVENT)						*/
/* SHOW HELP SUBFRAME							*/
/* PARAMETERS:								*/
/*		ITEM	ITEM (BUTTON) WHICH CAUSES ALL THIS		*/
/*		EVENT							*/
void help_window(item, event)
Panel_item item;
Event *event;
{
  xv_set(help_frame, XV_SHOW, TRUE, NULL);
}

/* CHANGE_PANEL()						*/
/* CHANGES CLASSIFICATION METHOD				*/
void change_panel()
{
  Panel_item item;

  switch (xv_get(method_item, PANEL_VALUE)) {	/* YES, I KNOW IT'S UGLY :) */
#ifdef ALSM
  case 0:
    init_alsm_panel();
  break;
#undef ALSM
#else
#ifdef ART
  case 0:
    init_art_panel();
  break;
#undef ART
#else
#ifdef BP
  case 0:
    init_bp_panel();
  break;
#undef BP
#else
#ifdef SOM
   case 0:
   init_som_panel();
  break;
#undef SOM
#else
#ifdef LVQ
  case 0:
    init_lvq_panel();
  break;
#undef LVQ
#else
#ifdef LSM
   case 0:
   init_lsm_panel();
  break;
#undef LSM
#else
#ifdef KNN
   case 0:
   init_knn_panel();
  break;
#undef KNN
#else
#ifdef UPA
    init_upa_panel();
  break;
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#ifdef ART
  case 1:
    init_art_panel();
    break;
#undef ART
#else
#ifdef BP
  case 1:
    init_bp_panel();
    break;
#undef BP
#else
#ifdef SOM
  case 1:
    init_som_panel();
    break;
#undef SOM
#else
#ifdef LVQ
   case 1:
   init_lvq_panel();
    break;
#undef LVQ
#else
#ifdef LSM
  case 1:
    init_lsm_panel();
    break;
#undef LSM
#else
#ifdef KNN
   case 1:
   init_knn_panel();
    break;
#undef KNN
#else
#ifdef UPA
  case 1:
    init_upa_panel();
    break;
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#ifdef BP
  case 2:
    init_bp_panel();
    break;
#undef BP
#else
#ifdef SOM
  case 2:
    init_som_panel();
    break;
#undef SOM
#else
#ifdef LVQ
  case 2:
    init_lvq_panel();
    break;
#undef LVQ
#else
#ifdef LSM
  case 2:
    init_lsm_panel();
    break;
#undef LSM
#else
#ifdef KNN
  case 2:
    init_knn_panel();
    break;
#undef KNN
#else
#ifdef UPA
  case 2:
    init_upa_panel();
    break;
#endif
#endif
#endif
#endif
#endif
#endif
#ifdef SOM
  case 3:
    init_som_panel();
    break;
#undef SOM
#else
#ifdef LVQ
  case 3:
    init_lvq_panel();
    break;
#undef LVQ
#else
#ifdef LSM
  case 3:
    init_lsm_panel();
    break;
#undef LSM
#else
#ifdef KNN
  case 3:
    init_knn_panel();
    break;
#undef KNN
#else
#ifdef UPA
  case 3:
    init_upa_panel();
    break;
#endif
#endif
#endif
#endif
#endif
#ifdef LVQ
  case 4:
    init_lvq_panel();
    break;
#undef LVQ
#else
#ifdef LSM
  case 4:
    init_lsm_panel();
    break;
#undef LSM
#else
#ifdef KNN
  case 4:
    init_knn_panel();
    break;
#undef KNN
#else
#ifdef UPA
  case 4:
    init_upa_panel();
    break;
#endif
#endif
#endif
#endif
#ifdef LSM
  case 5:
    init_lsm_panel();
    break;
#undef LSM
#else
#ifdef KNN
  case 5:
    init_knn_panel();
    break;
#undef KNN
#else
#ifdef UPA
  case 5:
    init_upa_panel();
    break;
#endif
#endif
#endif
#ifdef KNN
  case 6:
    init_knn_panel();
    break;
#undef KNN
#else
#ifdef UPA
  case 6:
    init_upa_panel();
    break;
#endif
#endif
#ifdef UPA
  case 7:
    init_upa_panel();
    break;
#endif
  }
}

/* SHOW_NOTICE(PANEL,ITEM,EVENT,MESSU)					*/
/* WARNS THE USER WHEN SILLY THINGS HAS BEEN TRIED TO DO		*/
/* PARAMETERS:								*/
/*		PANEL	CURRENT PANEL					*/
/*		ITEM	ITEM (BUTTON) WHICH CAUSES ALL THIS		*/
/*		EVENT							*/
/*		MESSU	POINTER TO A STRING THAT IS GOING TO BE		*/
/*			DISPLAYED					*/
Xv_opaque
show_notice(panel, item, event, messu)
Panel panel;
Panel_item item;
Event *event;
char *messu;
{
  int result;
  result = notice_prompt(panel, NULL,
			 NOTICE_FOCUS_XY, event_x(event), event_y(event),
			 NOTICE_MESSAGE_STRINGS, messu, NULL,
			 NOTICE_BUTTON_YES, "Confirm", NULL);
}

/* INC_CLASS(ITEM,EVENT)						*/
/* INCREASE CLASS NUMBER IN MAIN PANEL					*/
/* PARAMETERS:								*/
/*		ITEM	ITEM (BUTTON) WHICH CAUSES ALL THIS		*/
/*		EVENT							*/
void inc_class(item, event)
Panel_item item;
Event *event;
{
  Frame fr;
  Panel_item tmp_item, maxi_item;
  int value, view_mode, cno, set;
  char messu[4];

  fr = (Panel) xv_get(item, PANEL_CLIENT_DATA);

  if (fr == frame) {
    tmp_item = class_item;
    maxi_item = max_class_item;
    view_mode = (int) xv_get(view_class_item, PANEL_VALUE);
    cno = atoi((char *) xv_get(class_item, PANEL_VALUE));
    set = 2;
  } else {
    tmp_item = edclass_item;
    maxi_item = edmax_class_item;
    view_mode = (int) xv_get(edview_class_item, PANEL_VALUE);
    cno = atoi((char *) xv_get(edclass_item, PANEL_VALUE));
    set = 1;
  }

  value = atoi((char *) xv_get(tmp_item, PANEL_VALUE));

  if (value == atoi((char *) xv_get(maxi_item, PANEL_VALUE)) - 1) {
    show_notice(xv_get(item, PANEL_CLIENT_DATA), item, event, "Cannot increase class number");
    return;
  }
  /* DRAW SPECTRA FROM CORRECT LIST */
  if (view_mode == 1) {
    /* CLEAR CANVAS AND DRAW COORDINATE AXIS */
    draw_empty_spec();
    if (set == 1)
      draw_class(start, cno + 1);
    else
      draw_class(lstart, cno + 1);
    highlight = -1;
  }
  sprintf(messu, "%d", value + 1);
  xv_set(tmp_item, PANEL_VALUE, messu, NULL);
}

/* DEC_CLASS(ITEM,EVENT)						*/
/* DECREASE CLASS NUMBER IN MAIN PANEL					*/
/* PARAMETERS:								*/
/*		ITEM	ITEM (BUTTON) WHICH CAUSES ALL THIS		*/
/*		EVENT							*/
void dec_class(item, event)
Panel_item item;
Event *event;
{
  Frame fr;
  Panel_item tmp_item;
  int value, view_mode, cno, set;
  char messu[4];

  fr = (Panel) xv_get(item, PANEL_CLIENT_DATA);

  if (fr == frame) {
    tmp_item = class_item;
    view_mode = (int) xv_get(view_class_item, PANEL_VALUE);
    cno = atoi((char *) xv_get(class_item, PANEL_VALUE));
    set = 2;
  } else {
    tmp_item = edclass_item;
    view_mode = (int) xv_get(edview_class_item, PANEL_VALUE);
    cno = atoi((char *) xv_get(edclass_item, PANEL_VALUE));
    set = 1;
  }

  value = atoi((char *) xv_get(tmp_item, PANEL_VALUE));

  if (value == 0) {
    show_notice(fr, item, event, "Cannot decrease class number");
    return;
  }
  /* DRAW SPECTRA FROM CORRECT LIST */
  if (view_mode == 1) {
    /* CLEAR CANVAS AND DRAW COORDINATE AXIS */
    draw_empty_spec();
    if (set == 1)
      draw_class(start, cno - 1);
    else
      draw_class(lstart, cno - 1);
    highlight = -1;
  }
  sprintf(messu, "%d", value - 1);
  xv_set(tmp_item, PANEL_VALUE, messu, NULL);
}

/* TOGGLE_DOWN(ITEM,EVENT)						*/
/* HIGHLIGHT PREVIOUS SPECTRUM IN LINKED LIST				*/
/* PARAMETERS:								*/
/*		ITEM	ITEM (BUTTON) WHICH CAUSES ALL THIS		*/
/*		EVENT							*/
void toggle_down(item, event)
Panel_item item;
Event *event;
{
  Frame fr;
  int view_mode, ind = -1, cno, count = 0, list_total;
  char messu[4];
  Spec *list, *next;

  fr = (Panel) xv_get(item, PANEL_CLIENT_DATA);

  if (fr == frame) {
    list = lstart;
    list_total = list2_total;
    view_mode = (int) xv_get(view_class_item, PANEL_VALUE);
    cno = atoi((char *) xv_get(class_item, PANEL_VALUE));
  } else {
    list = start;
    list_total = list1_total;
    view_mode = (int) xv_get(edview_class_item, PANEL_VALUE);
    cno = atoi((char *) xv_get(edclass_item, PANEL_VALUE));
  }

  if (list == NULL) {
    show_notice(fr, item, event, "Empty linked list");
    return;
  }
  if (highlight == 0) {
    show_notice(fr, item, event, "First spectrum in linked list");
    return;
  }
  if (view_mode == 0) {
    draw_spek(get_spec(list, highlight), 1);
    highlight--;
    draw_spek(get_spec(list, highlight), 3);
  } else {
    next = list;
    while (next) {
      if (next->class == cno) {
	if (ind == -1)
	  ind = count;
	if (highlight == -1) {	/* first */
	  highlight = ind;
	  draw_spek(get_spec(list, highlight), 3);
	  return;
	}
	if (count == highlight) {
	  draw_spek(get_spec(list, highlight), 1);
	  highlight = ind;
	  draw_spek(get_spec(list, highlight), 3);
	  return;
	}
	ind = count;
      }
      next = next->next;
      count++;
    }
  }
}

/* TOGGLE_UP(ITEM,EVENT)						*/
/* HIGHLIGHT NEXT SPECTRUM IN LINKED LIST				*/
/* PARAMETERS:								*/
/*		ITEM	ITEM (BUTTON) WHICH CAUSES ALL THIS		*/
/*		EVENT							*/
void toggle_up(item, event)
Panel_item item;
Event *event;
{
  Frame fr;
  int view_mode, ind = -1, count = 0, cno, light, list_total;
  char messu[4];
  Spec *list, *next;

  fr = (Panel) xv_get(item, PANEL_CLIENT_DATA);

  if (fr == frame) {
    list = lstart;
    list_total = list2_total;
    view_mode = (int) xv_get(view_class_item, PANEL_VALUE);
    cno = atoi((char *) xv_get(class_item, PANEL_VALUE));
  } else {
    list = start;
    list_total = list1_total;
    view_mode = (int) xv_get(edview_class_item, PANEL_VALUE);
    cno = atoi((char *) xv_get(edclass_item, PANEL_VALUE));
  }

  if (list == NULL) {
    show_notice(fr, item, event, "Empty linked list");
    return;
  }
  if (highlight == list_total) {
    show_notice(fr, item, event, "Last spectrum in linked list");
    return;
  }
  if (view_mode == 0) {
    if (highlight > 0)
      draw_spek(get_spec(list, highlight - 1), 1);
    draw_spek(get_spec(list, highlight), 3);
    if (highlight < list_total - 1)
      highlight++;
  } else {
    next = list;
    while (next) {
      if (next->class == cno) {
	if (ind == -1)
	  ind = count;
	if (highlight == -1) {	/* ensimmainen */
	  highlight = ind;
	  draw_spek(get_spec(list, highlight), 3);
	  return;
	}
	if (count > highlight) {
	  draw_spek(get_spec(list, highlight), 1);
	  highlight = count;
	  draw_spek(get_spec(list, highlight), 3);
	  return;
	}
      }
      next = next->next;
      count++;
    }
  }
}

/* VIEW_CLASS(ITEM,EVENT)						*/
/* VIEW ALL SPECTRA CLASSES TOGETHER OR ONE CLASS AT A TIME		*/
/* PARAMETERS:								*/
/*		ITEM	ITEM (BUTTON) WHICH CAUSES ALL THIS		*/
/*		EVENT							*/
void view_class(item, event)
Panel_item item;
Event *event;
{
  Frame fr;
  Spec *this, *next;
  int view_mode, set, cno;

  fr = (Panel) xv_get(item, PANEL_CLIENT_DATA);

  if (fr == frame) {
    view_mode = (int) xv_get(view_class_item, PANEL_VALUE);
    set = 2;
    cno = atoi((char *) xv_get(class_item, PANEL_VALUE));
  } else {
    view_mode = (int) xv_get(edview_class_item, PANEL_VALUE);
    set = 1;
    cno = atoi((char *) xv_get(edclass_item, PANEL_VALUE));
  }

  /* CLEAR CANVAS AND DRAW COORDINATE AXIS */
  draw_empty_spec();

  /* DRAW SPECTRA FROM CORRECT LIST */
  if (view_mode == 0) {
    if (set == 1)
      draw_list(start);
    else
      draw_list(lstart);
    highlight = 0;
  } else {
    if (set == 1)
      draw_class(start, cno);
    else
      draw_class(lstart, cno);
    highlight = -1;
  }
}

/* QUIT(ITEM)								*/
/* QUITS PROGRAM							*/
/* PARAMETERS:								*/
/*		ITEM	ITEM (BUTTON) WHICH CAUSES ALL THIS		*/
Xv_opaque quit_proc(item, event)
Panel_item item;
Event *event;
{
  int result;
  result = notice_prompt(main_panel, NULL,
			 NOTICE_FOCUS_XY, event_x(event), event_y(event),
			 NOTICE_MESSAGE_STRINGS, "Really quitting?", NULL,
			 NOTICE_BUTTON_YES, "Yes",
			 NOTICE_BUTTON_NO, "No",
			 NULL);

  if (result == NOTICE_YES) {
    xv_destroy_safe(editor_frame);
    xv_destroy_safe(frame);
  }
}
